@@ -42,8 +42,10 @@
 //
 //////////////////////////////////////////////////////////////////////
 
-
-#define STORAGE(type, variable) type  variable[TAU_MAX_THREADS] 
+#define STORAGE(type, variable) type variable[TAU_MAX_THREADS]
+#ifdef TAU_MULTIPLE_COUNTERS
+#define MULTSTORAGE(type, variable) type variable[TAU_MAX_THREADS][MAX_TAU_COUNTERS]
+#endif//TAU_MULTIPLE_COUNTERS
 
 class FunctionInfo
 {
@@ -74,13 +76,19 @@ public:
 	void FunctionInfoInit(TauGroup_t PGroup, const char *PGroupName, 
 	  bool InitData, int tid );
         
-
+#ifndef TAU_MULTIPLE_COUNTERS 
 	// Tell it about a function call finishing.
 	inline void ExcludeTime(double t, int tid);
 	// Removing void IncludeTime(double t, int tid);
  	// and replacing with 
         inline void AddInclTime(double t, int tid);
 	inline void AddExclTime(double t, int tid);
+#else//TAU_MULTIPLE_COUNTERS
+	inline void ExcludeTime(double *t, int tid);
+	inline void AddInclTime(double *t, int tid);
+	inline void AddExclTime(double *t, int tid);
+#endif//TAU_MULTIPLE_COUNTERS
+
 	inline void IncrNumCalls(int tid);
         inline void IncrNumSubrs(int tid);
 	inline bool GetAlreadyOnStack(int tid);
@@ -92,7 +100,7 @@ public:
 	//static TAU_STD_NAMESPACE vector<FunctionInfo*> FunctionDB[TAU_MAX_THREADS];
 
 #ifdef PROFILE_CALLS
-	list < pair<double,double> > *ExclInclCallList; 
+	std::list < std::pair<double,double> > *ExclInclCallList; 
 	// Make this a ptr to a list so that ~FunctionInfo doesn't destroy it.
 	// time spent in each call
 
@@ -109,14 +117,17 @@ public:
 #endif  // PROFILE_CALLSTACK
 
 private:
-
 	// A record of the information unique to this function.
 	// Statistics about calling this function.
-
 	STORAGE(long, NumCalls);
 	STORAGE(long, NumSubrs);
+#ifndef TAU_MULTIPLE_COUNTERS
 	STORAGE(double, ExclTime);
 	STORAGE(double, InclTime);
+#else //TAU_MULTIPLE_COUNTERS
+	MULTSTORAGE(double, ExclTime);
+	MULTSTORAGE(double, InclTime);
+#endif//TAU_MULTIPLE_COUNTERS
 	STORAGE(bool, AlreadyOnStack);
 #ifdef PROFILE_STATS
 	STORAGE(double, SumExclSqr);
@@ -126,23 +137,66 @@ public:
 	string Name;
 	string Type;
 	string GroupName;
+	string AllGroups;
 	long   FunctionId;
 	// Cough up the information about this function.
+	void SetName(string& str) { Name = str; }
 	const char* GetName() const { return Name.c_str(); }
+	void SetType(string& str) { Type = str; }
 	const char* GetType() const { return Type.c_str(); }
 	const char* GetPrimaryGroup() const { return GroupName.c_str(); }
-	void SetPrimaryGroupName(const char *newname) { GroupName = newname; }
-	void SetPrimaryGroupName(string newname) { GroupName = newname; }
+	const char* GetAllGroups() const { return AllGroups.c_str(); }
+	void SetPrimaryGroupName(const char *newname) 
+	{ 
+	  GroupName = newname;
+    	  AllGroups = newname; /* to make it to the profile */
+	}
+	void SetPrimaryGroupName(string newname) 
+        { 
+	  GroupName = newname; 
+	  AllGroups = newname; /* to make it to the profile */
+	}
+
 	long GetFunctionId() ;
 	long GetCalls(int tid) { return NumCalls[tid]; }
 	void SetCalls(int tid, long calls) { NumCalls[tid] = calls; }
 	long GetSubrs(int tid) { return NumSubrs[tid]; }
 	void SetSubrs(int tid, long subrs) { NumSubrs[tid] = subrs; }
+
+#ifndef TAU_MULTIPLE_COUNTERS
 	double GetExclTime(int tid) { return ExclTime[tid]; }
 	void SetExclTime(int tid, double excltime) { ExclTime[tid] = excltime; }
 	double GetInclTime(int tid) { return InclTime[tid]; }
 	void SetInclTime(int tid, double incltime) { InclTime[tid] = incltime; }
+#else//TAU_MULTIPLE_COUNTERS
+	//Returns the array of exclusive counter values.
+	//double * GetExclTime(int tid) { return ExclTime[tid]; }
+       	double * GetExclTime(int tid);
+	void SetExclTime(int tid, double *excltime) {
+	  for(int i=0;i<MAX_TAU_COUNTERS;i++)
+	    ExclTime[tid][i] = excltime[i];
+	}
+	void SetExclTimeZero(int tid) {
+          for(int i=0;i<MAX_TAU_COUNTERS;i++)
+            InclTime[tid][i] = 0;
+        }
+
+
+	//Returns the array of inclusive counter values.
+	//double * GetInclTime(int tid) { return InclTime[tid]; }
+	double * GetInclTime(int tid);
+	void SetInclTime(int tid, double *incltime) { 
+	  for(int i=0;i<MAX_TAU_COUNTERS;i++)
+	    InclTime[tid][i] = incltime[i];
+	}
+	void SetInclTimeZero(int tid) {
+          for(int i=0;i<MAX_TAU_COUNTERS;i++)
+            InclTime[tid][i] = 0;
+        }
+#endif//TAU_MULTIPLE_COUNTERS
+
 	TauGroup_t GetProfileGroup() const {return MyProfileGroup_; }
+	void SetProfileGroup(TauGroup_t gr) {MyProfileGroup_ = gr; }
 #ifdef PROFILE_STATS 
 	double GetSumExclSqr(int tid) { return SumExclSqr[tid]; }
 	void SetSumExclSqr(int tid, double s) { SumExclSqr[tid] = s; }
@@ -161,6 +215,7 @@ int& TheSafeToDumpData(void);
 //
 // For efficiency, make the timing updates inline.
 //
+#ifndef TAU_MULTIPLE_COUNTERS
 inline void 
 FunctionInfo::ExcludeTime(double t, int tid)
 { // called by a function to decrease its parent functions time
@@ -179,6 +234,30 @@ FunctionInfo::AddExclTime(double t, int tid)
 {
  	ExclTime[tid] += t; // Add Total Time to Exclusive time (-ve)
 }
+#else //TAU_MULTIPLE_COUNTERS
+inline void 
+FunctionInfo::ExcludeTime(double *t, int tid)
+{ // called by a function to decrease its parent functions time
+  // exclude from it the time spent in child function
+  for(int i=0;i<MAX_TAU_COUNTERS;i++)
+    ExclTime[tid][i] -= t[i];
+}
+	
+
+inline void 
+FunctionInfo::AddInclTime(double *t, int tid)
+{
+  for(int i=0;i<MAX_TAU_COUNTERS;i++)
+    InclTime[tid][i] += t[i]; // Add Inclusive time
+}
+
+inline void
+FunctionInfo::AddExclTime(double *t, int tid)
+{
+  for(int i=0;i<MAX_TAU_COUNTERS;i++)
+    ExclTime[tid][i] += t[i]; // Add Total Time to Exclusive time (-ve)
+}
+#endif//TAU_MULTIPLE_COUNTERS
 
 inline void
 FunctionInfo::IncrNumCalls(int tid)
@@ -208,6 +287,6 @@ FunctionInfo::GetAlreadyOnStack(int tid)
 #endif /* _FUNCTIONINFO_H_ */
 /***************************************************************************
  * $RCSfile: FunctionInfo.h,v $   $Author: sameer $
- * $Revision: 1.17 $   $Date: 2002/01/24 23:27:47 $
- * POOMA_VERSION_ID: $Id: FunctionInfo.h,v 1.17 2002/01/24 23:27:47 sameer Exp $ 
+ * $Revision: 1.26 $   $Date: 2002/11/28 00:48:40 $
+ * POOMA_VERSION_ID: $Id: FunctionInfo.h,v 1.26 2002/11/28 00:48:40 sameer Exp $ 
  ***************************************************************************/
