@@ -24,7 +24,7 @@ extern "C" {
 #endif /* TAU_PAPI */
 
 
-#define SIZE_OF_INIT_ARRAY 8 //!!Change this if functions are added to the system!!
+#define SIZE_OF_INIT_ARRAY 11 //!!Change this if functions are added to the system!!
 
 
 //Some useful typedefs
@@ -58,9 +58,18 @@ class MultipleCounterLayer
   static bool sgiTimersMCLInit(int functionPosition);
   static void sgiTimersMCL(int tid, double values[]);
 
+  static bool crayTimersMCLInit(int functionPosition);
+  static void crayTimersMCL(int tid, double values[]);
+
   static bool cpuTimeMCLInit(int functionPosition);
   static void cpuTimeMCL(int tid, double values[]);
 
+  static bool tauMUSEMCLInit(int functionPosition);
+  static void tauMUSEMCL(int tid, double values[]);
+  
+  static bool tauMPIMessageSizeMCLInit(int functionPosition);
+  static void tauMPIMessageSizeMCL(int tid, double values[]);
+  
   static bool papiMCLInit(int functionPosition);
   static void papiMCL(int tid, double values[]);
 
@@ -92,10 +101,25 @@ class MultipleCounterLayer
   static int sgiTimersMCL_FP;
 #endif // SGI_TIMERS
 
+#ifdef CRAY_TIMERS
+  static int crayTimersMCL_CP[1];
+  static int crayTimersMCL_FP;
+#endif // CRAY_TIMERS
+
 #ifdef CPU_TIME
   static int cpuTimeMCL_CP[1];
   static int cpuTimeMCL_FP;
 #endif // CPU_TIME
+
+#ifdef TAU_MUSE
+  static int tauMUSEMCL_CP[1];
+  static int tauMUSEMCL_FP;
+#endif // TAU_MUSE
+
+#ifdef TAU_MPI
+  static int tauMPIMessageSizeMCL_CP[1];
+  static int tauMPIMessageSizeMCL_FP;
+#endif // TAU_MPI
   
 #ifdef TAU_PAPI
   static int papiMCL_CP[MAX_TAU_COUNTERS];
@@ -128,6 +152,7 @@ class MultipleCounterLayer
   static int numberOfActiveFunctions;
   static char * names[MAX_TAU_COUNTERS];
   static bool counterUsed[MAX_TAU_COUNTERS];
+  static int numberOfCounters[MAX_TAU_COUNTERS];
 };
 
 #endif /* TAU_MULTIPLE_COUNTERS */
