@@ -62,7 +62,6 @@
 
 #include <Profile/RtsLayer.h>
 
-
 #include <Profile/FunctionInfo.h>
 			  
 #include <Profile/UserEvent.h>
@@ -75,6 +74,12 @@
 
 #include <Profile/WindowsThreadLayer.h>
 
+#include <Profile/TauMuse.h>
+
+#include <Profile/TauCompensate.h>
+
+#include <Profile/TauHandler.h>
+
 //////////////////////////////////////////////////////////////////////
 //
 // class Profiler
@@ -146,12 +151,22 @@ public:
 	double StartTime;
 #else //TAU_MULTIPLE_COUNTERS
 	double StartTime[MAX_TAU_COUNTERS];
+	double CurrentTime[MAX_TAU_COUNTERS];
+	double TotalTime[MAX_TAU_COUNTERS];
 #endif//TAU_MULTIPLE_COUNTERS
 	FunctionInfo * ThisFunction;
 #ifdef TAU_CALLPATH
 	FunctionInfo * CallPathFunction;
 	bool 	       AddInclCallPathFlag; 
 #endif // TAU_CALLPATH
+#ifdef TAU_COMPENSATE
+	/* Compensate for instrumentation overhead based on total number of 
+	child calls executed under the given timer */
+	long NumChildren;
+	void SetNumChildren(long n);
+	long GetNumChildren(void);
+	void AddNumChildren(long value);
+#endif /* TAU_COMPENSATE */
 	Profiler * ParentProfiler; 
 
 
@@ -183,6 +198,6 @@ private:
 #endif /* PROFILER_H */
 /***************************************************************************
  * $RCSfile: Profiler.h,v $   $Author: sameer $
- * $Revision: 1.46 $   $Date: 2003/07/18 18:49:49 $
- * POOMA_VERSION_ID: $Id: Profiler.h,v 1.46 2003/07/18 18:49:49 sameer Exp $ 
+ * $Revision: 1.50 $   $Date: 2004/02/26 22:28:35 $
+ * POOMA_VERSION_ID: $Id: Profiler.h,v 1.50 2004/02/26 22:28:35 sameer Exp $ 
  ***************************************************************************/
