@@ -38,6 +38,8 @@
 //////////////////////////////////////////////////////////////////////
 extern "C" void Tau_start_timer(void * function_info);
 extern "C" void Tau_stop_timer(void * function_info); 
+extern "C" void Tau_create_top_level_timer_if_necessary(void);
+extern "C" void Tau_stop_top_level_timer_if_necessary(void);
 
 #define TAU_TYPE_STRING(profileString, str) static string profileString(str);
 #define TAU_PROFILE(name, type, group) \
@@ -125,12 +127,21 @@ extern "C" void Tau_stop_timer(void * function_info);
 #define TAU_ENABLE_ALL_GROUPS()			RtsLayer::enableAllGroups()
 #define TAU_DISABLE_ALL_GROUPS()		RtsLayer::disableAllGroups()
 #define TAU_GET_PROFILE_GROUP(group)		RtsLayer::getProfileGroup(group)
+#define TAU_ENABLE_TRACKING_MEMORY()		TauEnableTrackingMemory()
+#define TAU_DISABLE_TRACKING_MEMORY()		TauDisableTrackingMemory()
+#define TAU_TRACK_MEMORY()			TauTrackMemoryUtilization()
+#define TAU_TRACK_MEMORY_HERE()			TauTrackMemoryHere()
+#define TAU_ENABLE_TRACKING_MUSE_EVENTS()	TauEnableTrackingMuseEvents()
+#define TAU_DISABLE_TRACKING_MUSE_EVENTS()	TauDisableTrackingMuseEvents()
+#define TAU_TRACK_MUSE_EVENTS()			TauTrackMuseEvents()
+#define TAU_SET_INTERRUPT_INTERVAL(value)	TauSetInterruptInterval(value)
 
 #ifdef NO_RTTI
 /* #define CT(obj) string(#obj) */
-#define CT(obj) " "
+#define CT(obj) string(" ")
 #else // RTTI is present
-#define CT(obj) string(RtsLayer::CheckNotNull(typeid(obj).name())) 
+//#define CT(obj) string(RtsLayer::CheckNotNull(typeid(obj).name())) 
+#define CT(obj) RtsLayer::GetRTTI(typeid(obj).name())
 #endif //NO_RTTI
 
 #else /* PROFILING_ON */
@@ -185,6 +196,13 @@ extern "C" void Tau_stop_timer(void * function_info);
 #define TAU_DISABLE_GROUP_NAME(group)
 #define TAU_ENABLE_ALL_GROUPS()			
 #define TAU_DISABLE_ALL_GROUPS()	
+#define TAU_ENABLE_TRACKING_MEMORY()
+#define TAU_DISABLE_TRACKING_MEMORY()
+#define TAU_TRACK_MEMORY()
+#define TAU_ENABLE_TRACKING_MUSE_EVENTS()	
+#define TAU_DISABLE_TRACKING_MUSE_EVENTS()
+#define TAU_TRACK_MUSE_EVENTS()		
+#define TAU_SET_INTERRUPT_INTERVAL(value)
 
 #define CT(obj)
 
@@ -211,6 +229,6 @@ extern "C" void Tau_stop_timer(void * function_info);
 #endif /* _TAU_API_H_ */
 /***************************************************************************
  * $RCSfile: TauAPI.h,v $   $Author: sameer $
- * $Revision: 1.26 $   $Date: 2003/07/18 18:49:50 $
- * POOMA_VERSION_ID: $Id: TauAPI.h,v 1.26 2003/07/18 18:49:50 sameer Exp $ 
+ * $Revision: 1.32 $   $Date: 2004/06/10 18:08:30 $
+ * POOMA_VERSION_ID: $Id: TauAPI.h,v 1.32 2004/06/10 18:08:30 sameer Exp $ 
  ***************************************************************************/
