@@ -113,7 +113,33 @@
 #define TAU_DISABLE_ALL_GROUPS()            	Tau_disable_all_groups() 
 #define TAU_DISABLE_GROUP_NAME(group)           Tau_disable_group_name(group)
 #define TAU_GET_PROFILE_GROUP(group)            Tau_get_profile_group(group)
+#define TAU_BCAST_DATA(data)  	Tau_bcast_data(data)
+#define TAU_REDUCE_DATA(data)  	Tau_reduce_data(data)
+#define TAU_ALLTOALL_DATA(data) Tau_alltoall_data(data) 
+#define TAU_SCATTER_DATA(data)  Tau_scatter_data(data) 
+#define TAU_GATHER_DATA(data)  	Tau_gather_data(data)
+#define TAU_ALLREDUCE_DATA(data)  	Tau_allreduce_data(data)
+#define TAU_ALLGATHER_DATA(data)  	Tau_allgather_data(data)
+#define TAU_REDUCESCATTER_DATA(data)  	Tau_reducescatter_data(data)
+#define TAU_SCAN_DATA(data)  		Tau_scan_data(data)
+#define TAU_ENABLE_TRACKING_MEMORY()	Tau_enable_tracking_memory()
+#define TAU_DISABLE_TRACKING_MEMORY()	Tau_disable_tracking_memory()
+#define TAU_TRACK_MEMORY()		Tau_track_memory()
+#define TAU_TRACK_MEMORY_HERE()		Tau_track_memory_here()
+#define TAU_ENABLE_TRACKING_MUSE_EVENTS()	Tau_enable_tracking_muse_events()	
+#define TAU_DISABLE_TRACKING_MUSE_EVENTS()	Tau_disable_tracking_muse_events()
+#define TAU_TRACK_MUSE_EVENTS()			Tau_track_muse_events()		
+#define TAU_SET_INTERRUPT_INTERVAL(value)	Tau_set_interrupt_interval(value)
 
+extern void Tau_bcast_data(int data);
+extern void Tau_reduce_data(int data);
+extern void Tau_alltoall_data(int data);
+extern void Tau_scatter_data(int data);
+extern void Tau_gather_data(int data);
+extern void Tau_allreduce_data(int data);
+extern void Tau_allgather_data(int data);
+extern void Tau_reducescatter_data(int data);
+extern void Tau_scan_data(int data);
 
 extern void * Tau_get_profiler(char *fname, char *type, TauGroup_t  group, char *gr_name);
 extern void Tau_start_timer(void *profiler);
@@ -154,6 +180,16 @@ extern TauGroup_t Tau_enable_group_name(char *group);
 extern TauGroup_t Tau_disable_group_name(char *group);
 extern TauGroup_t Tau_get_profile_group(char *group);
 extern void Tau_profile_c_timer(void **ptr, char *fname, char *type, TauGroup_t group, char *group_name);
+extern void Tau_create_top_level_timer_if_necessary(void);
+extern void Tau_stop_top_level_timer_if_necessary(void);
+extern void Tau_track_memory(void);
+extern void Tau_track_muse_events(void);
+extern void Tau_enable_tracking_memory(void);
+extern void Tau_disable_tracking_memory(void);
+extern void Tau_enable_tracking_muse_events(void);
+extern void Tau_disable_tracking_muse_events(void);
+extern void Tau_set_interrupt_interval(int value);
+
 
 
 
@@ -205,6 +241,20 @@ extern void Tau_profile_c_timer(void **ptr, char *fname, char *type, TauGroup_t
 #define TAU_ENABLE_GROUP_NAME(group)            
 #define TAU_DISABLE_GROUP_NAME(group)          
 #define TAU_GET_PROFILE_GROUP(group) 
+#define TAU_BCAST_DATA(data)  	
+#define TAU_REDUCE_DATA(data)  
+#define TAU_ALLREDUCE_DATA(data)  
+#define TAU_ALLTOALL_DATA(data)
+#define TAU_SCATTER_DATA(data)
+#define TAU_GATHER_DATA(data)
+#define TAU_ALLGATHER_DATA(data)
+#define TAU_ENABLE_TRACKING_MEMORY()
+#define TAU_DISABLE_TRACKING_MEMORY()
+#define TAU_TRACK_MEMORY()
+#define TAU_ENABLE_TRACKING_MUSE_EVENTS()	
+#define TAU_DISABLE_TRACKING_MUSE_EVENTS()
+#define TAU_TRACK_MUSE_EVENTS()		
+#define TAU_SET_INTERRUPT_INTERVAL(value)
 
 #define CT(obj)
 
@@ -229,7 +279,7 @@ extern void Tau_profile_c_timer(void **ptr, char *fname, char *type, TauGroup_t
 
 /***************************************************************************
  * $RCSfile: TauCAPI.h,v $   $Author: sameer $
- * $Revision: 1.24 $   $Date: 2003/07/18 18:49:51 $
- * POOMA_VERSION_ID: $Id: TauCAPI.h,v 1.24 2003/07/18 18:49:51 sameer Exp $
+ * $Revision: 1.30 $   $Date: 2004/06/10 18:08:29 $
+ * POOMA_VERSION_ID: $Id: TauCAPI.h,v 1.30 2004/06/10 18:08:29 sameer Exp $
  ***************************************************************************/
 
