@@ -58,8 +58,12 @@ using namespace std;
 #endif //TAU_WINDOWS
 
 #ifdef TRACING_ON
+#ifdef TAU_EPILOG
+#include "elg_trc.h"
+#else /* TAU_EPILOG */
 #define PCXX_EVENT_SRC
 #include "Profile/pcxx_events.h"
+#endif /* TAU_EPILOG */
 #endif // TRACING_ON 
 
 
@@ -87,6 +91,18 @@ int& TheSafeToDumpData()
   return SafeToDumpData;
 }
 
+#ifdef TAU_EPILOG 
+//////////////////////////////////////////////////////////////////////
+// Initialize EPILOG Tracing package
+//////////////////////////////////////////////////////////////////////
+int TauInitEpilog(void)
+{
+  DEBUGPROFMSG("Calling elg_open"<<endl;);
+  elg_open();
+  return 1;
+}
+#endif /* TAU_EPILOG */
+
 //////////////////////////////////////////////////////////////////////
 // Member Function Definitions For class FunctionInfo
 //////////////////////////////////////////////////////////////////////
@@ -97,6 +113,9 @@ int& TheSafeToDumpData()
 void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup, 
 	const char *ProfileGroupName, bool InitData, int tid)
 {
+  //Need to keep track of all the groups this function is a member of.
+  AllGroups = ProfileGroupName;
+
 #ifdef TRACING_ON
 	GroupName = RtsLayer::PrimaryGroup(ProfileGroupName);
 #endif //TRACING_ON
@@ -105,6 +124,10 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup,
 // we know that it couldn't be already on the call stack.
 	RtsLayer::LockDB();
 // Use LockDB to avoid a possible race condition.
+
+	//Add function name to the name list.
+	Profiler::theFunctionList(NULL, NULL, true, (const char *)GetName());
+
         if (InitData) 
         {
 	  SetAlreadyOnStack(false, tid);
@@ -119,8 +142,15 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup,
 	    SetAlreadyOnStack(false, i);
 #endif /* JAVA  */
      	    NumSubrs[i] = 0;
-       	    ExclTime[i] = 0;
+#ifndef TAU_MULTIPLE_COUNTERS
+	    ExclTime[i] = 0;
        	    InclTime[i] = 0;
+#else //TAU_MULTIPLE_COUNTERS
+	    for(int j=0;j<MAX_TAU_COUNTERS;j++){
+	      ExclTime[i][j] = 0;
+	      InclTime[i][j] = 0;
+	    } 
+#endif//TAU_MULTIPLE_COUNTERS
  	  }
 	}
 
@@ -139,12 +169,21 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup,
 	// Important in the presence of concurrent threads.
 	TheFunctionDB().push_back(this);
 #ifdef TRACING_ON
+#ifdef TAU_EPILOG
+        static int tau_elg_init=TauInitEpilog();
+	string tau_elg_name(Name+" "+Type);
+	FunctionId = elg_def_region(tau_elg_name.c_str(), ELG_NO_ID, ELG_NO_LNO,
+		ELG_NO_LNO, GroupName.c_str(), ELG_FUNCTION);
+	DEBUGPROFMSG("elg_def_region: "<<tau_elg_name<<": returns "<<FunctionId<<endl;);
+#else /* TAU_EPILOG */
 	// FOR Tracing, we should make the two a single operation 
 	// when threads are supported for traces. There needs to be 
 	// a lock in RtsLayer that can be locked while the push_back
 	// and size operations are done (this should be atomic). 
 	// Function Id is the index into the DB vector
 	FunctionId = TheFunctionDB().size();
+	SetFlushEvents(tid);
+#endif /* TAU_EPILOG */
 #endif //TRACING_ON
 	RtsLayer::UnLockDB();
 		
@@ -219,6 +258,24 @@ FunctionInfo::~FunctionInfo()
   TheSafeToDumpData() = 0;
 }
 
+#ifdef TAU_MULTIPLE_COUNTERS
+double * FunctionInfo::GetExclTime(int tid){
+  double * tmpCharPtr = (double *) malloc( sizeof(double) * MAX_TAU_COUNTERS);
+  for(int i=0;i<MAX_TAU_COUNTERS;i++){
+    tmpCharPtr[i] = ExclTime[tid][i];
+  }
+  return tmpCharPtr;
+}
+
+double * FunctionInfo::GetInclTime(int tid){
+  double * tmpCharPtr = (double *) malloc( sizeof(double) * MAX_TAU_COUNTERS);
+  for(int i=0;i<MAX_TAU_COUNTERS;i++){
+    tmpCharPtr[i] = InclTime[tid][i];
+  }
+  return tmpCharPtr;
+}
+#endif //TAU_MULTIPLE_COUNTERS
+
 #ifdef PROFILE_CALLS
 //////////////////////////////////////////////////////////////////////
 
@@ -251,6 +308,6 @@ long FunctionInfo::GetFunctionId(void)
 
 /***************************************************************************
  * $RCSfile: FunctionInfo.cpp,v $   $Author: sameer $
- * $Revision: 1.26 $   $Date: 2002/01/15 04:21:08 $
- * POOMA_VERSION_ID: $Id: FunctionInfo.cpp,v 1.26 2002/01/15 04:21:08 sameer Exp $ 
+ * $Revision: 1.32 $   $Date: 2003/05/20 18:44:28 $
+ * POOMA_VERSION_ID: $Id: FunctionInfo.cpp,v 1.32 2003/05/20 18:44:28 sameer Exp $ 
  ***************************************************************************/
