@@ -23,6 +23,15 @@ using namespace std;
 #include <unistd.h>
 #endif // CPU_TIME
 
+#ifdef CRAY_TIMERS
+#include <intrinsics.h>
+#include <sys/param.h>
+#endif // CRAY_TIMERS
+
+#ifdef TAU_MPI
+extern "C" TauUserEvent sendevent;
+#endif /* TAU_MPI */
+
 //Initialize static members.
 char MultipleCounterLayer::environment[25][10] = {
   {"COUNTER1"},{"COUNTER2"},{"COUNTER3"},{"COUNTER4"},{"COUNTER5"},
@@ -38,6 +47,12 @@ int MultipleCounterLayer::linuxTimerMCL_CP[1];
 int MultipleCounterLayer::linuxTimerMCL_FP;
 #endif //TAU_LINUX_TIMERS
 
+#ifdef CRAY_TIMERS
+int MultipleCounterLayer::crayTimersMCL_CP[1];
+int MultipleCounterLayer::crayTimersMCL_FP;
+#endif // CRAY_TIMERS
+
+
 #ifdef SGI_TIMERS
 int MultipleCounterLayer::sgiTimersMCL_CP[1];
 int MultipleCounterLayer::sgiTimersMCL_FP;
@@ -48,6 +63,16 @@ int MultipleCounterLayer::cpuTimeMCL_CP[1];
 int MultipleCounterLayer::cpuTimeMCL_FP;
 #endif // CPU_TIME
 
+#ifdef TAU_MUSE
+int MultipleCounterLayer::tauMUSEMCL_CP[1];
+int MultipleCounterLayer::tauMUSEMCL_FP;
+#endif /* TAU_MUSE */
+
+#ifdef TAU_MPI
+int MultipleCounterLayer::tauMPIMessageSizeMCL_CP[1];
+int MultipleCounterLayer::tauMPIMessageSizeMCL_FP;
+#endif /* TAU_MPI */
+
 #ifdef TAU_PAPI
 int MultipleCounterLayer::papiMCL_CP[MAX_TAU_COUNTERS];
 int MultipleCounterLayer::papiWallClockMCL_CP[1];
@@ -75,6 +100,9 @@ firstListType MultipleCounterLayer::initArray[] = {gettimeofdayMCLInit,
 						   linuxTimerMCLInit,
 						   sgiTimersMCLInit,
 						   cpuTimeMCLInit,
+						   crayTimersMCLInit,
+						   tauMUSEMCLInit,
+						   tauMPIMessageSizeMCLInit,
 						   papiMCLInit,
 						   papiWallClockMCLInit,
 						   papiVirtualMCLInit,
@@ -83,6 +111,7 @@ firstListType MultipleCounterLayer::initArray[] = {gettimeofdayMCLInit,
 int MultipleCounterLayer::numberOfActiveFunctions = 0;
 secondListType MultipleCounterLayer::functionArray[] = { };
 char * MultipleCounterLayer::names[] = { };
+int MultipleCounterLayer::numberOfCounters[] = { };
 bool MultipleCounterLayer::counterUsed[] = { };
 
 bool MultipleCounterLayer::initializeMultiCounterLayer(void)
@@ -101,6 +130,7 @@ bool MultipleCounterLayer::initializeMultiCounterLayer(void)
       functionArray[a] = NULL;
       MultipleCounterLayer::names[a] = NULL;
       MultipleCounterLayer::counterUsed[a] = false; //Don't use setter function as we are already in RtsLayer::LockDB();
+      MultipleCounterLayer::numberOfCounters[a] = 0;
 #ifdef TAU_PAPI 
       MultipleCounterLayer::papiMCL_CP[a] = -1;
       MultipleCounterLayer::PAPI_CounterCodeList[a] = -1;
@@ -132,6 +162,22 @@ bool MultipleCounterLayer::initializeMultiCounterLayer(void)
     MultipleCounterLayer::cpuTimeMCL_FP = -1;
 #endif // CPU_TIME
 
+#ifdef TAU_MUSE
+    MultipleCounterLayer::tauMUSEMCL_CP[0] = -1;
+    MultipleCounterLayer::tauMUSEMCL_FP = -1;
+#endif /* TAU_MUSE */
+
+#ifdef TAU_MPI
+    MultipleCounterLayer::tauMPIMessageSizeMCL_CP[0] = -1;
+    MultipleCounterLayer::tauMPIMessageSizeMCL_FP = -1;
+#endif /* TAU_MPI */
+
+#ifdef CRAY_TIMERS
+    MultipleCounterLayer::crayTimersMCL_CP[0] = -1;
+    MultipleCounterLayer::crayTimersMCL_FP = -1;
+#endif // CRAY_TIMERS
+
+
 #ifdef TAU_PAPI
     MultipleCounterLayer::papiWallClockMCL_CP[0] = -1;
     MultipleCounterLayer::papiVirtualMCL_CP[0] = -1;
@@ -175,7 +221,7 @@ bool MultipleCounterLayer::initializeMultiCounterLayer(void)
     //Give a warning of not.  It should not break the system,
     //but it is nice to give a warning.
     if(numberOfActiveFunctions == 0)
-      cout << "Warning: No functions active ... no profiles will be created!" << endl;
+      cout << "Warning: No multi counter fncts active ... are the env variables COUNTER<1-N> set?" << endl;
   }
   RtsLayer::UnLockDB(); // mutual exclusion primitive
    
@@ -300,145 +346,150 @@ void MultipleCounterLayer::theCounterListInternal(const char ***inPtr,
   *tmpPtr = tmpCounterUsedList;
 }
 
-bool MultipleCounterLayer::gettimeofdayMCLInit(int functionPosition)
-{
-  //This function uses the Unix gettimeofday function.
-  //Checks for GET_TIME_OF_DAY
-
-  for(int i=0; i<MAX_TAU_COUNTERS; i++)
-    {
+bool MultipleCounterLayer::gettimeofdayMCLInit(int functionPosition){
+  for(int i=0; i<MAX_TAU_COUNTERS; i++){
       if(MultipleCounterLayer::names[i] != NULL){
 	if(strcmp(MultipleCounterLayer::names[i], "GET_TIME_OF_DAY") == 0){
-	
-	  //cout << "gettimeofdayMCL is active." << endl;
-
-	  //Set the counter position.
 	  gettimeofdayMCL_CP[0] = i;
-
-	  //Indicate that this function is being used.
 	  MultipleCounterLayer::counterUsed[i] = true;
-
-	  //Update the functionArray.
-	  //cout << "Inserting gettimeofdayMCL in position: " 
-	  //   << functionPosition << endl;
+	  MultipleCounterLayer::numberOfCounters[i] = 1;
 	  MultipleCounterLayer::functionArray[functionPosition] = gettimeofdayMCL;
 	  gettimeofdayMCL_FP = functionPosition;
-	  //Now just return with beginCountersPosition incremented.
 	  return true;
 	}
       }
     }
-
-  //If we are here, then this function is not active.
-  //cout << "gettimeofdayMCL is not active." << endl;
   return false;
 }
 
 bool MultipleCounterLayer::sgiTimersMCLInit(int functionPosition){
 #ifdef SGI_TIMERS
-
-  //This function uses SGI fast nanosecs timer.
-  //Checks for SGI_TIMERS
-
   for(int i=0; i<MAX_TAU_COUNTERS; i++){
     if(MultipleCounterLayer::names[i] != NULL){
       if(strcmp(MultipleCounterLayer::names[i], "SGI_TIMERS") == 0){
-	
-	//cout << "sgiTimersMCL is active." << endl;
-	
-	//Set the counter position.
 	sgiTimersMCL_CP[0] = i;
-	
-	//Indicate that this function is being used.
 	MultipleCounterLayer::counterUsed[i] = true;
-	
-	//Update the functionArray.
-	//cout << "Inserting sgiTimersMCL in position: " << functionPosition << endl;
+	MultipleCounterLayer::numberOfCounters[i] = 1;
 	MultipleCounterLayer::functionArray[functionPosition] = sgiTimersMCL;
 	sgiTimersMCL_FP = functionPosition;
-	//Now just return with beginCountersPosition incremented.
 	return true;
       }
     }
   }
-  
-  //If we are here, then this function is not active.
-  //cout << "sgiTimersMCL is not active." << endl;
   return false;
 #else //SGI_TIMERS
   return false;
 #endif//SGI_TIMERS
 }
 
-bool MultipleCounterLayer::cpuTimeMCLInit(int functionPosition){
-#ifdef CPU_TIME
-
-  //This function uses cpu time.
-  //Checks for CPU_TIME
-
+bool MultipleCounterLayer::crayTimersMCLInit(int functionPosition){
+#ifdef CRAY_TIMERS
   for(int i=0; i<MAX_TAU_COUNTERS; i++){
     if(MultipleCounterLayer::names[i] != NULL){
-      if(strcmp(MultipleCounterLayer::names[i], "CPU_TIME") == 0){
-	
-	//cout << "cpuTimeMCL is active." << endl;
-	
-	//Set the counter position.
-	cpuTimeMCL_CP[0] = i;
-	
-	//Indicate that this function is being used.
+      if(strcmp(MultipleCounterLayer::names[i], "CRAY_TIMERS") == 0){
+	crayTimersMCL_CP[0] = i;
 	MultipleCounterLayer::counterUsed[i] = true;
-	
-	//Update the functionArray.
-	//cout << "Inserting cpuTimeMCL in position: " << functionPosition << endl;
+	MultipleCounterLayer::numberOfCounters[i] = 1;
+	MultipleCounterLayer::functionArray[functionPosition] = crayTimersMCL;
+	crayTimersMCL_FP = functionPosition;
+	return true;
+      }
+    }
+  }
+  return false;
+#else //CRAY_TIMERS
+  return false;
+#endif//CRAY_TIMERS
+}
+
+bool MultipleCounterLayer::cpuTimeMCLInit(int functionPosition){
+#ifdef CPU_TIME
+  for(int i=0; i<MAX_TAU_COUNTERS; i++){
+    if(MultipleCounterLayer::names[i] != NULL){
+      if(strcmp(MultipleCounterLayer::names[i], "CPU_TIME") == 0){
+	cpuTimeMCL_CP[0] = i;
+	MultipleCounterLayer::counterUsed[i] = true;
+	MultipleCounterLayer::numberOfCounters[i] = 1;
 	MultipleCounterLayer::functionArray[functionPosition] = cpuTimeMCL;
 	cpuTimeMCL_FP = functionPosition;
-	//Now just return with beginCountersPosition incremented.
 	return true;
       }
     }
   }
-  
-  //If we are here, then this function is not active.
-  //cout << "cpuTimeMCL is not active." << endl;
   return false;
 #else //CPU_TIME
   return false;
 #endif//CPU_TIME
 }
 
+bool MultipleCounterLayer::tauMUSEMCLInit(int functionPosition){
+#ifdef TAU_MUSE
+  for(int i=0; i<MAX_TAU_COUNTERS; i++){
+    if(MultipleCounterLayer::names[i] != NULL){
+      if(strcmp(MultipleCounterLayer::names[i], "TAU_MUSE") == 0){
+	tauMUSEMCL_CP[0] = i;
+	MultipleCounterLayer::counterUsed[i] = true;
+	MultipleCounterLayer::numberOfCounters[i] = 1;
+	MultipleCounterLayer::functionArray[functionPosition] = tauMUSEMCL;
+	tauMUSEMCL_FP = functionPosition;
+	return true;
+      }
+    }
+  }
+  return false;
+#else //TAU_MUSE
+  return false;
+#endif//TAU_MUSE
+}
+
+bool MultipleCounterLayer::tauMPIMessageSizeMCLInit(int functionPosition){
+#ifdef TAU_MPI
+  for(int i=0; i<MAX_TAU_COUNTERS; i++){
+    if(MultipleCounterLayer::names[i] != NULL){
+      if(strcmp(MultipleCounterLayer::names[i], "TAU_MPI_MESSAGE_SIZE") == 0){
+	tauMPIMessageSizeMCL_CP[0] = i;
+	MultipleCounterLayer::counterUsed[i] = true;
+	MultipleCounterLayer::numberOfCounters[i] = 1;
+	MultipleCounterLayer::functionArray[functionPosition] = tauMPIMessageSizeMCL;
+	tauMPIMessageSizeMCL_FP = functionPosition;
+	return true;
+      }
+    }
+  }
+  //If we are here, then this function is not active.
+  return false;
+#else //TAU_MPI
+  return false;
+#endif//TAU_MPI
+}
+
+
 bool MultipleCounterLayer::papiMCLInit(int functionPosition){
 #ifdef TAU_PAPI
-  //This function uses the papi layer counters.
-  
   bool returnValue = false;
-  
   for(int i=0; i<MAX_TAU_COUNTERS; i++){
       if(MultipleCounterLayer::names[i] != NULL){
 	if (strstr(MultipleCounterLayer::names[i],"PAPI") != NULL) {
-	  
+	  //Reset the name if this is a native event.
+	  if (strstr(MultipleCounterLayer::names[i],"NATIVE") != NULL) {
+	    //Shift the string down.
+	    int counter = 0;
+	    while(names[i][12+counter]!='\0'){
+	      MultipleCounterLayer::names[i][counter]=MultipleCounterLayer::names[i][12+counter];
+	      counter++;
+	    }
+	    MultipleCounterLayer::names[i][counter]='\0';
+	    cout << "Adjusted counter name is: " << names[i] << endl;
+	  }
 	  PapiLayer::multiCounterPapiInit();
-
 	  int tmpCode = PapiLayer::map_eventnames(MultipleCounterLayer::names[i]);
-	  
 	  if(tmpCode != -1){
-	 
-	    //cout << "Found a papi counter: " << MultipleCounterLayer::names[i] << endl;
-
-	    //Check if this is possible on this machine!
-	    if((PAPI_query_event(tmpCode) == PAPI_OK)){
-	      
-	      //Set the counter position.
+	    if((PAPI_query_event(tmpCode) == PAPI_OK)){//Check if this is possible on this machine!
 	      papiMCL_CP[numberOfPapiHWCounters] = i;
-	      
-	      //Set the counter code.
-	      MultipleCounterLayer::PAPI_CounterCodeList[numberOfPapiHWCounters] = tmpCode;
-	      
-	      //Update the number of Papi counters.
-	      numberOfPapiHWCounters++;
-	      
-	      //Indicate that this position is being used.
+	      MultipleCounterLayer::PAPI_CounterCodeList[numberOfPapiHWCounters] = tmpCode;//Set the counter code.
+	      numberOfPapiHWCounters++;//Update the number of Papi counters.
 	      MultipleCounterLayer::counterUsed[i] = true;
-	      
+	      MultipleCounterLayer::numberOfCounters[i] = 1;
 	      returnValue = true;
 	    }
 	    else{
@@ -449,91 +500,51 @@ bool MultipleCounterLayer::papiMCLInit(int functionPosition){
       }
     }
   if(returnValue){
-    //If we found viable Papi events, update the function array.
-    //cout << "Inserting papiMCL in position: " << functionPosition << endl;
     MultipleCounterLayer::functionArray[functionPosition] = papiMCL;
-
     papiMCL_FP = functionPosition;
   }
-
-  //if(!returnValue)
-    //cout << "papiMCL is not active." << endl;
   return returnValue;
 #else //TAU_PAPI
   return false;
 #endif//TAU_PAPI
 }
 
-bool MultipleCounterLayer::papiWallClockMCLInit(int functionPosition)
-{
+bool MultipleCounterLayer::papiWallClockMCLInit(int functionPosition){
 #ifdef TAU_PAPI
-  //This function uses the papi PAPI_get_real_usec()  function.
-  //Checks for P_WALL_CLOCK_TIME
-
-  for(int i=0; i<MAX_TAU_COUNTERS; i++)
-    {
+  for(int i=0; i<MAX_TAU_COUNTERS; i++){
       if(MultipleCounterLayer::names[i] != NULL){
 	if(strcmp(MultipleCounterLayer::names[i], "P_WALL_CLOCK_TIME") == 0){
-
-	  //cout << "papiWallClockMCL is active." << endl;
 	  PapiLayer::multiCounterPapiInit();
-
-	  //Set the counter position.
 	  papiWallClockMCL_CP[0] = i;
-
-	  //Indicate that this function is being used.
 	  MultipleCounterLayer::counterUsed[i] = true;
-
-	  //Update the functionArray.
-	  //cout << "Inserting papiWallClockMCL in position: " << functionPosition << endl;
+	  MultipleCounterLayer::numberOfCounters[i] = 1;
 	  MultipleCounterLayer::functionArray[functionPosition] = papiWallClockMCL;
 	  papiWallClockMCL_FP = functionPosition;
-	  //Now just return with beginCountersPosition incremented.
 	  return true;
 	}
       }
     }
-
-  //If we are here, then this function is not active.
-  //cout << "papiWallClockMCL is not active." << endl;
   return false;
 #else  // TAU_PAPI
   return false;
 #endif // TAU_PAPI
 }
 
-bool MultipleCounterLayer::papiVirtualMCLInit(int functionPosition)
-{
+bool MultipleCounterLayer::papiVirtualMCLInit(int functionPosition){
 #ifdef TAU_PAPI
-  //This function uses the papi PAPI_get_virt_usec()  function.
-  //Checks for P_VIRTUAL_TIME
-
-  for(int i=0; i<MAX_TAU_COUNTERS; i++)
-    {
+  for(int i=0; i<MAX_TAU_COUNTERS; i++){
       if(MultipleCounterLayer::names[i] != NULL){
         if(strcmp(MultipleCounterLayer::names[i], "P_VIRTUAL_TIME") == 0){
-
-          //cout << "papiVirtualMCL is active." << endl;
           PapiLayer::multiCounterPapiInit();
-
-          //Set the counter position.
           papiVirtualMCL_CP[0] = i;
-
-	  //Indicate that this function is being used.
 	  MultipleCounterLayer::counterUsed[i] = true;
-
-          //Update the functionArray.
-          //cout << "Inserting papiVirtualMCL in position: " << functionPosition << endl;
+	  MultipleCounterLayer::numberOfCounters[i] = 1;
           MultipleCounterLayer::functionArray[functionPosition] = papiVirtualMCL;
 	  papiVirtualMCL_FP = functionPosition;
-          //Now just return with beginCountersPosition incremented.
           return true;
         }
       }
     }
-
-  //If we are here, then this function is not active.
-  //cout << "papiVirtualMCL is not active." << endl;
   return false;
 #else  // TAU_PAPI
   return false;
@@ -542,51 +553,31 @@ bool MultipleCounterLayer::papiVirtualMCLInit(int functionPosition)
 
 bool MultipleCounterLayer::pclMCLInit(int functionPosition){
 #ifdef  TAU_PCL
-  //This function uses the pcl layer counters.
-  
-  //  bool returnValue = false;
-  
   for(int i=0; i<MAX_TAU_COUNTERS; i++){
     if(MultipleCounterLayer::names[i] != NULL){
       if (strstr(MultipleCounterLayer::names[i],"PCL") != NULL) {
-	
 	PCL_Layer::multiCounterPCLInit(&MultipleCounterLayer::descr);
-	
 	int tmpCode = PCL_Layer::map_eventnames(MultipleCounterLayer::names[i]);
-	
 	if(tmpCode != -1){
-	  
-	  //cout << "Found a pcl counter: " << MultipleCounterLayer::names[i] << endl;
-	  
-	  //Set the counter position.
 	  pclMCL_CP[numberOfPCLHWCounters] = i;
-	  
-	  //Set the counter code.
 	  MultipleCounterLayer::PCL_CounterCodeList[numberOfPCLHWCounters] = tmpCode;
-	  
-	  //Update the number of Papi counters.
 	  numberOfPCLHWCounters++;
 	}
       }
     }
   }
-
   if(numberOfPCLHWCounters != 0){
-    //Now check whether these pcl events are possible.
+    //Check whether these pcl events are possible.
     if(PCLquery(descr, PCL_CounterCodeList, numberOfPCLHWCounters, PCL_Mode) == PCL_SUCCESS){
-      //We found viable Pcl events, update the counterUsed and function arrays.
       for(int j=0;j<numberOfPCLHWCounters;j++){
 	MultipleCounterLayer::counterUsed[pclMCL_CP[j]] = true;
+	MultipleCounterLayer::numberOfCounters[pclMCL_CP[j]] = 1;
       }
-      //cout << "Inserting pclMCL in position: " << functionPosition << endl;
       MultipleCounterLayer::functionArray[functionPosition] = pclMCL;
-      //papiVirtualMCL_FP = functionPosition;
-
       return true;
     }
     else{
       cout << "Requested pcl events, or event combination not possible!" << endl;
-      //cout << "pclMCL is not active." << endl;
       return false;
     } 
   }
@@ -598,34 +589,18 @@ bool MultipleCounterLayer::pclMCLInit(int functionPosition){
 
 bool MultipleCounterLayer::linuxTimerMCLInit(int functionPosition){
 #ifdef  TAU_LINUX_TIMERS
-//This function uses the linux timer interface.
-  //Checks for LINUX_TIMERS
-
-  for(int i=0; i<MAX_TAU_COUNTERS; i++)
-    {
+  for(int i=0; i<MAX_TAU_COUNTERS; i++){
       if(MultipleCounterLayer::names[i] != NULL){
 	if(strcmp(MultipleCounterLayer::names[i], "LINUX_TIMERS") == 0){
-	
-	  //cout << "linuxTimerMCL is active." << endl;
-
-	  //Set the counter position.
 	  linuxTimerMCL_CP[0] = i;
-
-	  //Indicate that this function is being used.
 	  MultipleCounterLayer::counterUsed[i] = true;
-
-	  //Update the functionArray.
-	  //cout << "Inserting linuxTimerMCL in position: " << functionPosition << endl;
+	  MultipleCounterLayer::numberOfCounters[i] = 1;
 	  MultipleCounterLayer::functionArray[functionPosition] = linuxTimerMCL;
 	  linuxTimerMCL_FP = functionPosition;
-	  //Now just return with beginCountersPosition incremented.
 	  return true;
 	}
       }
     }
-
-  //If we are here, then this function is not active.
-  //cout << "linuxTimerMCL is not active." << endl;
   return false;
 #else //TAU_LINUX_TIMERS
 return false;
@@ -646,6 +621,13 @@ void MultipleCounterLayer::sgiTimersMCL(int tid, double values[]){
 #endif//SGI_TIMERS
 }
 
+void MultipleCounterLayer::crayTimersMCL(int tid, double values[]){
+#ifdef  CRAY_TIMERS
+  long long tick = _rtc();
+  values[crayTimersMCL_CP[0]] = (double) tick/HZ; 
+#endif // CRAY_TIMERS
+}
+
 void MultipleCounterLayer::cpuTimeMCL(int tid, double values[]){
 #ifdef  CPU_TIME
   struct rusage current_usage;
@@ -657,6 +639,18 @@ void MultipleCounterLayer::cpuTimeMCL(int tid, double values[]){
 #endif//CPU_TIME
 }
 
+void MultipleCounterLayer::tauMUSEMCL(int tid, double values[]){
+#ifdef TAU_MUSE 
+  values[tauMUSEMCL_CP[0]] = TauMuseQuery();
+#endif//TAU_MUSE
+}
+
+void MultipleCounterLayer::tauMPIMessageSizeMCL(int tid, double values[]){
+#ifdef TAU_MPI
+  values[tauMPIMessageSizeMCL_CP[0]] = sendevent.GetSumValue(tid); //Currently TAU_EVENT_DATATYPE is a double.
+#endif//TAU_MPI
+}
+
 void MultipleCounterLayer::papiMCL(int tid, double values[]){
 #ifdef TAU_PAPI
   //******************************************
@@ -683,9 +677,20 @@ void MultipleCounterLayer::papiMCL(int tid, double values[]){
       
       PAPI_create_eventset(&(ThreadList[tid]->EventSet));
       
+#ifndef PAPI_VERSION
+/* PAPI 2 support goes here */
       int resultCode = PAPI_add_events(&(ThreadList[tid]->EventSet),
 				       PAPI_CounterCodeList,
 				       numberOfPapiHWCounters);
+#elif (PAPI_VERSION_MAJOR(PAPI_VERSION) == 3)
+/* PAPI 3 support goes here */
+      int resultCode = PAPI_add_events(ThreadList[tid]->EventSet,
+				       PAPI_CounterCodeList,
+				       numberOfPapiHWCounters);
+#else
+/* PAPI future support goes here */
+#error "Compiling against a not yet released PAPI version"
+#endif 
 
       if(resultCode != PAPI_OK){
 	cout << "Error adding Papi events!" << endl;
@@ -738,8 +743,20 @@ void MultipleCounterLayer::papiMCL(int tid, double values[]){
 }
 
 void MultipleCounterLayer::papiWallClockMCL(int tid, double values[]){
+static long long oldvalue = 0L;
+static long long offset = 0;
+long long newvalue = 0L;
 #ifdef TAU_PAPI
-  values[papiWallClockMCL_CP[0]] = PAPI_get_real_usec();
+  newvalue = PAPI_get_real_usec();
+  if (newvalue < oldvalue) 
+  {
+    offset += UINT_MAX;
+    DEBUGPROFMSG("WARNING: papi counter overflow. Fixed in TAU! new = "
+	 <<newvalue <<" old = " <<oldvalue<<" offset = "<<offset <<endl;);
+    DEBUGPROFMSG("Returning "<<newvalue + offset<<endl;);
+  }
+  oldvalue = newvalue;
+  values[papiWallClockMCL_CP[0]] = newvalue + offset; 
 #endif // TAU_PAPI
 }
 void MultipleCounterLayer::papiVirtualMCL(int tid, double values[]){
@@ -829,21 +846,12 @@ inline double TauGetMHzMCL(void)
   return ratings;
 }
 ///////////////////////////////////////////////////////////////////////////
-#ifdef __ia64__
+// Fix Intel compiler does not support this asm statements
+extern "C" unsigned long long getLinuxHighResolutionTscCounter(void);
 inline unsigned long long getLinuxHighResolutionTscCounterMCL(void)
 {
-  unsigned long long tmp;
-  __asm__ __volatile__("mov %0=ar.itc" : "=r"(tmp) :: "memory");
-  return tmp;
+  return getLinuxHighResolutionTscCounter();
 }
-#else /* IA64 */
-inline unsigned long long getLinuxHighResolutionTscCounterMCL(void)
-{
-   unsigned long high, low;
-   __asm__ __volatile__(".byte 0x0f,0x31" : "=a" (low), "=d" (high));
-   return ((unsigned long long) high << 32) + low;
-}
-#endif /* IA64 */
 ///////////////////////////////////////////////////////////////////////////
 
 #endif //TAU_LINUX_TIMERS
