@@ -115,12 +115,12 @@ void TauAppShutdown(void)
 char * TauGetCounterString(void)
 {
   char *header = new char[64];
-  char *tau_env = NULL;
 #ifdef SGI_HW_COUNTERS
   return "templated_functions_hw_counters";
 #elif (defined (TAU_PAPI) || defined (TAU_PCL) \
 		|| defined(TAU_PAPI_WALLCLOCKTIME) \
 		|| defined(TAU_PAPI_VIRTUAL))
+  char *tau_env = NULL;
 
 #ifdef TAU_PAPI
   tau_env = getenv("PAPI_EVENT");
@@ -140,12 +140,16 @@ char * TauGetCounterString(void)
     return "templated_functions_MULTI_P_WALL_CLOCK_TIME";
 #endif /* TAU_PAPI_WALLCLOCKTIME */
 
+
 #ifdef TAU_PAPI_VIRTUAL
     return "templated_functions_MULTI_P_VIRTUAL_TIME";
 #endif /* TAU_PAPI_VIRTUAL */
     return "templated_functions_hw_counters";
   }
 #else  // ! (TAU_PAPI/PCL) => SGI_TIMERS, TULIP_TIMERS 
+#ifdef TAU_MUSE
+    return "templated_functions_hw_counters";
+#endif /* TAU_MUSE */
   return "templated_functions";
 #endif // ALL options
 }
@@ -157,6 +161,7 @@ char * TauGetCounterString(void)
 
 void Profiler::Start(int tid)
 { 
+      unsigned long long TimeStamp = 0L;
       DEBUGPROFMSG("Profiler::Start: MyProfileGroup_ = " << MyProfileGroup_ 
         << " Mask = " << RtsLayer::TheProfileMask() <<endl;);
       if ((MyProfileGroup_ & RtsLayer::TheProfileMask()) 
@@ -164,6 +169,24 @@ void Profiler::Start(int tid)
 	if (ThisFunction == (FunctionInfo *) NULL) return; // Mapping
       DEBUGPROFMSG("Profiler::Start Entering " << ThisFunction->GetName()<<endl;);
 	
+#ifdef TAU_COMPENSATE
+	SetNumChildren(0); /* for instrumentation perturbation compensation */
+#endif /* TAU_COMPENSATE */
+	// Initialization is over, now record the time it started
+#ifndef TAU_MULTIPLE_COUNTERS 
+	StartTime =  RtsLayer::getUSecD(tid) ;
+	TimeStamp += (unsigned long long) StartTime;
+#else //TAU_MULTIPLE_COUNTERS
+	//Initialize the array to zero, as some of the elements will
+	//not be set by counting functions.
+	for(int i=0;i<MAX_TAU_COUNTERS;i++){
+	  StartTime[i]=0;
+	}
+	//Now get the start times.
+	RtsLayer::getUSecD(tid, StartTime);	  
+	TimeStamp += (unsigned long long) StartTime[0]; // USE COUNTER1 for tracing
+#endif//TAU_MULTIPLE_COUNTERS
+
 #ifdef TAU_CALLPATH
         CallPathStart(tid);
 #endif // TAU_CALLPATH
@@ -172,9 +195,11 @@ void Profiler::Start(int tid)
 #ifdef TAU_EPILOG
 	DEBUGPROFMSG("Calling elg_enter: ["<<ThisFunction->GetFunctionId()<<"] "
 	     << ThisFunction->GetName()<<endl;);
-	elg_enter(ThisFunction->GetFunctionId(), ELG_NO_ID, ELG_NO_LNO);
+	elg_enter(ThisFunction->GetFunctionId());
 #else /* TAU_EPILOG */
-	TraceEvent(ThisFunction->GetFunctionId(), 1, tid); // 1 is for entry
+	TraceEvent(ThisFunction->GetFunctionId(), 1, tid, TimeStamp, 1); 
+	// 1 is for entry in second parameter and for use TimeStamp in last
+	DEBUGPROFMSG("Start TimeStamp for Tracing = "<<TimeStamp<<endl;);
 #endif /* TAU_EPILOG */
 #endif /* TRACING_ON */
 
@@ -198,18 +223,6 @@ void Profiler::Start(int tid)
 	  AddInclFlag = false;
 	}
 	
-	// Initialization is over, now record the time it started
-#ifndef TAU_MULTIPLE_COUNTERS 
-	StartTime =  RtsLayer::getUSecD(tid) ;
-#else //TAU_MULTIPLE_COUNTERS
-	//Initialize the array to zero, as some of the elements will
-	//not be set by counting functions.
-	for(int i=0;i<MAX_TAU_COUNTERS;i++){
-	  StartTime[i]=0;
-	}
-	//Now get the start times.
-	RtsLayer::getUSecD(tid, StartTime);	  
-#endif//TAU_MULTIPLE_COUNTERS
 	DEBUGPROFMSG("Start Time = "<< StartTime<<endl;);
 #endif // PROFILING_ON
   	
@@ -322,6 +335,8 @@ Profiler& Profiler::operator= (const Profiler& X)
 
 void Profiler::Stop(int tid)
 {
+      unsigned long long TimeStamp = 0L; 
+      if (CurrentProfiler[tid] == NULL) return;
       DEBUGPROFMSG("Profiler::Stop: MyProfileGroup_ = " << MyProfileGroup_ 
         << " Mask = " << RtsLayer::TheProfileMask() <<endl;);
       if ((MyProfileGroup_ & RtsLayer::TheProfileMask()) 
@@ -329,37 +344,75 @@ void Profiler::Stop(int tid)
 	if (ThisFunction == (FunctionInfo *) NULL) return; // Mapping
         DEBUGPROFMSG("Profiler::Stop for routine = " << ThisFunction->GetName()<<endl;);
 
+#ifdef TAU_COMPENSATE 
+	DEBUGPROFMSG("Profiler::Stop - "<<ThisFunction->GetName()<<" NumChildren = "
+        <<GetNumChildren()<<endl;);
+#endif /* TAU_COMPENSATE */
+
+#ifdef TAU_COMPENSATE
+#ifndef TAU_MULTIPLE_COUNTERS 
+	double tover = TauGetTimerOverhead(TauFullTimerOverhead);
+	double tnull = TauGetTimerOverhead(TauNullTimerOverhead);
+#endif /* TAU_MULTIPLE_COUNTERS */
+#endif /* TAU_COMPENSATE */
+
+#ifndef TAU_MULTIPLE_COUNTERS
+	double CurrentTime = RtsLayer::getUSecD(tid);
+	double TotalTime = CurrentTime - StartTime;
+	TimeStamp += (unsigned long long) CurrentTime; 
+
+
+#if (defined(TAU_COMPENSATE ) && defined(PROFILING_ON))
+	/* To compensate for timing overhead, shrink the totaltime! */
+	TotalTime = TotalTime - tnull - GetNumChildren() * tover; 
+	if (TotalTime < 0 ) {
+	  TotalTime = 0;
+	  DEBUGPROFMSG("TotalTime negative in "<<ThisFunction->GetName()<<endl;);
+        }
+#endif /* TAU_COMPENSATE && PROFILING_ON */
+#else //TAU_MULTIPLE_COUNTERS
+	// first initialize the CurrentTime
+        int i;
+	for (i=0; i < MAX_TAU_COUNTERS; i++)
+	{
+	  CurrentTime[i] = 0;
+	}
+	//Get the current counter values.
+	RtsLayer::getUSecD(tid, CurrentTime);
+
+#ifdef PROFILING_ON
+#ifdef TAU_COMPENSATE
+	double *tover = TauGetTimerOverhead(TauFullTimerOverhead);
+	double *tnull = TauGetTimerOverhead(TauNullTimerOverhead);
+#endif /* TAU_COMPENSATE */
+	for(int k=0;k<MAX_TAU_COUNTERS;k++){
+	  TotalTime[k] = CurrentTime[k] - StartTime[k];
+#ifdef TAU_COMPENSATE 
+  	  /* To compensate for timing overhead, shrink the totaltime! */
+	  TotalTime[k] = TotalTime[k] - tnull[k] - GetNumChildren() * tover[k]; 
+	  if (TotalTime[k] < 0 ) {
+	    TotalTime[k] = 0;
+	    DEBUGPROFMSG("TotalTime[" <<k<<"] negative in "<<ThisFunction->GetName()<<endl;);
+          }
+#endif /* TAU_COMPENSATE */
+	}
+#endif // PROFILING_ON
+	TimeStamp += (unsigned long long) CurrentTime[0]; // USE COUNTER1
+	
+#endif//TAU_MULTIPLE_COUNTERS
 
 #ifdef TRACING_ON
 #ifdef TAU_EPILOG
         DEBUGPROFMSG("Calling elg_exit(): "<< ThisFunction->GetName()<<endl;);
 	elg_exit();
 #else /* TAU_EPILOG */
-	TraceEvent(ThisFunction->GetFunctionId(), -1, tid); // -1 is for exit
+	TraceEvent(ThisFunction->GetFunctionId(), -1, tid, TimeStamp, 1); 
+	// -1 is for exit, 1 is for use TimeStamp in the last argument
+	DEBUGPROFMSG("Stop TimeStamp for Tracing = "<<TimeStamp<<endl;);
 #endif /* TAU_EPILOG */
 #endif //TRACING_ON
 
 #ifdef PROFILING_ON  // Calculations relevent to profiling only 
-#ifndef TAU_MULTIPLE_COUNTERS
-	double TotalTime = RtsLayer::getUSecD(tid) - StartTime;
-#else //TAU_MULTIPLE_COUNTERS
-	double CurrentTime[MAX_TAU_COUNTERS];
-	for(int j=0;j<MAX_TAU_COUNTERS;j++){
-	  CurrentTime[j]=0;
-	}
-	//Get the current counter values.
-	RtsLayer::getUSecD(tid, CurrentTime);
-
-	double TotalTime[MAX_TAU_COUNTERS];
-	for(int i=0;i<MAX_TAU_COUNTERS;i++){
-	  TotalTime[i]=0;
-	}
-
-	for(int k=0;k<MAX_TAU_COUNTERS;k++){
-	  TotalTime[k] = CurrentTime[k] - StartTime[k];
-	}
-	
-#endif//TAU_MULTIPLE_COUNTERS
 
 #ifdef TAU_CALLPATH
 	    CallPathStop(TotalTime, tid);
@@ -389,6 +442,30 @@ void Profiler::Stop(int tid)
 	ThisFunction->AddExclTime(TotalTime, tid);
 	// In either case we need to add time to the exclusive time.
 
+#ifdef TAU_COMPENSATE
+#ifndef TAU_MULTIPLE_COUNTERS
+	if (ThisFunction->GetExclTime(tid) < 0) 
+	{
+ 	  ThisFunction->SetExclTime(tid, 0.0);
+	}
+#else /* TAU_MULTIPLE_COUNTERS */
+ 	/* Lets get the vector of exclusive times for each counter */
+	double * exclcounters= ThisFunction->GetExclTime(tid);
+        for(i = 0; i < MAX_TAU_COUNTERS; i++)	
+	{
+	  if (exclcounters[i] < 0)
+	    exclcounters[i] = 0.0; 
+	}
+ 	ThisFunction->SetExclTime(tid, exclcounters);
+#endif /* TAU_MULTIPLE_COUNTERS */
+#ifdef TAU_CALLPATH
+	if ((ParentProfiler != NULL) && (CallPathFunction->GetExclTime(tid) < 0)) 
+	{
+ 	  CallPathFunction->SetExclTime(tid, 0.0);
+	}
+#endif /* TAU_CALLPATH */
+#endif /* TAU_COMPENSATE */
+
 #if ( defined(PROFILE_CALLS) || defined(PROFILE_STATS)|| defined(PROFILE_CALLSTACK) )
 	ExclTimeThisCall += TotalTime;
 	DEBUGPROFMSG("nct "<< RtsLayer::myNode()  << ","
@@ -401,10 +478,18 @@ void Profiler::Stop(int tid)
 
 #ifdef PROFILE_CALLS
 	ThisFunction->AppendExclInclTimeThisCall(ExclTimeThisCall, TotalTime);
+#ifdef TAU_CALLPATH
+	if (CallPathFunction)
+	  CallPathFunction->AppendExclInclTimeThisCall(ExclTimeThisCall, TotalTime);
+#endif /* TAU_CALLPATH */
 #endif // PROFILE_CALLS
 
 #ifdef PROFILE_STATS
 	ThisFunction->AddSumExclSqr(ExclTimeThisCall*ExclTimeThisCall, tid);
+#ifdef TAU_CALLPATH
+	if (CallPathFunction)
+	  CallPathFunction->AddSumExclSqr(ExclTimeThisCall*ExclTimeThisCall, tid);
+#endif /* TAU_CALLPATH */
 #endif // PROFILE_STATS
 
 	if (ParentProfiler != (Profiler *) NULL) {
@@ -428,6 +513,11 @@ void Profiler::Stop(int tid)
 	  ParentProfiler->ExcludeTimeThisCall(TotalTime);
 #endif //PROFILE_CALLS || PROFILE_STATS || PROFILE_CALLSTACK
 
+#ifdef TAU_COMPENSATE
+	ParentProfiler->AddNumChildren(GetNumChildren()+1);
+        /* Add 1 and my children to my parents total number of children */
+#endif /* TAU_COMPENSATE */
+
 	}
 	
 #endif //PROFILING_ON
@@ -761,7 +851,6 @@ int Profiler::dumpFunctionValues(const char **inFuncs,
 
 #endif //PROFILING_ON
 #ifdef PROFILE_CALLS
-	long listSize, numCalls;
 	list<pair<double,double> >::iterator iter;
 #endif // PROFILE_CALLS
  	double excltime, incltime; 
@@ -1182,7 +1271,6 @@ int Profiler::DumpData(bool increment, int tid, char *prefix)
  	int numFunc, numEvents;
 #endif //PROFILING_ON
 #ifdef PROFILE_CALLS
-	long listSize, numCalls;
 	list<pair<double,double> >::iterator iter;
 #endif // PROFILE_CALLS
  	double excltime, incltime; 
@@ -1842,9 +1930,8 @@ int Profiler::dumpFunctionValues(const char **inFuncs,
 			 << " Excl : " << 0 << " Incl : "
 			 << 0 << endl;);
 	    
-	    fprintf(fp,"\"%s %s\" %ld %ld %.16G %.16G ", (*it)->GetName(),
-		    (*it)->GetType(), (*it)->GetCalls(tid), (*it)->GetSubrs(tid),
-		    0, 0);
+	    fprintf(fp,"\"%s %s\" %ld %ld 0 0 ", (*it)->GetName(),
+	    (*it)->GetType(), (*it)->GetCalls(tid), (*it)->GetSubrs(tid));
 	  }
 	  fprintf(fp,"0 "); // Indicating - profile calls is turned off
 	  fprintf(fp,"GROUP=\"%s\" \n", (*it)->GetAllGroups());
@@ -2252,9 +2339,8 @@ int Profiler::DumpData(bool increment, int tid, char *prefix){
 		       << " Excl : " << 0 << " Incl : "
 		       << 0 << endl;);
 	  
-	  fprintf(fp,"\"%s %s\" %ld %ld %.16G %.16G ", (*it)->GetName(),
-		  (*it)->GetType(), (*it)->GetCalls(tid), (*it)->GetSubrs(tid),
-		  0, 0);
+	  fprintf(fp,"\"%s %s\" %ld %ld 0 0 ", (*it)->GetName(),
+		  (*it)->GetType(), (*it)->GetCalls(tid), (*it)->GetSubrs(tid));
 	}
 	fprintf(fp,"0 "); // Indicating - profile calls is turned off
 	fprintf(fp,"GROUP=\"%s\" \n", (*it)->GetAllGroups());
@@ -2516,11 +2602,41 @@ void Profiler::CallStackTrace(int tid)
 /*-----------------------------------------------------------------*/
 #endif //PROFILE_CALLSTACK
 
+#ifdef TAU_COMPENSATE
+//////////////////////////////////////////////////////////////////////
+//  Profiler::GetNumChildren()
+//  Description: Returns the total number of child timers (including 
+//  children of children) executed under the given profiler. 
+//////////////////////////////////////////////////////////////////////
+long Profiler::GetNumChildren(void)
+{
+  return NumChildren;
+}
+
+//////////////////////////////////////////////////////////////////////
+//  Profiler::SetNumChildren(value)
+//  Description: Sets the total number of child timers.
+//////////////////////////////////////////////////////////////////////
+void Profiler::SetNumChildren(long value)
+{
+  NumChildren = value;
+}
+
+//////////////////////////////////////////////////////////////////////
+//  Profiler::AddNumChildren(value)
+//  Description: increments by value the number of child timers.
+//////////////////////////////////////////////////////////////////////
+void Profiler::AddNumChildren(long value)
+{
+  NumChildren += value;
+}
+#endif /* TAU_COMPENSATE */
+
 
 /***************************************************************************
  * $RCSfile: Profiler.cpp,v $   $Author: sameer $
- * $Revision: 1.84 $   $Date: 2003/07/22 17:21:22 $
- * POOMA_VERSION_ID: $Id: Profiler.cpp,v 1.84 2003/07/22 17:21:22 sameer Exp $ 
+ * $Revision: 1.97 $   $Date: 2004/06/16 00:26:55 $
+ * POOMA_VERSION_ID: $Id: Profiler.cpp,v 1.97 2004/06/16 00:26:55 sameer Exp $ 
  ***************************************************************************/
 
 	
