@@ -43,6 +43,11 @@
 #endif /* HITACHI */
 #include "Profile/Profiler.h"
 
+#ifdef CRAY_TIMERS
+#include <intrinsics.h>
+#include <sys/param.h>
+#endif // CRAY_TIMERS
+
 #ifdef TAU_XLC
 #define strcasecmp strcmp
 #define strncasecmp strncmp 
@@ -121,7 +126,11 @@ bool& RtsLayer::TheEnableInstrumentation(void)
 /////////////////////////////////////////////////////////////////////////
 int& RtsLayer::TheNode(void)
 {
-  static int Node = -1;
+#ifdef TAU_SETNODE0
+  static int Node = 0;
+#else /* TAU_SETNODE0  */
+  static int Node =-1;
+#endif /* TAU_SETNODE0 */
  
   return Node;
 }
@@ -521,6 +530,15 @@ double RtsLayer::getUSecD (int tid) {
   return (tp.tv_sec * 1e6 + (tp.tv_nsec * 1e-3)) ;
 
 #else  // SGI_TIMERS
+#ifdef CRAY_TIMERS
+  long long tick = _rtc();
+  return (double) tick/HZ;
+#endif // CRAY_TIMERS
+#ifdef TAU_ALPHA_TIMERS
+  struct timespec currenttime;
+  clock_gettime(CLOCK_REALTIME, &currenttime);
+  return (currenttime.tv_sec * 1e6 + (currenttime.tv_nsec * 1e-3));
+#endif /* TAU_ALPHA_TIMERS */
 #ifdef TAU_LINUX_TIMERS
   return (double) getLinuxHighResolutionTscCounter()/TauGetMHz();
 #else /* TAU_LINUX_TIMERS */
@@ -528,9 +546,78 @@ double RtsLayer::getUSecD (int tid) {
 #if (defined(TAU_WINDOWS))
   return TauWindowsUsecD();
 #else // TAU_WINDOWS 
+#ifdef TAU_MUSE
+#ifdef DEBUG_PROF
+  // TO CHECK IF THE VALUE IS MONOTONICALLY INCREASING
+  double queryValue = 0.0;
+  static double lastQueryValue = 0.0;
+  queryValue = TauMuseQuery();
+  char msg[200];
+  if(queryValue < lastQueryValue){
+        if(queryValue < 0){
+		DEBUGPROFMSG("TauMuseQuery() came out negative!!!!!."<<endl;);
+        }else{
+		DEBUGPROFMSG("TauMuseQuery() less than lastQueryValue.!!!!!"<<endl;);
+		sprintf(msg,"TauMuseQuery() lastQueryValue=%f\n",lastQueryValue);
+		DEBUGPROFMSG(msg);
+		sprintf(msg,"TauMuseQuery() queryValue=%f\n",queryValue);
+		DEBUGPROFMSG(msg);
+        }
+        queryValue = lastQueryValue;
+  }
+  lastQueryValue = queryValue;
+  return queryValue;
+#else //DEBUG_PROF
+  
+  return TauMuseQuery();
+
+#endif //DEBUG_PROF
+#else /* TAU_MUSE */
+#ifdef TAU_MUSE_MULTIPLE
+#ifdef DEBUG_PROF
+  // TO CHECK IF THE VALUE IS MONOTONICALLY INCREASING
+  double queryValue = 0.0;
+  static double lastQueryValue = 0.0;
+  char msg[200];
+  double data[10];
+  int size = 10;
+  queryValue = TauMuseMultipleQuery(data,size);
+  if(queryValue < lastQueryValue){
+        if(queryValue < 0){
+		DEBUGPROFMSG("TauMuseQuery() came out negative!!!!!."<<endl;);
+        }else{
+		DEBUGPROFMSG("TauMuseQuery() less than lastQueryValue.!!!!!"<<endl;);
+		sprintf(msg,"TauMuseQuery() lastQueryValue=%f\n",lastQueryValue);
+		DEBUGPROFMSG(msg);
+		sprintf(msg,"TauMuseQuery() queryValue=%f\n",queryValue);
+		DEBUGPROFMSG(msg);
+        }
+        queryValue = lastQueryValue;
+  }
+  lastQueryValue = queryValue;
+  return queryValue;
+#else //DEBUG_PROF
+ 
+  double data[10];
+  int size=10; 
+  return TauMuseMultipleQuery(data,size);
+
+#endif //DEBUG_PROF
+#else //TAU_MUSE_MULTIPLE
   struct timeval tp;
+  static double last_timestamp = 0.0;
+  double timestamp;
   gettimeofday (&tp, 0);
-  return ( (double) tp.tv_sec * 1e6 + tp.tv_usec );
+  timestamp = (double) tp.tv_sec * 1e6 + tp.tv_usec;
+  if (timestamp < last_timestamp)
+  {
+     DEBUGPROFMSG("RtsLayer::getUSecD(): gettimeofday() goes back in time. Fixing ...."<<endl;);
+     timestamp = last_timestamp;
+  }
+  last_timestamp = timestamp;
+  return timestamp;
+#endif // TAU_MUSE_EVENT
+#endif /* TAU_MUSE */
 #endif // TAU_WINDOWS
 #else  // TULIP_TIMERS by default.  
   return pcxx_GetUSecD();
@@ -803,6 +890,20 @@ void RtsLayer::ProfileInit(int& argc, char**& argv)
   int i;
   int ret_argc;
   char **ret_argv;
+
+#ifdef TAU_COMPENSATE
+#ifndef TAU_MULTIPLE_COUNTERS
+  double tover = TauGetTimerOverhead(TauNullTimerOverhead);
+  if (tover < 0) tover = 0;
+#else /* TAU_MULTIPLE_COUNTERS */
+  double* tover = TauGetTimerOverhead(TauNullTimerOverhead);
+  for (i = 0; i < MAX_TAU_COUNTERS; i++)
+  { /* iterate through all counters and reset null overhead to zero 
+       if necessary */
+    if (tover[i] < 0) tover[i] = 0;
+  }
+#endif /* TAU_MULTIPLE_COUNTERS */
+#endif /* TAU_COMPENSATE */
   
   ret_argc = 1;
   ret_argv = new char *[argc];
@@ -831,32 +932,9 @@ void RtsLayer::ProfileInit(int& argc, char**& argv)
 //////////////////////////////////////////////////////////////////////
 bool RtsLayer::isCtorDtor(const char *name)
 {
-
-  // If the destructor a static object is called, it could have a null name
-  // after main is over. Treat it like a Dtor and return true.
-  if (name[0] == 0) {
-    DEBUGPROFMSG("isCtorDtor name is NULL" << endl;);
-    return true; 
-  }
-  DEBUGPROFMSG("RtsLayer::isCtorDtor("<< name <<")" <<endl;);
-#ifdef JAVA
-      /* In Java : is allowed, as the top level thread name could be something 
-	 like THREAD=RMI ConnectionExpiration-[192.168.180.1:1871]; */
-      DEBUGPROFMSG("JAVA RtsLayer::isCtorDtor("<<name<<") returns false! "<<endl;);
-      return false;
-#endif /* JAVA */
-  if (myThread() == 0) 
-  {
-    if (strchr(name,'~') == NULL) // a destructor 
-      if (strchr(name,':') == NULL) // could be a constructor 
-        return false;
-      else  
-        return true; /* in C++ a : in a name could signify a ctor/dtor */
-    else  
-      return true;
-  }
-  else 
-    return false; /* in C++, on other threads don't check for this */
+  return false;
+  // For now, we always return false. So, no matter what, the profile file
+  // is always created! 
 }
 
 //////////////////////////////////////////////////////////////////////
@@ -1018,8 +1096,32 @@ int RtsLayer::DumpEDF(int tid)
 	return 1;
 }
 
+#ifdef __GNUC__
+#ifndef NO_RTTI
+#include <cxxabi.h>
+#endif /* NO_RTTI */
+#endif /* __GNUC__ */
+
+/////////////////////////////////////////////////////////////////////////
+std::string RtsLayer::GetRTTI(const char *name)
+{
+#ifdef __GNUC__
+#ifndef NO_RTTI
+  std::size_t len;
+  int stat;
+  char *ptr = NULL;
+  const std::string mangled = name;
+  return abi::__cxa_demangle(mangled.c_str(), ptr, &len, &stat);
+#else /* NO_RTTI */
+  return string(name);
+#endif /* NO_RTTI */
+#else
+  return string(CheckNotNull(name));
+#endif /* GNUC */
+}
+
 /***************************************************************************
  * $RCSfile: RtsLayer.cpp,v $   $Author: sameer $
- * $Revision: 1.43 $   $Date: 2002/07/26 20:10:14 $
- * POOMA_VERSION_ID: $Id: RtsLayer.cpp,v 1.43 2002/07/26 20:10:14 sameer Exp $ 
+ * $Revision: 1.56 $   $Date: 2004/03/03 01:07:49 $
+ * POOMA_VERSION_ID: $Id: RtsLayer.cpp,v 1.56 2004/03/03 01:07:49 sameer Exp $ 
  ***************************************************************************/
