@@ -58,7 +58,12 @@ extern "C" void * Tau_get_profiler(char *fname, char *type, TauGroup_t group, ch
 
   // since we're using new, we should set InitData to true in FunctionInfoInit
   if (group == TAU_MESSAGE)
-    f = new FunctionInfo(fname, type, group, "MPI", true);
+  {
+    if (gr_name && strcmp(gr_name, "TAU_MESSAGE") == 0)
+      f = new FunctionInfo(fname, type, group, "MPI", true);
+    else 
+      f = new FunctionInfo(fname, type, group, gr_name, true);
+  }
   else 
     f = new FunctionInfo(fname, type, group, gr_name, true);
 //  p = new Profiler(f, group, true);
@@ -94,6 +99,11 @@ extern "C" void Tau_stop_timer(void * function_info)
 #pragma omp critical
   printf("STOP tid = %d, profiler= %x\n", RtsLayer::myThread(), p);
 */
+    if (p->ThisFunction != f)
+    {
+      printf("TAU: Runtime overlap in timers: %s and %s\n",
+        p->ThisFunction->GetName(), f->GetName());
+    }
     p->Stop();
     delete p;
   }
@@ -287,6 +297,17 @@ extern "C" int& tau_totalnodes(int set_or_get, int value)
 
 #ifdef TAU_MPI
 TAU_REGISTER_EVENT(sendevent,"Message size sent to all nodes");
+TAU_REGISTER_EVENT(recvevent,"Message size received from all nodes");
+TAU_REGISTER_EVENT(bcastevent,"Message size for broadcast");
+TAU_REGISTER_EVENT(reduceevent,"Message size for reduce");
+TAU_REGISTER_EVENT(reducescatterevent,"Message size for reduce-scatter");
+TAU_REGISTER_EVENT(scanevent,"Message size for scan");
+TAU_REGISTER_EVENT(allreduceevent,"Message size for all-reduce");
+TAU_REGISTER_EVENT(alltoallevent,"Message size for all-to-all");
+TAU_REGISTER_EVENT(scatterevent,"Message size for scatter");
+TAU_REGISTER_EVENT(gatherevent,"Message size for gather");
+TAU_REGISTER_EVENT(allgatherevent,"Message size for all-gather");
+
 TauUserEvent**& TheMsgVolEvent()
 {
   static TauUserEvent **u = 0; 
@@ -297,14 +318,17 @@ TauUserEvent **u;
 */
 int register_events(void)
 {
+#ifdef TAU_EACH_SEND
   char str[256];
   int i;
+
   TheMsgVolEvent() = (TauUserEvent **) malloc(sizeof(TauUserEvent *)*tau_totalnodes(0,0));
   for (i =0; i < tau_totalnodes(0,0); i++)
   {
     sprintf(str, "Message size sent to node %d", i);
     TheMsgVolEvent()[i] = (TauUserEvent *) new TauUserEvent((const char *)str);
   }
+#endif /* TAU_EACH_SEND */
   return 0;
 }
 ///////////////////////////////////////////////////////////////////////////
@@ -316,22 +340,74 @@ extern "C" void Tau_trace_sendmsg(int type, int destination, int length)
         RtsLayer::myNode(), type, destination, length);
 #endif /* DEBUG_PROF */
   TAU_EVENT(sendevent, length);
+#ifdef TAU_EACH_SEND
   TheMsgVolEvent()[destination]->TriggerEvent(length, RtsLayer::myThread());
+#endif /* TAU_EACH_SEND */
   TAU_TRACE_SENDMSG(type, destination, length);
 }
+///////////////////////////////////////////////////////////////////////////
+extern "C" void Tau_trace_recvmsg(int type, int source, int length)
+{
+  TAU_EVENT(recvevent, length);
+  TAU_TRACE_RECVMSG(type, source, length);
+}
 
+extern "C" void Tau_bcast_data(int data)
+{
+  TAU_EVENT(bcastevent, data);
+}
+
+extern "C" void Tau_reduce_data(int data)
+{
+  TAU_EVENT(reduceevent, data);
+}
+
+extern "C" void Tau_alltoall_data(int data)
+{
+  TAU_EVENT(alltoallevent, data);
+}
+
+extern "C" void Tau_scatter_data(int data)
+{
+  TAU_EVENT(scatterevent, data);
+}
+
+extern "C" void Tau_gather_data(int data)
+{
+  TAU_EVENT(gatherevent, data);
+}
+
+extern "C" void Tau_allgather_data(int data)
+{
+  TAU_EVENT(allgatherevent, data);
+}
+
+extern "C" void Tau_allreduce_data(int data)
+{
+  TAU_EVENT(gatherevent, data);
+}
+
+extern "C" void Tau_scan_data(int data)
+{
+  TAU_EVENT(scanevent, data);
+}
+
+extern "C" void Tau_reducescatter_data(int data)
+{
+  TAU_EVENT(reducescatterevent, data);
+}
 #else /* !TAU_MPI */
 ///////////////////////////////////////////////////////////////////////////
 extern "C" void Tau_trace_sendmsg(int type, int destination, int length)
 {
   TAU_TRACE_SENDMSG(type, destination, length);
 }
-#endif /* TAU_MPI */
 ///////////////////////////////////////////////////////////////////////////
 extern "C" void Tau_trace_recvmsg(int type, int source, int length)
 {
   TAU_TRACE_RECVMSG(type, source, length);
 }
+#endif /* TAU_MPI */
 
 ///////////////////////////////////////////////////////////////////////////
 // User Defined Events 
@@ -425,11 +501,88 @@ extern "C" void Tau_profile_c_timer(void **ptr, char *fname, char *type, TauGrou
   return;
 }
 
+/* We need a routine that will create a top level parent profiler and give
+ * it a dummy name for the application, if just the MPI wrapper interposition
+ * library is used without any instrumentation in main */
+extern "C" void Tau_create_top_level_timer_if_necessary(void)
+{
+  int tid = RtsLayer::myThread();
+  FunctionInfo *ptr;
+  if (Profiler::CurrentProfiler[tid] == NULL) 
+  {
+    ptr = (FunctionInfo *) Tau_get_profiler(".TAU application", " ", TAU_DEFAULT, "TAU_DEFAULT");
+    if (ptr)
+    {
+      Tau_start_timer(ptr);
+    }
+  }
+}
 
+extern "C" void Tau_stop_top_level_timer_if_necessary(void)
+{
+  int tid = RtsLayer::myThread();
+  if (Profiler::CurrentProfiler[tid] && 
+      Profiler::CurrentProfiler[tid]->ParentProfiler == NULL && 
+      strcmp(Profiler::CurrentProfiler[tid]->ThisFunction->GetName(), ".TAU application") == 0)
+  {
+    DEBUGPROFMSG("Found top level .TAU application timer"<<endl;);  
+    TAU_GLOBAL_TIMER_STOP();
+  }
+}
+
+    
+///////////////////////////////////////////////////////////////////////////
+extern "C" void Tau_track_memory(void)
+{
+  TAU_TRACK_MEMORY();
+} 
+
+///////////////////////////////////////////////////////////////////////////
+extern "C" void Tau_track_memory_here(void)
+{
+  TAU_TRACK_MEMORY_HERE();
+} 
+
+///////////////////////////////////////////////////////////////////////////
+extern "C" void Tau_track_muse_events(void)
+{
+  TAU_TRACK_MUSE_EVENTS();
+} 
+
+///////////////////////////////////////////////////////////////////////////
+extern "C" void Tau_enable_tracking_memory(void)
+{
+  TAU_ENABLE_TRACKING_MEMORY();
+} 
+
+///////////////////////////////////////////////////////////////////////////
+extern "C" void Tau_disable_tracking_memory(void)
+{
+  TAU_DISABLE_TRACKING_MEMORY();
+} 
+
+///////////////////////////////////////////////////////////////////////////
+extern "C" void Tau_enable_tracking_muse_events(void)
+{
+  TAU_ENABLE_TRACKING_MUSE_EVENTS();
+} 
+
+
+///////////////////////////////////////////////////////////////////////////
+extern "C" void Tau_disable_tracking_muse_events(void)
+{
+  TAU_DISABLE_TRACKING_MUSE_EVENTS();
+} 
+
+///////////////////////////////////////////////////////////////////////////
+extern "C" void Tau_set_interrupt_interval(int value)
+{
+  TAU_SET_INTERRUPT_INTERVAL(value);
+} 
 
 /***************************************************************************
  * $RCSfile: TauCAPI.cpp,v $   $Author: sameer $
- * $Revision: 1.34 $   $Date: 2003/07/22 17:39:52 $
- * VERSION: $Id: TauCAPI.cpp,v 1.34 2003/07/22 17:39:52 sameer Exp $
+ * $Revision: 1.42 $   $Date: 2004/06/10 18:07:27 $
+ * VERSION: $Id: TauCAPI.cpp,v 1.42 2004/06/10 18:07:27 sameer Exp $
  ***************************************************************************/
 
