@@ -1,5 +1,5 @@
 /* 
-	ThreadDataWindow.java
+	MeanDataWindow.java
 
 	Title:			jRacy
 	Author:			Robert Bell
@@ -15,7 +15,7 @@ import javax.swing.*;
 import javax.swing.border.*;
 import javax.swing.event.*;
 
-public class MeanDataWindow extends JFrame implements ActionListener, Observer
+public class MeanDataWindow extends JFrame implements ActionListener, MenuListener, Observer, ChangeListener
 {
 	
 	public MeanDataWindow()
@@ -29,18 +29,20 @@ public class MeanDataWindow extends JFrame implements ActionListener, Observer
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "MDW01");
+			jRacy.systemError(e, null, "MDW01");
 		}
 	}
 	
-	public MeanDataWindow(StaticMainWindowData inSMWData)
+	public MeanDataWindow(Trial inTrial, StaticMainWindowData inSMWData)
 	{
 		try{
+			
+			trial = inTrial;
+			sMWData = inSMWData;
+			
 			setLocation(new java.awt.Point(300, 200));
 			setSize(new java.awt.Dimension(700, 450));
 			
-			sMWData = inSMWData;
-			
 			currentSMWMeanData = null;
 			
 			inclusive = false;
@@ -48,7 +50,7 @@ public class MeanDataWindow extends JFrame implements ActionListener, Observer
 	 		unitsString = "milliseconds";
 			
 			//Now set the title.
-			this.setTitle("Mean Data Window");
+			this.setTitle("Mean Data Window: " + trial.getProfilePathName());
 			
 			//Add some window listener code
 				addWindowListener(new java.awt.event.WindowAdapter() {
@@ -65,14 +67,14 @@ public class MeanDataWindow extends JFrame implements ActionListener, Observer
 				//where they are.
 				jRacy.helpWindow.writeText("This is the mean data window");
 				jRacy.helpWindow.writeText("");
-				jRacy.helpWindow.writeText("This window shows you the mean values for all functions.");
+				jRacy.helpWindow.writeText("This window shows you the mean values for all mappings.");
 				jRacy.helpWindow.writeText("");
 				jRacy.helpWindow.writeText("Use the options menu to select different ways of displaying the data.");
 				jRacy.helpWindow.writeText("");
-				jRacy.helpWindow.writeText("Right click on any function within this window to bring up a popup");
+				jRacy.helpWindow.writeText("Right click on any mapping within this window to bring up a popup");
 				jRacy.helpWindow.writeText("menu. In this menu you can change or reset the default colour");
-				jRacy.helpWindow.writeText("for the function, or to show more details about the function.");
-				jRacy.helpWindow.writeText("You can also left click any function to hightlight it in the system.");
+				jRacy.helpWindow.writeText("for the mapping, or to show more details about the mapping.");
+				jRacy.helpWindow.writeText("You can also left click any mapping to hightlight it in the system.");
 			}
 				
 			//Sort the local data.
@@ -108,71 +110,63 @@ public class MeanDataWindow extends JFrame implements ActionListener, Observer
 			//Options menu.
 			//******************************
 			JMenu optionsMenu = new JMenu("Options");
+			optionsMenu.addMenuListener(this);
 			
-			//Add a submenu.
+			//***********
+			//Submenu.
 			JMenu sortMenu = new JMenu("Sort by ...");
+			
 			sortGroup = new ButtonGroup();
-			
-			functionIDButton = new JRadioButtonMenuItem("function ID", false);
-			//Add a listener for this radio button.
-			functionIDButton.addActionListener(this);
-			
-			nameButton = new JRadioButtonMenuItem("name", false);
-			//Add a listener for this radio button.
+			//Add listeners
 			nameButton.addActionListener(this);
-			
-			millisecondButton = new JRadioButtonMenuItem("millisecond", true);
-			//Add a listener for this radio button.
 			millisecondButton.addActionListener(this);
-			
-			sortGroup.add(functionIDButton);
 			sortGroup.add(nameButton);
 			sortGroup.add(millisecondButton);
-			
-			sortMenu.add(functionIDButton);
 			sortMenu.add(nameButton);
 			sortMenu.add(millisecondButton);
-			optionsMenu.add(sortMenu);
-			//End Submenu.
 			
-			//Add a submenu.
-			JMenu sortOrderMenu = new JMenu("Sort Order");
+			
 			sortOrderGroup = new ButtonGroup();
-			
-			ascendingButton = new JRadioButtonMenuItem("Ascending", false);
-			//Add a listener for this radio button.
+			//Add listeners
 			ascendingButton.addActionListener(this);
-			
-			descendingButton = new JRadioButtonMenuItem("Descending", true);
-			//Add a listener for this radio button.
 			descendingButton.addActionListener(this);
-			
 			sortOrderGroup.add(ascendingButton);
 			sortOrderGroup.add(descendingButton);
+			sortMenu.add(ascendingButton);
+			sortMenu.add(descendingButton);
 			
-			sortOrderMenu.add(ascendingButton);
-			sortOrderMenu.add(descendingButton);
-			optionsMenu.add(sortOrderMenu);
-			//End Submenu.
+			sortMenu.insertSeparator(2);
+			
+			optionsMenu.add(sortMenu);
+			//End - Submenu.
+			//***********
 			
 			//Add a submenu.
-			JMenu inclusiveExclusiveMenu = new JMenu("Select Inclusive or Exclusive");
-			inclusiveExclusiveGroup = new ButtonGroup();
-			inclusiveRadioButton = new JRadioButtonMenuItem("Inclusive", false);
-			//Add a listener for this radio button.
+			JMenu metricMenu = new JMenu("Select Metric");
+			metricGroup = new ButtonGroup();
+			
+			//Add listeners
 			inclusiveRadioButton.addActionListener(this);
-			exclusiveRadioButton = new JRadioButtonMenuItem("Exclusive", true);
-			//Add a listener for this radio button.
 			exclusiveRadioButton.addActionListener(this);
-			inclusiveExclusiveGroup.add(inclusiveRadioButton);
-			inclusiveExclusiveGroup.add(exclusiveRadioButton);
-			inclusiveExclusiveMenu.add(inclusiveRadioButton);
-			inclusiveExclusiveMenu.add(exclusiveRadioButton);
-			optionsMenu.add(inclusiveExclusiveMenu);
+			numOfCallsRadioButton.addActionListener(this);
+			numOfSubRoutinesRadioButton.addActionListener(this);
+			userSecPerCallRadioButton.addActionListener(this);
+			
+			metricGroup.add(inclusiveRadioButton);
+			metricGroup.add(exclusiveRadioButton);
+			metricGroup.add(numOfCallsRadioButton);
+			metricGroup.add(numOfSubRoutinesRadioButton);
+			metricGroup.add(userSecPerCallRadioButton);
+			metricMenu.add(inclusiveRadioButton);
+			metricMenu.add(exclusiveRadioButton);
+			metricMenu.add(numOfCallsRadioButton);
+			metricMenu.add(numOfSubRoutinesRadioButton);
+			metricMenu.add(userSecPerCallRadioButton);
+			optionsMenu.add(metricMenu);
 			//End Submenu.
 			
 			//Add a submenu.
-			JMenu valuePercentMenu = new JMenu("Select Value or Percent");
+			valuePercentMenu = new JMenu("Select Value or Percent");
 			valuePercentGroup = new ButtonGroup();
 			
 			percentButton = new JRadioButtonMenuItem("Percent", true);
@@ -192,7 +186,7 @@ public class MeanDataWindow extends JFrame implements ActionListener, Observer
 			//End Submenu.
 			
 			//Add a submenu.
-			JMenu unitsMenu = new JMenu("Select Units");
+			unitsMenu = new JMenu("Select Units");
 			unitsGroup = new ButtonGroup();
 			
 			secondsButton = new JRadioButtonMenuItem("Seconds", false);
@@ -217,6 +211,11 @@ public class MeanDataWindow extends JFrame implements ActionListener, Observer
 			optionsMenu.add(unitsMenu);
 			//End Submenu.
 			
+			displaySlidersButton = new JRadioButtonMenuItem("Display Sliders", false);
+			//Add a listener for this radio button.
+			displaySlidersButton.addActionListener(this);
+			optionsMenu.add(displaySlidersButton);
+			
 			//******************************
 			//End - Options menu.
 			//******************************
@@ -226,13 +225,24 @@ public class MeanDataWindow extends JFrame implements ActionListener, Observer
 			//Window menu.
 			//******************************
 			JMenu windowsMenu = new JMenu("Windows");
+			windowsMenu.addMenuListener(this);
 			
 			//Add a submenu.
-			JMenuItem functionLedgerItem = new JMenuItem("Show Function Ledger");
-			functionLedgerItem.addActionListener(this);
-			windowsMenu.add(functionLedgerItem);
+			JMenuItem mappingLedgerItem = new JMenuItem("Show Function Ledger");
+			mappingLedgerItem.addActionListener(this);
+			windowsMenu.add(mappingLedgerItem);
 			
 			//Add a submenu.
+			mappingGroupLedgerItem = new JMenuItem("Show Group Ledger");
+			mappingGroupLedgerItem.addActionListener(this);
+			windowsMenu.add(mappingGroupLedgerItem);
+			
+			//Add a submenu.
+			userEventLedgerItem = new JMenuItem("Show User Event Ledger");
+			userEventLedgerItem.addActionListener(this);
+			windowsMenu.add(userEventLedgerItem);
+			
+			//Add listeners
 			JMenuItem closeAllSubwindowsItem = new JMenuItem("Close All Sub-Windows");
 			closeAllSubwindowsItem.addActionListener(this);
 			windowsMenu.add(closeAllSubwindowsItem);
@@ -276,10 +286,10 @@ public class MeanDataWindow extends JFrame implements ActionListener, Observer
 			//Create and add the componants.
 			//******************************
 			//Setting up the layout system for the main window.
-			Container contentPane = getContentPane();
-			GridBagLayout gbl = new GridBagLayout();
+			contentPane = getContentPane();
+			gbl = new GridBagLayout();
 			contentPane.setLayout(gbl);
-			GridBagConstraints gbc = new GridBagConstraints();
+			gbc = new GridBagConstraints();
 			gbc.insets = new Insets(5, 5, 5, 5);
 			
 			//Create some borders.
@@ -290,47 +300,38 @@ public class MeanDataWindow extends JFrame implements ActionListener, Observer
 			//**********
 			//Panel and ScrollPane definition.
 			//**********
-			meanDataWindowPanelRef = new MeanDataWindowPanel(this, sMWData);
+			meanDataWindowPanelRef = new MeanDataWindowPanel(trial, this, sMWData);
 			
 			//**********
 			//End - Panel and ScrollPane definition.
 			//**********
 			
 			//The scroll panes into which the list shall be placed.
-			JScrollPane meanDataWindowPanelScrollPane = new JScrollPane(meanDataWindowPanelRef);
+			meanDataWindowPanelScrollPane = new JScrollPane(meanDataWindowPanelRef);
 			meanDataWindowPanelScrollPane.setBorder(mainloweredbev);
 			meanDataWindowPanelScrollPane.setPreferredSize(new Dimension(500, 450));
 			
+			//Do the slider stuff, but don't add.  By default, sliders are off.
+			String sliderMultipleStrings[] = {"1.00", "0.75", "0.50", "0.25", "0.10"};
+			sliderMultiple = new JComboBox(sliderMultipleStrings);
+			sliderMultiple.addActionListener(this);
+			
 			barLengthSlider.setPaintTicks(true);
 			barLengthSlider.setMajorTickSpacing(5);
 			barLengthSlider.setMinorTickSpacing(1);
 			barLengthSlider.setPaintLabels(true);
 			barLengthSlider.setSnapToTicks(true);
-			barLengthSlider.addChangeListener(meanDataWindowPanelRef);
-			
-			
-			//Now add the componants to the main screen.
-			gbc.fill = GridBagConstraints.NONE;
-			gbc.anchor = GridBagConstraints.EAST;
-			gbc.weightx = 0.475;
-			gbc.weighty = 0.01;
-			addCompItem(barLengthLabel, gbc, 0, 0, 1, 1);
-			
-			gbc.fill = GridBagConstraints.HORIZONTAL;
-			gbc.anchor = GridBagConstraints.WEST;
-			gbc.weightx = 0.475;
-			gbc.weighty = 0.01;
-			addCompItem(barLengthSlider, gbc, 1, 0, 1, 1);
+			barLengthSlider.addChangeListener(this);
 			
 			gbc.fill = GridBagConstraints.BOTH;
 			gbc.anchor = GridBagConstraints.CENTER;
 			gbc.weightx = 0.95;
 			gbc.weighty = 0.98;
-			addCompItem(meanDataWindowPanelScrollPane, gbc, 0, 1, 2, 1);
+			addCompItem(meanDataWindowPanelScrollPane, gbc, 0, 0, 1, 1);
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "MDW02");
+			jRacy.systemError(e, null, "MDW02");
 		}
 			
 		
@@ -360,23 +361,11 @@ public class MeanDataWindow extends JFrame implements ActionListener, Observer
 					dispose();
 					System.exit(0);
 				}
-				else if(arg.equals("function ID"))
-				{
-					if(functionIDButton.isSelected())
-					{
-						sortByFunctionID = true;
-						sortByName = false;
-						sortByMillisecond = false;
-						sortLocalData();
-						//Call repaint.
-						meanDataWindowPanelRef.repaint();
-					}
-				}
 				else if(arg.equals("name"))
 				{
 					if(nameButton.isSelected())
 					{
-						sortByFunctionID = false;
+						sortByMappingID = false;
 						sortByName = true;
 						sortByMillisecond = false;
 						sortLocalData();
@@ -384,11 +373,11 @@ public class MeanDataWindow extends JFrame implements ActionListener, Observer
 						meanDataWindowPanelRef.repaint();
 					}
 				}
-				else if(arg.equals("millisecond"))	//Note the difference in case from the millisecond option below.
+				else if(arg.equals("Selected Metric"))	//Note the difference in case from the millisecond option below.
 				{
 					if(millisecondButton.isSelected())
 					{
-						sortByFunctionID = false;
+						sortByMappingID = false;
 						sortByName = false;
 						sortByMillisecond = true;
 						sortLocalData();
@@ -401,6 +390,7 @@ public class MeanDataWindow extends JFrame implements ActionListener, Observer
 					if(descendingButton.isSelected())
 					{
 						descendingOrder = true;
+						sortLocalData();
 						meanDataWindowPanelRef.repaint();
 					}
 				}
@@ -417,7 +407,8 @@ public class MeanDataWindow extends JFrame implements ActionListener, Observer
 				{
 					if(inclusiveRadioButton.isSelected())
 					{
-						inclusive = true;
+						metric = "Inclusive";
+						//Sort the local data.
 						sortLocalData();
 						//Call repaint.
 						meanDataWindowPanelRef.repaint();
@@ -427,7 +418,41 @@ public class MeanDataWindow extends JFrame implements ActionListener, Observer
 				{
 					if(exclusiveRadioButton.isSelected())
 					{
-						inclusive = false;
+						metric = "Exclusive";
+						//Sort the local data.
+						sortLocalData();
+						//Call repaint.
+						meanDataWindowPanelRef.repaint();
+					}
+				}
+				else if(arg.equals("Number of Calls"))
+				{
+					if(numOfCallsRadioButton.isSelected())
+					{
+						metric = "Number of Calls";
+						//Sort the local data.
+						sortLocalData();
+						//Call repaint.
+						meanDataWindowPanelRef.repaint();
+					}
+				}
+				else if(arg.equals("Number of Subroutines"))
+				{
+					if(numOfSubRoutinesRadioButton.isSelected())
+					{
+						metric = "Number of Subroutines";
+						//Sort the local data.
+						sortLocalData();
+						//Call repaint.
+						meanDataWindowPanelRef.repaint();
+					}
+				}
+				else if(arg.equals("Per Call Value"))
+				{
+					if(userSecPerCallRadioButton.isSelected())
+					{
+						metric = "Per Call Value";
+						//Sort the local data.
 						sortLocalData();
 						//Call repaint.
 						meanDataWindowPanelRef.repaint();
@@ -479,16 +504,39 @@ public class MeanDataWindow extends JFrame implements ActionListener, Observer
 						meanDataWindowPanelRef.repaint();
 					}
 				}
+				else if(arg.equals("Display Sliders"))
+				{
+					if(displaySlidersButton.isSelected())
+					{	
+						displaySiders(true);
+					}
+					else
+					{
+						displaySiders(false);
+					}
+				}
 				else if(arg.equals("Show Function Ledger"))
 				{
 					//In order to be in this window, I must have loaded the data. So,
-					//just show the function ledger window.
-					(jRacy.staticSystemData.getGlobalMapping()).displayFunctionLedger();
+					//just show the mapping ledger window.
+					(trial.getGlobalMapping()).displayMappingLedger(0);
+				}
+				else if(arg.equals("Show Group Ledger"))
+				{
+					//In order to be in this window, I must have loaded the data. So,
+					//just show the mapping ledger window.
+					(trial.getGlobalMapping()).displayMappingLedger(1);
+				}
+				else if(arg.equals("Show User Event Ledger"))
+				{
+					//In order to be in this window, I must have loaded the data. So,
+					//just show the mapping ledger window.
+					(trial.getGlobalMapping()).displayMappingLedger(2);
 				}
 				else if(arg.equals("Close All Sub-Windows"))
 				{
 					//Close the all subwindows.
-					jRacy.systemEvents.updateRegisteredObjects("subWindowCloseEvent");
+					trial.getSystemEvents().updateRegisteredObjects("subWindowCloseEvent");
 				}
 				else if(arg.equals("About Racy"))
 				{
@@ -503,23 +551,96 @@ public class MeanDataWindow extends JFrame implements ActionListener, Observer
 					//where they are.
 					jRacy.helpWindow.writeText("This is the mean data window");
 					jRacy.helpWindow.writeText("");
-					jRacy.helpWindow.writeText("This window shows you the mean values for all functions.");
+					jRacy.helpWindow.writeText("This window shows you the mean values for all mappings.");
 					jRacy.helpWindow.writeText("");
 					jRacy.helpWindow.writeText("Use the options menu to select different ways of displaying the data.");
 					jRacy.helpWindow.writeText("");
-					jRacy.helpWindow.writeText("Right click on any function within this window to bring up a popup");
+					jRacy.helpWindow.writeText("Right click on any mapping within this window to bring up a popup");
 					jRacy.helpWindow.writeText("menu. In this menu you can change or reset the default colour");
-					jRacy.helpWindow.writeText("for the function, or to show more details about the function.");
-					jRacy.helpWindow.writeText("You can also left click any function to hightlight it in the system.");
+					jRacy.helpWindow.writeText("for the mapping, or to show more details about the mapping.");
+					jRacy.helpWindow.writeText("You can also left click any mapping to hightlight it in the system.");
 				}
 			}
+			else if(EventSrc == sliderMultiple)
+			{
+				meanDataWindowPanelRef.changeInMultiples();
+			}
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "MDW03");
+			jRacy.systemError(e, null, "MDW03");
 		}
 	}
 	
+	//******************************
+	//Change listener code.
+	//******************************
+	public void stateChanged(ChangeEvent event)
+	{
+		meanDataWindowPanelRef.changeInMultiples();
+	}
+	//******************************
+	//End - Change listener code.
+	//******************************
+	
+	//******************************
+	//MenuListener code.
+	//******************************
+	public void menuSelected(MenuEvent evt)
+	{
+		try
+		{
+			String trialName = trial.getCounterName();
+			trialName = trialName.toUpperCase();
+			boolean isDefault = false;
+			boolean isTimeMetric = false;
+			
+			if(trialName.equals("DEFAULT"))	
+				isDefault = true;
+			else if(trialName.indexOf("TIME") != -1)
+				isTimeMetric = true;
+			
+			if(trial.groupNamesPresent())
+				mappingGroupLedgerItem.setEnabled(true);
+			else
+				mappingGroupLedgerItem.setEnabled(false);
+				
+			if(trial.userEventsPresent())
+				userEventLedgerItem.setEnabled(true);
+			else
+				userEventLedgerItem.setEnabled(false);
+				
+				
+			
+			if((metric.equals("Number of Calls")) || (metric.equals("Number of Subroutines")) || (metric.equals("Per Call Value"))){
+				valuePercentMenu.setEnabled(false);
+				unitsMenu.setEnabled(false);}
+			else if(percent){
+				valuePercentMenu.setEnabled(true);
+				unitsMenu.setEnabled(false);}
+			else{
+				valuePercentMenu.setEnabled(true);
+				unitsMenu.setEnabled(true);}
+		}
+		catch(Exception e)
+		{
+			jRacy.systemError(e, null, "MDW04");
+		}
+		
+	}
+	
+	public void menuDeselected(MenuEvent evt)
+	{
+	}
+	
+	public void menuCanceled(MenuEvent evt)
+	{
+	}
+	
+	//******************************
+	//End - MenuListener code.
+	//******************************
+	
 	//Observer functions.
 	public void update(Observable o, Object arg)
 	{
@@ -535,6 +656,11 @@ public class MeanDataWindow extends JFrame implements ActionListener, Observer
 				//Just need to call a repaint on the ThreadDataWindowPanel.
 				meanDataWindowPanelRef.repaint();
 			}
+			else if(tmpString.equals("dataEvent"))
+			{
+				sortLocalData();
+				meanDataWindowPanelRef.repaint();
+			}
 			else if(tmpString.equals("subWindowCloseEvent"))
 			{
 				closeThisWindow();
@@ -542,71 +668,134 @@ public class MeanDataWindow extends JFrame implements ActionListener, Observer
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "MDW04");
+			jRacy.systemError(e, null, "MDW05");
 		}
 	}
 	
-	//Updates the sorted lists after a change of sorting method takes place.
 	private void sortLocalData()
-	{
+	{	
 		try
 		{
-			if(sortByFunctionID)
+			if(sortByMappingID)
 			{
-				if(inclusive)
+				if(metric.equals("Inclusive"))
 				{
 					if(descendingOrder)
 						currentSMWMeanData = sMWData.getSMWMeanData("FIdDI");
 					else
 						currentSMWMeanData = sMWData.getSMWMeanData("FIdAI");
 				}
-				else
+				else if(metric.equals("Exclusive"))
 				{
 					if(descendingOrder)
 						currentSMWMeanData = sMWData.getSMWMeanData("FIdDE");
 					else
 						currentSMWMeanData = sMWData.getSMWMeanData("FIdAE");
 				}
+				else if(metric.equals("Number of Calls"))
+				{
+					if(descendingOrder)
+						currentSMWMeanData = sMWData.getSMWMeanData("FIdDNC");
+					else
+						currentSMWMeanData = sMWData.getSMWMeanData("FIdANC");
+				}
+				else if(metric.equals("Number of Subroutines"))
+				{
+					if(descendingOrder)
+						currentSMWMeanData = sMWData.getSMWMeanData("FIdDNS");
+					else
+						currentSMWMeanData = sMWData.getSMWMeanData("FIdANS");
+				}
+				else if(metric.equals("Per Call Value"))
+				{
+					if(descendingOrder)
+						currentSMWMeanData = sMWData.getSMWMeanData("FIdDUS");
+					else
+						currentSMWMeanData = sMWData.getSMWMeanData("FIdAUS");
+				}
 			}
 			else if(sortByName)
 			{
 				
-				if(inclusive)
+				if(metric.equals("Inclusive"))
 				{
 					if(descendingOrder)
 						currentSMWMeanData = sMWData.getSMWMeanData("NDI");
 					else
 						currentSMWMeanData = sMWData.getSMWMeanData("NAI");
 				}
-				else
+				else if(metric.equals("Exclusive"))
 				{
 					if(descendingOrder)
 						currentSMWMeanData = sMWData.getSMWMeanData("NDE");
 					else
 						currentSMWMeanData = sMWData.getSMWMeanData("NAE");
 				}
+				else if(metric.equals("Number of Calls"))
+				{
+					if(descendingOrder)
+						currentSMWMeanData = sMWData.getSMWMeanData("NDNC");
+					else
+						currentSMWMeanData = sMWData.getSMWMeanData("NANC");
+				}
+				else if(metric.equals("Number of Subroutines"))
+				{
+					if(descendingOrder)
+						currentSMWMeanData = sMWData.getSMWMeanData("NDNS");
+					else
+						currentSMWMeanData = sMWData.getSMWMeanData("NANS");
+				}
+				else if(metric.equals("Per Call Value"))
+				{
+					if(descendingOrder)
+						currentSMWMeanData = sMWData.getSMWMeanData("NDUS");
+					else
+						currentSMWMeanData = sMWData.getSMWMeanData("NAUS");
+				}
 			}
 			else if(sortByMillisecond)
 			{
-				if(inclusive)
+				
+				if(metric.equals("Inclusive"))
 				{
 					if(descendingOrder)
 						currentSMWMeanData = sMWData.getSMWMeanData("MDI");
 					else
 						currentSMWMeanData = sMWData.getSMWMeanData("MAI");
 				}
-				else
+				else if(metric.equals("Exclusive"))
 				{
 					if(descendingOrder)
 						currentSMWMeanData = sMWData.getSMWMeanData("MDE");
 					else
 						currentSMWMeanData = sMWData.getSMWMeanData("MAE");
 				}
+				else if(metric.equals("Number of Calls"))
+				{
+					if(descendingOrder)
+						currentSMWMeanData = sMWData.getSMWMeanData("MDNC");
+					else
+						currentSMWMeanData = sMWData.getSMWMeanData("MANC");
+				}
+				else if(metric.equals("Number of Subroutines"))
+				{
+					if(descendingOrder)
+						currentSMWMeanData = sMWData.getSMWMeanData("MDNS");
+					else
+						currentSMWMeanData = sMWData.getSMWMeanData("MANS");
+				}
+				else if(metric.equals("Per Call Value"))
+				{
+					if(descendingOrder)
+						currentSMWMeanData = sMWData.getSMWMeanData("MDUS");
+					else
+						currentSMWMeanData = sMWData.getSMWMeanData("MAUS");
+				}
 			}
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "MDW05");
+			jRacy.systemError(e, null, "MDW06");
 		}
 	}
 	
@@ -622,6 +811,10 @@ public class MeanDataWindow extends JFrame implements ActionListener, Observer
 		return inclusive;
 	}
 	
+	public String getMetric(){
+		return metric;
+	}
+	
 	public boolean isPercent()
 	{
 		return percent;
@@ -634,7 +827,93 @@ public class MeanDataWindow extends JFrame implements ActionListener, Observer
 	
 	public int getSliderValue()
 	{
-		return barLengthSlider.getValue();
+		int tmpInt = -1;
+		
+		try
+		{
+			tmpInt = barLengthSlider.getValue();
+		}
+		catch(Exception e)
+		{
+			jRacy.systemError(e, null, "MDW07");
+		}
+		
+		return tmpInt;
+	}
+	
+	public double getSliderMultiple()
+	{
+		String tmpString = null;
+		try
+		{
+			tmpString = (String) sliderMultiple.getSelectedItem();
+			return Double.parseDouble(tmpString);
+		}
+		catch(Exception e)
+		{
+			jRacy.systemError(e, null, "MDW08");
+		}
+		
+		return 0;
+	}
+	
+	private void displaySiders(boolean displaySliders)
+	{
+		if(displaySliders)
+		{
+			//Since the menu option is a toggle, the only component that needs to be
+			//removed is that scrollPane.  We then add back in with new parameters.
+			//This might not be required as it seems to adjust well if left in, but just
+			//to be sure.
+			contentPane.remove(meanDataWindowPanelScrollPane);
+			
+			gbc.fill = GridBagConstraints.NONE;
+			gbc.anchor = GridBagConstraints.EAST;
+			gbc.weightx = 0.10;
+			gbc.weighty = 0.01;
+			addCompItem(sliderMultipleLabel, gbc, 0, 0, 1, 1);
+			
+			gbc.fill = GridBagConstraints.NONE;
+			gbc.anchor = GridBagConstraints.WEST;
+			gbc.weightx = 0.10;
+			gbc.weighty = 0.01;
+			addCompItem(sliderMultiple, gbc, 1, 0, 1, 1);
+			
+			gbc.fill = GridBagConstraints.NONE;
+			gbc.anchor = GridBagConstraints.EAST;
+			gbc.weightx = 0.10;
+			gbc.weighty = 0.01;
+			addCompItem(barLengthLabel, gbc, 2, 0, 1, 1);
+			
+			gbc.fill = GridBagConstraints.HORIZONTAL;
+			gbc.anchor = GridBagConstraints.WEST;
+			gbc.weightx = 0.70;
+			gbc.weighty = 0.01;
+			addCompItem(barLengthSlider, gbc, 3, 0, 1, 1);
+			
+			gbc.fill = GridBagConstraints.BOTH;
+			gbc.anchor = GridBagConstraints.CENTER;
+			gbc.weightx = 0.95;
+			gbc.weighty = 0.98;
+			addCompItem(meanDataWindowPanelScrollPane, gbc, 0, 1, 4, 1);
+		}
+		else
+		{
+			contentPane.remove(sliderMultipleLabel);
+			contentPane.remove(sliderMultiple);
+			contentPane.remove(barLengthLabel);
+			contentPane.remove(barLengthSlider);
+			contentPane.remove(meanDataWindowPanelScrollPane);
+			
+			gbc.fill = GridBagConstraints.BOTH;
+			gbc.anchor = GridBagConstraints.CENTER;
+			gbc.weightx = 0.95;
+			gbc.weighty = 0.98;
+			addCompItem(meanDataWindowPanelScrollPane, gbc, 0, 0, 1, 1);
+		}
+		
+		//Now call validate so that these componant changes are displayed.
+		validate();
 	}
 	
 	//Helper functions.
@@ -650,7 +929,7 @@ public class MeanDataWindow extends JFrame implements ActionListener, Observer
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "MDW06");
+			jRacy.systemError(e, null, "MDW09");
 		}
 	}
 	
@@ -673,34 +952,40 @@ public class MeanDataWindow extends JFrame implements ActionListener, Observer
 			}
 			
 			setVisible(false);
-			jRacy.systemEvents.deleteObserver(this);
+			trial.getSystemEvents().deleteObserver(this);
 			dispose();
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "MDW07");
+			jRacy.systemError(e, null, "MDW10");
 		}
 	}
 	
 	//******************************
 	//Instance data.
 	//******************************
+	private JMenu unitsMenu;
+	private JMenu valuePercentMenu;
+	private JMenuItem mappingGroupLedgerItem;
+	private JMenuItem userEventLedgerItem;
 	
 	private ButtonGroup sortGroup;
 	private ButtonGroup sortOrderGroup;
-	private ButtonGroup inclusiveExclusiveGroup;
+	private ButtonGroup metricGroup = null;
 	private ButtonGroup valuePercentGroup;
 	private ButtonGroup unitsGroup;
 	
-	private JRadioButtonMenuItem ascendingButton;
-	private JRadioButtonMenuItem descendingButton;
+	private JRadioButtonMenuItem ascendingButton =  new JRadioButtonMenuItem("Ascending", false);
+	private JRadioButtonMenuItem descendingButton = new JRadioButtonMenuItem("Descending", true);
 	
-	private JRadioButtonMenuItem functionIDButton;
-	private JRadioButtonMenuItem nameButton;
-	private JRadioButtonMenuItem millisecondButton;
+	private JRadioButtonMenuItem nameButton = new JRadioButtonMenuItem("name", false);
+	private JRadioButtonMenuItem millisecondButton =  new JRadioButtonMenuItem("Selected Metric", true);
 	
-	private JRadioButtonMenuItem inclusiveRadioButton;
-	private JRadioButtonMenuItem exclusiveRadioButton;
+	private JRadioButtonMenuItem inclusiveRadioButton =  new JRadioButtonMenuItem("Inclusive", false);
+	private JRadioButtonMenuItem exclusiveRadioButton = new JRadioButtonMenuItem("Exclusive", true);
+	private JRadioButtonMenuItem numOfCallsRadioButton =  new JRadioButtonMenuItem("Number of Calls", false);
+	private JRadioButtonMenuItem numOfSubRoutinesRadioButton = new JRadioButtonMenuItem("Number of Subroutines", false);
+	private JRadioButtonMenuItem userSecPerCallRadioButton = new JRadioButtonMenuItem("Per Call Value", false);
 	
 	private JRadioButtonMenuItem valueButton;
 	private JRadioButtonMenuItem percentButton;
@@ -709,20 +994,33 @@ public class MeanDataWindow extends JFrame implements ActionListener, Observer
 	private JRadioButtonMenuItem millisecondsButton;
 	private JRadioButtonMenuItem microsecondsButton;
 	
-	private JRadioButtonMenuItem showZeroFunctionsItem;
+	private JRadioButtonMenuItem showZeroMappingsItem;
+	
+	private JRadioButtonMenuItem displaySlidersButton;
+	
+	private JLabel sliderMultipleLabel = new JLabel("Slider Mulitiple");
+	private JComboBox sliderMultiple;
 	
 	private JLabel barLengthLabel = new JLabel("Bar Mulitiple");
-	
 	private JSlider barLengthSlider = new JSlider(0, 40, 1);
 	
+	private Container contentPane = null;
+	private GridBagLayout gbl = null;
+	private GridBagConstraints gbc = null;
+	
+	private JScrollPane meanDataWindowPanelScrollPane;
+	
  	private MeanDataWindowPanel meanDataWindowPanelRef;
  	
- 	private StaticMainWindowData sMWData;
+ 	private Trial trial = null;
+ 	private StaticMainWindowData sMWData = null;
  	
  	//Local data.
  	Vector currentSMWMeanData = null;
  	
- 	private boolean sortByFunctionID = false;
+ 	private String metric = "Exclusive";
+ 	
+ 	private boolean sortByMappingID = false;
 	private boolean sortByName = false;
 	private boolean sortByMillisecond = true;
 	
