@@ -15,7 +15,7 @@ import javax.swing.*;
 import javax.swing.border.*;
 import javax.swing.event.*;
 
-public class MeanTotalStatWindow extends JFrame implements ActionListener, Observer 
+public class MeanTotalStatWindow extends JFrame implements ActionListener, MenuListener, Observer 
 {
 	
 	public MeanTotalStatWindow()
@@ -29,21 +29,21 @@ public class MeanTotalStatWindow extends JFrame implements ActionListener, Obser
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "MTSW01");
+			jRacy.systemError(e, null, "MTSW01");
 		}
 	}
 	
-	public MeanTotalStatWindow(StaticMainWindowData inSMWData)
+	public MeanTotalStatWindow(Trial inTrial, StaticMainWindowData inSMWData)
 	{
 		try{
+			trial = inTrial;
+			sMWData = inSMWData;
+			
 			setLocation(new java.awt.Point(0, 0));
 			setSize(new java.awt.Dimension(800, 600));
 			
 			//Now set the title.
-			this.setTitle("Mean Total Stat Window");
-			
-			
-			sMWData = inSMWData;
+			this.setTitle("Mean Total Stat Window: " + trial.getProfilePathName());
 			
 			//Add some window listener code
 				addWindowListener(new java.awt.event.WindowAdapter() {
@@ -59,55 +59,35 @@ public class MeanTotalStatWindow extends JFrame implements ActionListener, Obser
 				jRacy.helpWindow.clearText();
 				//Since the data must have been loaded.  Tell them someting about
 				//where they are.
-				jRacy.helpWindow.writeText("This is the mean total function statistics window.");
+				jRacy.helpWindow.writeText("This is the mean total mapping statistics window.");
 				jRacy.helpWindow.writeText("");
-				jRacy.helpWindow.writeText("This window shows you the mean total statistics for all functions.");
+				jRacy.helpWindow.writeText("This window shows you the mean total statistics for all mappings.");
 				jRacy.helpWindow.writeText("");
 				jRacy.helpWindow.writeText("Use the options menu to select different ways of displaying the data.");
 				jRacy.helpWindow.writeText("");
-				jRacy.helpWindow.writeText("Right click on any function within this window to bring up a popup");
+				jRacy.helpWindow.writeText("Right click on any mapping within this window to bring up a popup");
 				jRacy.helpWindow.writeText("menu. In this menu you can change or reset the default colour");
-				jRacy.helpWindow.writeText("for the function, or to show more details about the function.");
-				jRacy.helpWindow.writeText("You can also left click any function to hightlight it in the system.");
+				jRacy.helpWindow.writeText("for the mapping, or to show more details about the mapping.");
+				jRacy.helpWindow.writeText("You can also left click any mapping to hightlight it in the system.");
 			}
 			
 			//******************************
 			//Code to generate the menus.
 			//******************************
-			
-			//Generic Duke Stuff.
-			Icon dukeStandingSmall = new ImageIcon("Duke_Standing_Small.gif");
-			Icon dukeWavingSmall = new ImageIcon("Duke_Waving_Small.gif");
-			JRadioButton menubarDuke = new JRadioButton(dukeStandingSmall);
-			JRadioButton fileMenuDuke = new JRadioButton(dukeStandingSmall);
-			JRadioButton optionsMenuDuke = new JRadioButton(dukeStandingSmall);
-			JRadioButton windowsMenuDuke = new JRadioButton(dukeStandingSmall);
-			JRadioButton helpMenuDuke = new JRadioButton(dukeStandingSmall);
-			menubarDuke.setRolloverIcon(dukeWavingSmall);
-			fileMenuDuke.setRolloverIcon(dukeWavingSmall);
-			optionsMenuDuke.setRolloverIcon(dukeWavingSmall);
-			windowsMenuDuke.setRolloverIcon(dukeWavingSmall);
-			helpMenuDuke.setRolloverIcon(dukeWavingSmall);
-			
-			
 			JMenuBar mainMenu = new JMenuBar();
 			
-			//Duke placement.
-			mainMenu.add(menubarDuke);
-			
 			//******************************
 			//File menu.
 			//******************************
 			JMenu fileMenu = new JMenu("File");
-			fileMenu.add(fileMenuDuke);
 			
 			//Add a menu item.
-			JMenuItem closeItem = new JMenuItem("Close This Window", dukeWavingSmall);
+			JMenuItem closeItem = new JMenuItem("Close This Window");
 			closeItem.addActionListener(this);
 			fileMenu.add(closeItem);
 			
 			//Add a menu item.
-			JMenuItem exitItem = new JMenuItem("Exit Racy!", dukeWavingSmall);
+			JMenuItem exitItem = new JMenuItem("Exit Racy!");
 			exitItem.addActionListener(this);
 			fileMenu.add(exitItem);
 			//******************************
@@ -118,15 +98,14 @@ public class MeanTotalStatWindow extends JFrame implements ActionListener, Obser
 			//Options menu.
 			//******************************
 			JMenu optionsMenu = new JMenu("Options");
-			optionsMenu.add(optionsMenuDuke);
 			
 			//Add a submenu.
 			JMenu sortMenu = new JMenu("Sort by ...");
 			sortGroup = new ButtonGroup();
 			
-			functionIDButton = new JRadioButtonMenuItem("function ID", false);
+			mappingIDButton = new JRadioButtonMenuItem("function ID", false);
 			//Add a listener for this radio button.
-			functionIDButton.addActionListener(this);
+			mappingIDButton.addActionListener(this);
 			
 			nameButton = new JRadioButtonMenuItem("name", false);
 			//Add a listener for this radio button.
@@ -136,11 +115,11 @@ public class MeanTotalStatWindow extends JFrame implements ActionListener, Obser
 			//Add a listener for this radio button.
 			millisecondButton.addActionListener(this);
 			
-			sortGroup.add(functionIDButton);
+			sortGroup.add(mappingIDButton);
 			sortGroup.add(nameButton);
 			sortGroup.add(millisecondButton);
 			
-			sortMenu.add(functionIDButton);
+			sortMenu.add(mappingIDButton);
 			sortMenu.add(nameButton);
 			sortMenu.add(millisecondButton);
 			optionsMenu.add(sortMenu);
@@ -167,19 +146,27 @@ public class MeanTotalStatWindow extends JFrame implements ActionListener, Obser
 			//End Submenu.
 			
 			//Add a submenu.
-			JMenu inclusiveExclusiveMenu = new JMenu("Select Inclusive or Exclusive");
-			inclusiveExclusiveGroup = new ButtonGroup();
-			inclusiveRadioButton = new JRadioButtonMenuItem("Inclusive", false);
-			//Add a listener for this radio button.
+			JMenu metricMenu = new JMenu("Select Metric");
+			metricGroup = new ButtonGroup();
+			
+			//Add listeners
 			inclusiveRadioButton.addActionListener(this);
-			exclusiveRadioButton = new JRadioButtonMenuItem("Exclusive", true);
-			//Add a listener for this radio button.
 			exclusiveRadioButton.addActionListener(this);
-			inclusiveExclusiveGroup.add(inclusiveRadioButton);
-			inclusiveExclusiveGroup.add(exclusiveRadioButton);
-			inclusiveExclusiveMenu.add(inclusiveRadioButton);
-			inclusiveExclusiveMenu.add(exclusiveRadioButton);
-			optionsMenu.add(inclusiveExclusiveMenu);
+			numOfCallsRadioButton.addActionListener(this);
+			numOfSubRoutinesRadioButton.addActionListener(this);
+			userSecPerCallRadioButton.addActionListener(this);
+			
+			metricGroup.add(inclusiveRadioButton);
+			metricGroup.add(exclusiveRadioButton);
+			metricGroup.add(numOfCallsRadioButton);
+			metricGroup.add(numOfSubRoutinesRadioButton);
+			metricGroup.add(userSecPerCallRadioButton);
+			metricMenu.add(inclusiveRadioButton);
+			metricMenu.add(exclusiveRadioButton);
+			metricMenu.add(numOfCallsRadioButton);
+			metricMenu.add(numOfSubRoutinesRadioButton);
+			metricMenu.add(userSecPerCallRadioButton);
+			optionsMenu.add(metricMenu);
 			//End Submenu.
 					
 			
@@ -196,12 +183,22 @@ public class MeanTotalStatWindow extends JFrame implements ActionListener, Obser
 			//Window menu.
 			//******************************
 			JMenu windowsMenu = new JMenu("Windows");
-			windowsMenu.add(windowsMenuDuke);
+			windowsMenu.addMenuListener(this);
 			
 			//Add a submenu.
-			JMenuItem functionLedgerItem = new JMenuItem("Show Function Ledger");
-			functionLedgerItem.addActionListener(this);
-			windowsMenu.add(functionLedgerItem);
+			JMenuItem mappingLedgerItem = new JMenuItem("Show Function Ledger");
+			mappingLedgerItem.addActionListener(this);
+			windowsMenu.add(mappingLedgerItem);
+			
+			//Add a submenu.
+			mappingGroupLedgerItem = new JMenuItem("Show Group Ledger");
+			mappingGroupLedgerItem.addActionListener(this);
+			windowsMenu.add(mappingGroupLedgerItem);
+			
+			//Add a submenu.
+			userEventLedgerItem = new JMenuItem("Show User Event Ledger");
+			userEventLedgerItem.addActionListener(this);
+			windowsMenu.add(userEventLedgerItem);
 			
 			//Add a submenu.
 			JMenuItem closeAllSubwindowsItem = new JMenuItem("Close All Sub-Windows");
@@ -216,7 +213,6 @@ public class MeanTotalStatWindow extends JFrame implements ActionListener, Obser
 			//Help menu.
 			//******************************
 			JMenu helpMenu = new JMenu("Help");
-			helpMenu.add(helpMenuDuke);
 			
 			//Add a menu item.
 			JMenuItem aboutItem = new JMenuItem("About Racy");
@@ -263,7 +259,7 @@ public class MeanTotalStatWindow extends JFrame implements ActionListener, Obser
 			//**********
 			//Panel and ScrollPane definition.
 			//**********
-			meanTotalStatWindowPanelRef = new MeanTotalStatWindowPanel(this);
+			meanTotalStatWindowPanelRef = new MeanTotalStatWindowPanel(trial, this);
 			
 			//The scroll panes into which the list shall be placed.
 			JScrollPane totalStatWindowPanelScrollPane = new JScrollPane(meanTotalStatWindowPanelRef);
@@ -282,7 +278,7 @@ public class MeanTotalStatWindow extends JFrame implements ActionListener, Obser
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "MTSW02");
+			jRacy.systemError(e, null, "MTSW02");
 		}
 	}
 	
@@ -312,13 +308,13 @@ public class MeanTotalStatWindow extends JFrame implements ActionListener, Obser
 				}
 				else if(arg.equals("Adjust Racy Colors"))
 				{
-					jRacy.clrChooser.showColorChooser();	//The ColorChooser class maintains all the state.
+					trial.getColorChooser().showColorChooser();	//The ColorChooser class maintains all the state.
 				}
 				else if(arg.equals("function ID"))
 				{
-					if(functionIDButton.isSelected())
+					if(mappingIDButton.isSelected())
 					{
-						sortByFunctionID = true;
+						sortByMappingID = true;
 						sortByName = false;
 						sortByMillisecond = false;
 						//Call repaint.
@@ -329,7 +325,7 @@ public class MeanTotalStatWindow extends JFrame implements ActionListener, Obser
 				{
 					if(nameButton.isSelected())
 					{
-						sortByFunctionID = false;
+						sortByMappingID = false;
 						sortByName = true;
 						sortByMillisecond = false;
 						//Call repaint.
@@ -340,7 +336,7 @@ public class MeanTotalStatWindow extends JFrame implements ActionListener, Obser
 				{
 					if(millisecondButton.isSelected())
 					{
-						sortByFunctionID = false;
+						sortByMappingID = false;
 						sortByName = false;
 						sortByMillisecond = true;
 						//Call repaint.
@@ -367,7 +363,7 @@ public class MeanTotalStatWindow extends JFrame implements ActionListener, Obser
 				{
 					if(inclusiveRadioButton.isSelected())
 					{
-						inclusive = true;
+						metric = "Inclusive";
 						//Call repaint.
 						meanTotalStatWindowPanelRef.repaint();
 					}
@@ -376,7 +372,34 @@ public class MeanTotalStatWindow extends JFrame implements ActionListener, Obser
 				{
 					if(exclusiveRadioButton.isSelected())
 					{
-						inclusive = false;
+						metric = "Exclusive";
+						//Call repaint.
+						meanTotalStatWindowPanelRef.repaint();
+					}
+				}
+				else if(arg.equals("Number of Calls"))
+				{
+					if(numOfCallsRadioButton.isSelected())
+					{
+						metric = "Number of Calls";
+						//Call repaint.
+						meanTotalStatWindowPanelRef.repaint();
+					}
+				}
+				else if(arg.equals("Number of Subroutines"))
+				{
+					if(numOfSubRoutinesRadioButton.isSelected())
+					{
+						metric = "Number of Subroutines";
+						//Call repaint.
+						meanTotalStatWindowPanelRef.repaint();
+					}
+				}
+				else if(arg.equals("Per Call Value"))
+				{
+					if(userSecPerCallRadioButton.isSelected())
+					{
+						metric = "Per Call Value";
 						//Call repaint.
 						meanTotalStatWindowPanelRef.repaint();
 					}
@@ -384,13 +407,25 @@ public class MeanTotalStatWindow extends JFrame implements ActionListener, Obser
 				else if(arg.equals("Show Function Ledger"))
 				{
 					//In order to be in this window, I must have loaded the data. So,
-					//just show the function ledger window.
-					(jRacy.staticSystemData.getGlobalMapping()).displayFunctionLedger();
+					//just show the mapping ledger window.
+					(trial.getGlobalMapping()).displayMappingLedger(0);
+				}
+				else if(arg.equals("Show Group Ledger"))
+				{
+					//In order to be in this window, I must have loaded the data. So,
+					//just show the mapping ledger window.
+					(trial.getGlobalMapping()).displayMappingLedger(1);
+				}
+				else if(arg.equals("Show User Event Ledger"))
+				{
+					//In order to be in this window, I must have loaded the data. So,
+					//just show the mapping ledger window.
+					(trial.getGlobalMapping()).displayMappingLedger(2);
 				}
 				else if(arg.equals("Close All Sub-Windows"))
 				{
 					//Close the all subwindows.
-					jRacy.systemEvents.updateRegisteredObjects("subWindowCloseEvent");
+					trial.getSystemEvents().updateRegisteredObjects("subWindowCloseEvent");
 				}
 				else if(arg.equals("About Racy"))
 				{
@@ -403,25 +438,61 @@ public class MeanTotalStatWindow extends JFrame implements ActionListener, Obser
 					jRacy.helpWindow.show();
 					//Since the data must have been loaded.  Tell them someting about
 					//where they are.
-					jRacy.helpWindow.writeText("This is the mean total function statistics window.");
+					jRacy.helpWindow.writeText("This is the mean total mapping statistics window.");
 					jRacy.helpWindow.writeText("");
-					jRacy.helpWindow.writeText("This window shows you the mean total statistics for all functions.");
+					jRacy.helpWindow.writeText("This window shows you the mean total statistics for all mappings.");
 					jRacy.helpWindow.writeText("");
 					jRacy.helpWindow.writeText("Use the options menu to select different ways of displaying the data.");
 					jRacy.helpWindow.writeText("");
-					jRacy.helpWindow.writeText("Right click on any function within this window to bring up a popup");
+					jRacy.helpWindow.writeText("Right click on any mapping within this window to bring up a popup");
 					jRacy.helpWindow.writeText("menu. In this menu you can change or reset the default colour");
-					jRacy.helpWindow.writeText("for the function, or to show more details about the function.");
-					jRacy.helpWindow.writeText("You can also left click any function to hightlight it in the system.");
+					jRacy.helpWindow.writeText("for the mapping, or to show more details about the mapping.");
+					jRacy.helpWindow.writeText("You can also left click any mapping to hightlight it in the system.");
 				}
 			}
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "MTSW03");
+			jRacy.systemError(e, null, "MTSW03");
 		}
 	}
 	
+	//******************************
+	//MenuListener code.
+	//******************************
+	public void menuSelected(MenuEvent evt)
+	{
+		try
+		{
+			if(trial.groupNamesPresent())
+				mappingGroupLedgerItem.setEnabled(true);
+			else
+				mappingGroupLedgerItem.setEnabled(false);
+				
+			if(trial.userEventsPresent())
+				userEventLedgerItem.setEnabled(true);
+			else
+				userEventLedgerItem.setEnabled(false);
+		}
+		catch(Exception e)
+		{
+			jRacy.systemError(e, null, "SMW03");
+		}
+		
+	}
+	
+	public void menuDeselected(MenuEvent evt)
+	{
+	}
+	
+	public void menuCanceled(MenuEvent evt)
+	{
+	}
+	
+	//******************************
+	//End - MenuListener code.
+	//******************************
+	
 	//Observer functions.
 	public void update(Observable o, Object arg)
 	{
@@ -437,6 +508,11 @@ public class MeanTotalStatWindow extends JFrame implements ActionListener, Obser
 				//Just need to call a repaint on the ThreadDataWindowPanel.
 				meanTotalStatWindowPanelRef.repaint();
 			}
+			else if(tmpString.equals("dataEvent"))
+			{
+				lastSorting = "";
+				meanTotalStatWindowPanelRef.repaint();
+			}
 			else if(tmpString.equals("subWindowCloseEvent"))
 			{
 				closeThisWindow();
@@ -444,7 +520,7 @@ public class MeanTotalStatWindow extends JFrame implements ActionListener, Obser
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "MTSW04");
+			jRacy.systemError(e, null, "MTSW04");
 		}
 	} 
 	
@@ -461,7 +537,7 @@ public class MeanTotalStatWindow extends JFrame implements ActionListener, Obser
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "MTSW05");
+			jRacy.systemError(e, null, "MTSW05");
 		}
 	}
 	
@@ -469,118 +545,128 @@ public class MeanTotalStatWindow extends JFrame implements ActionListener, Obser
 	//Note:  This is only meant to be called by the TotalStatWindowPanel.
 	public Vector getStaticMainWindowSystemData()
 	{
-		try{
-			if(sortByFunctionID)
+		try
+		{
+			if(sortByMappingID)
 			{
-				if(descendingOrder)
+				if(metric.equals("Inclusive"))
 				{
-					if(!(lastSorting.equals("FIdDE")))
-					{
-						lastList = sMWData.getSMWMeanData("FIdDE");
-						lastSorting = "FIdDE";
-						return lastList;
-					}
+					if(descendingOrder)
+						return sMWData.getSMWMeanData("FIdDI");
 					else
-						return lastList;
+						return sMWData.getSMWMeanData("FIdAI");
 				}
-				else
+				else if(metric.equals("Exclusive"))
 				{
-					if(!(lastSorting.equals("FIdAE")))
-					{
-						lastList = sMWData.getSMWMeanData("FIdAE");
-						lastSorting = "FIdAE";
-						return lastList;
-					}
+					if(descendingOrder)
+						return sMWData.getSMWMeanData("FIdDE");
 					else
-						return lastList;
+						return sMWData.getSMWMeanData("FIdAE");
+				}
+				else if(metric.equals("Number of Calls"))
+				{
+					if(descendingOrder)
+						return sMWData.getSMWMeanData("FIdDNC");
+					else
+						return sMWData.getSMWMeanData("FIdANC");
+				}
+				else if(metric.equals("Number of Subroutines"))
+				{
+					if(descendingOrder)
+						return sMWData.getSMWMeanData("FIdDNS");
+					else
+						return sMWData.getSMWMeanData("FIdANS");
+				}
+				else if(metric.equals("Per Call Value"))
+				{
+					if(descendingOrder)
+						return sMWData.getSMWMeanData("FIdDUS");
+					else
+						return sMWData.getSMWMeanData("FIdAUS");
 				}
 			}
 			else if(sortByName)
 			{
-				if(descendingOrder)
+				
+				if(metric.equals("Inclusive"))
 				{
-					if(!(lastSorting.equals("NDE")))
-					{
-						lastList = sMWData.getSMWMeanData("NDE");
-						lastSorting = "NDE";
-						return lastList;
-					}
+					if(descendingOrder)
+						return sMWData.getSMWMeanData("NDI");
 					else
-						return lastList;
+						return sMWData.getSMWMeanData("NAI");
 				}
-				else
+				else if(metric.equals("Exclusive"))
 				{
-					if(!(lastSorting.equals("NAE")))
-					{
-						lastList = sMWData.getSMWMeanData("NAE");
-						lastSorting = "NAE";
-						return lastList;
-					}
+					if(descendingOrder)
+						return sMWData.getSMWMeanData("NDE");
 					else
-						return lastList;
+						return sMWData.getSMWMeanData("NAE");
+				}
+				else if(metric.equals("Number of Calls"))
+				{
+					if(descendingOrder)
+						return sMWData.getSMWMeanData("NDNC");
+					else
+						return sMWData.getSMWMeanData("NANC");
+				}
+				else if(metric.equals("Number of Subroutines"))
+				{
+					if(descendingOrder)
+						return sMWData.getSMWMeanData("NDNS");
+					else
+						return sMWData.getSMWMeanData("NANS");
+				}
+				else if(metric.equals("Per Call Value"))
+				{
+					if(descendingOrder)
+						return sMWData.getSMWMeanData("NDUS");
+					else
+						return sMWData.getSMWMeanData("NAUS");
 				}
 			}
 			else if(sortByMillisecond)
 			{
-				if(inclusive)
+				
+				if(metric.equals("Inclusive"))
 				{
-					
 					if(descendingOrder)
-					{
-						if(!(lastSorting.equals("MDI")))
-						{
-							lastList = sMWData.getSMWMeanData("MDI");
-							lastSorting = "MDI";
-							return lastList;
-						}
-						else
-							return lastList;
-					}
+						return sMWData.getSMWMeanData("MDI");
 					else
-					{
-						if(!(lastSorting.equals("MAI")))
-						{
-							lastList = sMWData.getSMWMeanData("MAI");
-							lastSorting = "MAI";
-							return lastList;
-						}
-						else
-							return lastList;
-					}
+						return sMWData.getSMWMeanData("MAI");
 				}
-				else
+				else if(metric.equals("Exclusive"))
 				{
 					if(descendingOrder)
-					{
-						if(!(lastSorting.equals("MDE")))
-						{
-							lastList = sMWData.getSMWMeanData("MDE");
-							lastSorting = "MDE";
-							return lastList;
-						}
-						else
-							return lastList;
-					}
+						return sMWData.getSMWMeanData("MDE");
 					else
-					{
-						if(!(lastSorting.equals("MAE")))
-						{
-							lastList = sMWData.getSMWMeanData("MAE");
-							lastSorting = "MAE";
-							return lastList;
-						}
-						else
-							return lastList;
-					}
+						return sMWData.getSMWMeanData("MAE");
+				}
+				else if(metric.equals("Number of Calls"))
+				{
+					if(descendingOrder)
+						return sMWData.getSMWMeanData("MDNC");
+					else
+						return sMWData.getSMWMeanData("MANC");
+				}
+				else if(metric.equals("Number of Subroutines"))
+				{
+					if(descendingOrder)
+						return sMWData.getSMWMeanData("MDNS");
+					else
+						return sMWData.getSMWMeanData("MANS");
+				}
+				else if(metric.equals("Per Call Value"))
+				{
+					if(descendingOrder)
+						return sMWData.getSMWMeanData("MDUS");
+					else
+						return sMWData.getSMWMeanData("MAUS");
 				}
 			}
-		
-			//Should not get here.  Return null ... will force and exception.
-			return null;
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "MTSW06");
+			jRacy.systemError(e, null, "MTSW06");
 		}
 		
 		return null;
@@ -605,36 +691,45 @@ public class MeanTotalStatWindow extends JFrame implements ActionListener, Obser
 			}
 			
 			setVisible(false);
-			jRacy.systemEvents.deleteObserver(this);
+			trial.getSystemEvents().deleteObserver(this);
 			dispose();
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "MTSW07");
+			jRacy.systemError(e, null, "MTSW07");
 		}
 	}
 	
 	//******************************
 	//Instance data.
 	//******************************
+	private Trial trial = null;
  	private MeanTotalStatWindowPanel meanTotalStatWindowPanelRef;
- 	private StaticMainWindowData sMWData = new StaticMainWindowData();
+ 	private StaticMainWindowData sMWData = null;
+ 	
+ 	private JMenuItem mappingGroupLedgerItem;
+	private JMenuItem userEventLedgerItem;
  	
  	ButtonGroup sortGroup;
 	ButtonGroup sortOrderGroup;
-	ButtonGroup inclusiveExclusiveGroup;
+	private ButtonGroup metricGroup = null;
 	
-	JRadioButtonMenuItem functionIDButton;
+	JRadioButtonMenuItem mappingIDButton;
 	JRadioButtonMenuItem nameButton;
 	JRadioButtonMenuItem millisecondButton;
 	
 	JRadioButtonMenuItem ascendingButton;
 	JRadioButtonMenuItem descendingButton;
 	
-	JRadioButtonMenuItem inclusiveRadioButton;
-	JRadioButtonMenuItem exclusiveRadioButton;
+	private JRadioButtonMenuItem inclusiveRadioButton =  new JRadioButtonMenuItem("Inclusive", false);
+	private JRadioButtonMenuItem exclusiveRadioButton = new JRadioButtonMenuItem("Exclusive", true);
+	private JRadioButtonMenuItem numOfCallsRadioButton =  new JRadioButtonMenuItem("Number of Calls", false);
+	private JRadioButtonMenuItem numOfSubRoutinesRadioButton = new JRadioButtonMenuItem("Number of Subroutines", false);
+	private JRadioButtonMenuItem userSecPerCallRadioButton = new JRadioButtonMenuItem("Per Call Value", false);
 	
-	boolean sortByFunctionID = false;
+	private String metric = "Exclusive";
+	
+	boolean sortByMappingID = false;
 	boolean sortByName = false;
 	boolean sortByMillisecond = true;
 	
