@@ -13,44 +13,66 @@ import java.lang.*;
 import java.io.*;
 import java.awt.*;
 import java.awt.event.*;
+import java.awt.print.*;
 import javax.swing.*;
 import javax.swing.border.*;
 import javax.swing.event.*;
 import javax.swing.colorchooser.*;
 
-public class StaticMainWindow extends JFrame implements ActionListener, Observer  
+public class StaticMainWindow extends JFrame implements ActionListener, MenuListener, Observer, ChangeListener  
 {	
 	//******************************
 	//Instance data.
 	//******************************
 	
+	Trial trial = null;
+	
 	//Create a file chooser to allow the user to select files for loading data.
 	JFileChooser fileChooser = new JFileChooser();
 		
 	//References for some of the componants for this frame.
 	private StaticMainWindowPanel sMWPanel;
-	private StaticMainWindowData sMWData = new StaticMainWindowData();
+	private StaticMainWindowData sMWData;
 	
-	ButtonGroup sortGroup;
-	ButtonGroup sortOrderGroup;
+	private ButtonGroup sortGroup;
+	private ButtonGroup sortOrderGroup;
 	
-	JRadioButtonMenuItem functionIDButton;
-	JRadioButtonMenuItem nameButton;
-	JRadioButtonMenuItem millisecondButton;
+	private JRadioButtonMenuItem mappingIDButton;
+	private JRadioButtonMenuItem nameButton;
+	private JRadioButtonMenuItem millisecondButton;
 	
-	JRadioButtonMenuItem ascendingButton;
-	JRadioButtonMenuItem descendingButton;
+	private JRadioButtonMenuItem ascendingButton;
+	private JRadioButtonMenuItem descendingButton;
 	
-	boolean sortByFunctionID = false;
+	private JRadioButtonMenuItem displaySlidersButton;
+	
+	private JMenuItem mappingGroupLedgerItem;
+	private JMenuItem userEventLedgerItem;
+	
+	private JLabel sliderMultipleLabel = new JLabel("Slider Mulitiple");
+	private JComboBox sliderMultiple;
+	
+	private JLabel barLengthLabel = new JLabel("Bar Mulitiple");
+	private JSlider barLengthSlider = new JSlider(0, 40, 1);
+	
+	private Container contentPane = null;
+	private GridBagLayout gbl = null;
+	private GridBagConstraints gbc = null;
+	
+	private JScrollPane scrollPane;
+	
+	boolean sortByMappingID = false;
 	boolean sortByName = false;
 	boolean sortByMillisecond = true;
 	
 	boolean descendingOrder = true;
 	
+	boolean displaySliders = false;
+	
 	
 	//Local data
- 	Vector currentSMWGeneralData = null;
- 	Vector currentSMWMeanData = null;
+ 	private Vector currentSMWGeneralData = null;
+ 	private Vector currentSMWMeanData = null;
 	
 	
 	//******************************
@@ -58,16 +80,20 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 	//******************************
 	
 	
-	public StaticMainWindow()
+	public StaticMainWindow(Trial inTrial)
 	{
 		try{
-			//Window Stuff.
-			setTitle("jRacy");
+			trial = inTrial;
 			
-			int windowWidth = 650;
-			int windowHeight = 300;
+			//Window Stuff.
+			setTitle("jRacy Main Window: " + trial.getProfilePathName());
+			
+			int windowWidth = 750;
+			int windowHeight = 400;
 			setSize(new java.awt.Dimension(windowWidth, windowHeight));
 			
+			sMWData = new StaticMainWindowData(trial);
+			
 			//Add some window listener code
 			addWindowListener(new java.awt.event.WindowAdapter() {
 				public void windowClosing(java.awt.event.WindowEvent evt) {
@@ -104,6 +130,7 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 			
 			//Add a submenu.
 			JMenu openMenu = new JMenu("Open ...");
+				/*
 				//Add a menu item.
 				JMenuItem openPprofDumpFileItem = new JMenuItem("Pprof Dump File");
 				openPprofDumpFileItem.addActionListener(this);
@@ -114,6 +141,12 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 				JMenuItem openjRacyOutputItem = new JMenuItem("jRacy Output File");
 				openjRacyOutputItem.addActionListener(this);
 				openMenu.add(openjRacyOutputItem);
+				*/
+				
+				//Add a menu item.
+				JMenuItem openExperimentManagerItem = new JMenuItem("jRacy Manager");
+				openExperimentManagerItem.addActionListener(this);
+				openMenu.add(openExperimentManagerItem);
 				
 			fileMenu.add(openMenu);
 			//End submenu.
@@ -121,9 +154,9 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 			//Add a submenu.
 			JMenu saveMenu = new JMenu("Save ...");
 				//Add a menu item.
-				JMenuItem savejRacyDataFileFileItem = new JMenuItem("To A jRacy Output File");
-				savejRacyDataFileFileItem.addActionListener(this);
-				saveMenu.add(savejRacyDataFileFileItem);
+				//JMenuItem savejRacyDataFileFileItem = new JMenuItem("To A jRacy Output File");
+				//savejRacyDataFileFileItem.addActionListener(this);
+				//saveMenu.add(savejRacyDataFileFileItem);
 				
 				
 				//Add a menu item.
@@ -134,6 +167,11 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 			fileMenu.add(saveMenu);
 			//End submenu.
 			
+			/*//Add a menu item.
+			JMenuItem printItem = new JMenuItem("Print");
+			printItem.addActionListener(this);
+			fileMenu.add(printItem);
+			*/
 			
 			//Add a menu item.
 			JMenuItem editPrefItem = new JMenuItem("Edit jRacy Preferences!");
@@ -158,9 +196,9 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 			JMenu sortMenu = new JMenu("Sort by ...");
 			sortGroup = new ButtonGroup();
 			
-			functionIDButton = new JRadioButtonMenuItem("function ID", false);
+			mappingIDButton = new JRadioButtonMenuItem("function ID", false);
 			//Add a listener for this radio button.
-			functionIDButton.addActionListener(this);
+			mappingIDButton.addActionListener(this);
 			
 			nameButton = new JRadioButtonMenuItem("name", false);
 			//Add a listener for this radio button.
@@ -170,11 +208,11 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 			//Add a listener for this radio button.
 			millisecondButton.addActionListener(this);
 			
-			sortGroup.add(functionIDButton);
+			sortGroup.add(mappingIDButton);
 			sortGroup.add(nameButton);
 			sortGroup.add(millisecondButton);
 			
-			sortMenu.add(functionIDButton);
+			sortMenu.add(mappingIDButton);
 			sortMenu.add(nameButton);
 			sortMenu.add(millisecondButton);
 			optionsMenu.add(sortMenu);
@@ -200,6 +238,10 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 			optionsMenu.add(sortOrderMenu);
 			//End Submenu.
 			
+			displaySlidersButton = new JRadioButtonMenuItem("Display Sliders", false);
+			//Add a listener for this radio button.
+			displaySlidersButton.addActionListener(this);
+			optionsMenu.add(displaySlidersButton);
 			//******************************
 			//End - Options menu.
 			//******************************
@@ -209,11 +251,22 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 			//Window menu.
 			//******************************
 			JMenu windowsMenu = new JMenu("Windows");
+			windowsMenu.addMenuListener(this);
 			
 			//Add a submenu.
-			JMenuItem functionLedgerItem = new JMenuItem("Show Function Ledger");
-			functionLedgerItem.addActionListener(this);
-			windowsMenu.add(functionLedgerItem);
+			JMenuItem mappingLedgerItem = new JMenuItem("Show Function Ledger");
+			mappingLedgerItem.addActionListener(this);
+			windowsMenu.add(mappingLedgerItem);
+			
+			//Add a submenu.
+			mappingGroupLedgerItem = new JMenuItem("Show Group Ledger");
+			mappingGroupLedgerItem.addActionListener(this);
+			windowsMenu.add(mappingGroupLedgerItem);
+			
+			//Add a submenu.
+			userEventLedgerItem = new JMenuItem("Show User Event Ledger");
+			userEventLedgerItem.addActionListener(this);
+			windowsMenu.add(userEventLedgerItem);
 			
 			//Add a submenu.
 			JMenuItem closeAllSubwindowsItem = new JMenuItem("Close All Sub-Windows");
@@ -229,15 +282,15 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 			//******************************
 			JMenu helpMenu = new JMenu("Help");
 			
-			//Add a menu item.
-			JMenuItem aboutItem = new JMenuItem("About Racy");
-			aboutItem.addActionListener(this);
-			helpMenu.add(aboutItem);
-			
 			//Add a menu item.
 			JMenuItem showHelpWindowItem = new JMenuItem("Show Help Window");
 			showHelpWindowItem.addActionListener(this);
 			helpMenu.add(showHelpWindowItem);
+			
+			//Add a menu item.
+			JMenuItem aboutItem = new JMenuItem("About Racy");
+			aboutItem.addActionListener(this);
+			helpMenu.add(aboutItem);
 			//******************************
 			//End - Help menu.
 			//******************************
@@ -260,10 +313,10 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 			//Create and add the componants.
 			//******************************
 			//Setting up the layout system for the main window.
-			Container contentPane = getContentPane();
-			GridBagLayout gbl = new GridBagLayout();
+			contentPane = getContentPane();
+			gbl = new GridBagLayout();
 			contentPane.setLayout(gbl);
-			GridBagConstraints gbc = new GridBagConstraints();
+			gbc = new GridBagConstraints();
 			gbc.insets = new Insets(5, 5, 5, 5);
 			
 			//Create some borders.
@@ -274,10 +327,10 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 			//**********
 			//Panel and ScrollPane definition.
 			//**********
-			sMWPanel = new StaticMainWindowPanel(this);
+			sMWPanel = new StaticMainWindowPanel(trial, this);
 			sMWPanel.setPreferredSize(new Dimension(600,300));
 			//The scroll panes into which the list shall be placed.
-			JScrollPane scrollPane = new JScrollPane(sMWPanel);
+			scrollPane = new JScrollPane(sMWPanel);
 			scrollPane.setBorder(mainloweredbev);
 			scrollPane.setPreferredSize(new Dimension(600, 300));
 			
@@ -285,6 +338,18 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 			//End - Panel and ScrollPane definition.
 			//**********
 			
+			//Do the slider stuff, but don't add.  By default, sliders are off.
+			String sliderMultipleStrings[] = {"1.00", "0.75", "0.50", "0.25", "0.10"};
+			sliderMultiple = new JComboBox(sliderMultipleStrings);
+			sliderMultiple.addActionListener(this);
+			
+			barLengthSlider.setPaintTicks(true);
+			barLengthSlider.setMajorTickSpacing(5);
+			barLengthSlider.setMinorTickSpacing(1);
+			barLengthSlider.setPaintLabels(true);
+			barLengthSlider.setSnapToTicks(true);
+			barLengthSlider.addChangeListener(this);
+			
 			gbc.fill = GridBagConstraints.BOTH;
 			gbc.anchor = GridBagConstraints.CENTER;
 			gbc.weightx = 1;
@@ -294,36 +359,19 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 			//******************************
 			//End - Create and add the componants.
 			//******************************
-			sMWPanel.repaint();
 			
-			//Ok, now check to see if a "pprof.dat" file exists.  If it does, load it.
-			File testForPprofDat = new File("pprof.dat");
-			if(testForPprofDat.exists())
-			{
-				System.out.println("Found pprof.dat ... loading");
-				
-				//Initialize the static data object.
-				jRacy.staticSystemData = new StaticSystemData();
-				jRacy.staticSystemData.buildStaticData(testForPprofDat);
-				
-				//Build the static main window data lists.
-				sMWData.buildStaticMainWindowDataLists();
-				
-				//Sort the local data.
-				sortLocalData();
-				
-				//Call a repaint of the sMWPanel
-				sMWPanel.repaint();
-			}
-			else
-			{
-				System.out.println("Did not find pprof.dat!");
-			}
+			//Build the static main window data lists.
+			sMWData.buildStaticMainWindowDataLists();
+			
+			//Sort the local data.
+			sortLocalData();
+			
+			//Call a repaint of the sMWPanel
+			sMWPanel.repaint();
 		}
 		catch(Exception e)
-		{
-			
-			jRacy.systemError(null, "SMW01");
+		{	
+			jRacy.systemError(e, null, "SMW01");
 		}
 			
 	}
@@ -343,6 +391,7 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 				String arg = evt.getActionCommand();
 				if(arg.equals("Pprof Dump File"))
 				{
+					/*
 					//Create a file chooser to allow the user to select the pprof dump file.
 					JFileChooser pprofDumpFileChooser = new JFileChooser();
 					
@@ -360,21 +409,38 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 						//Test to see if valid.
 						if(file != null)
 						{	
-							//Initialize the static data object.
-							jRacy.staticSystemData = new StaticSystemData();
-							sMWData = new StaticMainWindowData();
 							
-							//Call the garbage collector.
-							System.out.println("Cleaning the virtual machine memory footprint.");
-							jRacy.runtime.gc();
+							//To avoid some null pointer stuff, for the moment, explicitly close the ledger windows.
+							//Might want to re-think how the mapping ledger window is maintained.  Separate its data
+							//for example!
+							if(jRacy.debugIsOn)
+								System.out.println("Closing the mapping ledger windows.");
+							(trial.getGlobalMapping()).closeMappingLedger(0);
+							(trial.getGlobalMapping()).closeMappingLedger(1);
+							if(jRacy.debugIsOn)
+								System.out.println("End - Closing the mapping ledger windows.");
 							
 							//Closing all subwindows.
 							System.out.println("Closing all subwindows.");
-							jRacy.systemEvents.updateRegisteredObjects("subWindowCloseEvent");
+							trial.getSystemEvents().updateRegisteredObjects("subWindowCloseEvent");
+							
+							
+							jRacy.profilePathName = file.getCanonicalPath();
+				
+							setTitle("jRacy: " + jRacy.profilePathName);
+							
+							//Initialize the static data object.
+							jRacy.staticSystemData = new Trial();
+							sMWData = new StaticMainWindowData();
+							
+							//Call the garbage collector.
+							if(jRacy.debugIsOn)
+								System.out.println("Cleaning the virtual machine memory footprint.");
+							jRacy.runtime.gc();
 							
 							System.out.println("Building internal data system ... please wait ...");
 							
-							jRacy.staticSystemData.buildStaticData(file);
+							trial.buildStaticData(file);
 							
 							System.out.println("Finished building internal data system.");
 							
@@ -384,10 +450,10 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 							System.out.println("Finished building internal lists.");
 							
 							//Reset the hightlight colour.
-							jRacy.clrChooser.setHighlightColorFunctionID(-1);
+							jRacy.clrChooser.setHighlightColorMappingID(-1);
 							
 							//Indicate to the rest of the system that there has been a change of data.
-							jRacy.systemEvents.updateRegisteredObjects("dataSetChangeEvent");
+							trial.getSystemEvents().updateRegisteredObjects("dataSetChangeEvent");
 							
 							System.out.println("Done ... loading complete!");
 							
@@ -400,10 +466,12 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 							System.out.println("There was some sort of internal error!");
 						}
 					}
+				*/
 					
 				}
 				else if(arg.equals("jRacy Output File"))
 				{	
+					/*
 					//Set the directory to the current directory.
 					fileChooser.setCurrentDirectory(new File("."));
 					
@@ -420,20 +488,24 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 						if(file != null)
 						{
 							
+							jRacy.profilePathName = file.getCanonicalPath();
+				
+							setTitle("jRacy: " + jRacy.profilePathName);
+							
 							//Closing all subwindows.
 							System.out.println("Closing all subwindows.");
-							jRacy.systemEvents.updateRegisteredObjects("subWindowCloseEvent");
+							trial.getSystemEvents().updateRegisteredObjects("subWindowCloseEvent");
 							
 							
 							//Get the data object to which this file refers.
 							try
 							{
 								ObjectInputStream racyDataObjectIn = new ObjectInputStream(new FileInputStream(file));
-								jRacy.staticSystemData = (StaticSystemData) racyDataObjectIn.readObject();
+								jRacy.staticSystemData = (Trial) racyDataObjectIn.readObject();
 							}
 							catch(Exception e)
 							{
-								jRacy.systemError(null, "SMW02A");
+								jRacy.systemError(e, null, "SMW02A");
 							}
 							
 							sMWData = new StaticMainWindowData();
@@ -448,10 +520,10 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 							System.out.println("Finished building internal lists.");
 							
 							//Reset the hightlight colour.
-							jRacy.clrChooser.setHighlightColorFunctionID(-1);
+							jRacy.clrChooser.setHighlightColorMappingID(-1);
 							
 							//Indicate to the rest of the system that there has been a change of data.
-							jRacy.systemEvents.updateRegisteredObjects("dataSetChangeEvent");
+							trial.getSystemEvents().updateRegisteredObjects("dataSetChangeEvent");
 							
 							System.out.println("Done ... loading complete!");
 							
@@ -465,11 +537,17 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 															  	,JOptionPane.ERROR_MESSAGE);
 						}
 					}
+				*/
+				}
+				else if(arg.equals("jRacy Manager"))
+				{
+					jRacyManager jRM = new jRacyManager();
+					jRM.show();
 				}
 				else if(arg.equals("To A jRacy Output File"))
 				{
 					
-					
+					/*
 					//Set the directory to the current directory.
 					fileChooser.setCurrentDirectory(new File("."));
 					
@@ -506,11 +584,12 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 															  	,JOptionPane.ERROR_MESSAGE);
 						}
 					}
+					*/
 				}
 				else if(arg.equals("jRacy Preferrences"))
 				{
 					
-					
+					/*
 					//Set the directory to the current directory.
 					fileChooser.setCurrentDirectory(new File("."));
 					fileChooser.setSelectedFile(new File("jRacyPreferences.dat"));
@@ -531,7 +610,7 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 							{
 								//Write to the savedPreferences object.
 								jRacy.clrChooser.setSavedColors();
-								jRacy.jRacyPreferences.setSavedPreferences();
+								trial.getPreferences().setSavedPreferences();
 								
 								ObjectOutputStream prefsOut = new ObjectOutputStream(new FileOutputStream(file));
 								prefsOut.writeObject(jRacy.savedPreferences);
@@ -552,12 +631,12 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 															  	,JOptionPane.ERROR_MESSAGE);
 						}
 					}
+				*/
 				}
 				else if(arg.equals("Edit jRacy Preferences!"))
 				{
-					jRacy.jRacyPreferences.showPreferencesWindow();
-				}
-					
+					trial.getPreferences().showPreferencesWindow();
+				}	
 				else if(arg.equals("Exit jRacy!"))
 				{
 					setVisible(false);
@@ -566,9 +645,9 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 				}
 				else if(arg.equals("function ID"))
 				{
-					if(functionIDButton.isSelected())
+					if(mappingIDButton.isSelected())
 					{
-						sortByFunctionID = true;
+						sortByMappingID = true;
 						sortByName = false;
 						sortByMillisecond = false;
 						//Sort the local data.
@@ -581,7 +660,7 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 				{
 					if(nameButton.isSelected())
 					{
-						sortByFunctionID = false;
+						sortByMappingID = false;
 						sortByName = true;
 						sortByMillisecond = false;
 						//Sort the local data.
@@ -594,7 +673,7 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 				{
 					if(millisecondButton.isSelected())
 					{
-						sortByFunctionID = false;
+						sortByMappingID = false;
 						sortByName = false;
 						sortByMillisecond = true;
 						//Sort the local data.
@@ -625,13 +704,55 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 						sMWPanel.repaint();
 					}
 				}
+				else if(arg.equals("Display Sliders"))
+				{
+					if(displaySlidersButton.isSelected())
+					{	
+						displaySiders(true);
+					}
+					else
+					{
+						displaySiders(false);
+					}
+				}
 				else if(arg.equals("Show Function Ledger"))
+				{
+					
+					//Check to make sure that the system data file has been loaded!
+					if(sMWData.isDataLoaded())
+					{
+						//Grab the global mapping and bring up the mapping ledger window.
+						(trial.getGlobalMapping()).displayMappingLedger(0);
+					}
+					else
+					{
+						//Pop up an error!
+						JOptionPane.showMessageDialog(this, "Sorry, but you must load a pprof data file first!", "Selection Error!"
+																  ,JOptionPane.ERROR_MESSAGE);
+					}
+				}
+				else if(arg.equals("Show Group Ledger"))
 				{
 					//Check to make sure that the system data file has been loaded!
 					if(sMWData.isDataLoaded())
 					{
-						//Grab the global mapping and bring up the function ledger window.
-						(jRacy.staticSystemData.getGlobalMapping()).displayFunctionLedger();
+						//Grab the global mapping and bring up the mapping ledger window.
+						(trial.getGlobalMapping()).displayMappingLedger(1);
+					}
+					else
+					{
+						//Pop up an error!
+						JOptionPane.showMessageDialog(this, "Sorry, but you must load a pprof data file first!", "Selection Error!"
+																  ,JOptionPane.ERROR_MESSAGE);
+					}
+				}
+				else if(arg.equals("Show User Event Ledger"))
+				{
+					//Check to make sure that the system data file has been loaded!
+					if(sMWData.isDataLoaded())
+					{
+						//Grab the global mapping and bring up the mapping ledger window.
+						(trial.getGlobalMapping()).displayMappingLedger(2);
 					}
 					else
 					{
@@ -643,7 +764,7 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 				else if(arg.equals("Close All Sub-Windows"))
 				{
 					//Close the all subwindows.
-					jRacy.systemEvents.updateRegisteredObjects("subWindowCloseEvent");
+					trial.getSystemEvents().updateRegisteredObjects("subWindowCloseEvent");
 				}
 				else if(arg.equals("About Racy"))
 				{
@@ -665,22 +786,67 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 						jRacy.helpWindow.writeText("To create a pprof dump file, simply run pprof" +
 												  " with the -d option, and pipe the output to a file.");
 					}
-					else
-					{
-						jRacy.helpWindow.writeText("Welcome to jRacy");
-						jRacy.helpWindow.writeText("");
-						
-						
-					}
 				}
 			}
+			else if(EventSrc == sliderMultiple)
+			{
+				sMWPanel.changeInMultiples();
+			}
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "SMW02");
+			jRacy.systemError(e, null, "SMW02");
 		}
 	}
 	
+	//******************************
+	//MenuListener code.
+	//******************************
+	public void menuSelected(MenuEvent evt)
+	{
+		try
+		{
+			if(trial.groupNamesPresent())
+				mappingGroupLedgerItem.setEnabled(true);
+			else
+				mappingGroupLedgerItem.setEnabled(false);
+				
+			if(trial.userEventsPresent())
+				userEventLedgerItem.setEnabled(true);
+			else
+				userEventLedgerItem.setEnabled(false);
+		}
+		catch(Exception e)
+		{
+			jRacy.systemError(e, null, "SMW03");
+		}
+		
+	}
+	
+	public void menuDeselected(MenuEvent evt)
+	{
+	}
+	
+	public void menuCanceled(MenuEvent evt)
+	{
+	}
+	
+	//******************************
+	//End - MenuListener code.
+	//******************************
+	
+	//******************************
+	//Change listener code.
+	//******************************
+	public void stateChanged(ChangeEvent event)
+	{
+		sMWPanel.changeInMultiples();
+	}
+	//******************************
+	//End - Change listener code.
+	//******************************
+	
+	
 	//Observer functions.
 	public void update(Observable o, Object arg)
 	{
@@ -696,6 +862,11 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 				//Just need to call a repaint on the ThreadDataWindowPanel.
 				sMWPanel.repaint();
 			}
+			else if(tmpString.equals("dataEvent"))
+			{
+				sortLocalData();
+				sMWPanel.repaint();
+			}
 			else if(tmpString.equals("dataSetChangeEvent"))
 			{
 				//Clear any locally saved data.
@@ -708,9 +879,100 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "SMW03");
+			jRacy.systemError(e, null, "SMW04");
 		}
 	}
+	
+	public int getSliderValue()
+	{
+		int tmpInt = -1;
+		
+		try
+		{
+			tmpInt = barLengthSlider.getValue();
+		}
+		catch(Exception e)
+		{
+			jRacy.systemError(e, null, "SMW05");
+		}
+		
+		return tmpInt;
+	}
+	
+	public double getSliderMultiple()
+	{
+		String tmpString = null;
+		try
+		{
+			tmpString = (String) sliderMultiple.getSelectedItem();
+			return Double.parseDouble(tmpString);
+		}
+		catch(Exception e)
+		{
+			jRacy.systemError(e, null, "SMW06");
+		}
+		
+		return 0;
+	}
+	
+	private void displaySiders(boolean displaySliders)
+	{
+		if(displaySliders)
+		{
+			//Since the menu option is a toggle, the only component that needs to be
+			//removed is that scrollPane.  We then add back in with new parameters.
+			//This might not be required as it seems to adjust well if left in, but just
+			//to be sure.
+			contentPane.remove(scrollPane);
+			
+			gbc.fill = GridBagConstraints.NONE;
+			gbc.anchor = GridBagConstraints.EAST;
+			gbc.weightx = 0.10;
+			gbc.weighty = 0.01;
+			addCompItem(sliderMultipleLabel, gbc, 0, 0, 1, 1);
+			
+			gbc.fill = GridBagConstraints.NONE;
+			gbc.anchor = GridBagConstraints.WEST;
+			gbc.weightx = 0.10;
+			gbc.weighty = 0.01;
+			addCompItem(sliderMultiple, gbc, 1, 0, 1, 1);
+			
+			gbc.fill = GridBagConstraints.NONE;
+			gbc.anchor = GridBagConstraints.EAST;
+			gbc.weightx = 0.10;
+			gbc.weighty = 0.01;
+			addCompItem(barLengthLabel, gbc, 2, 0, 1, 1);
+			
+			gbc.fill = GridBagConstraints.HORIZONTAL;
+			gbc.anchor = GridBagConstraints.WEST;
+			gbc.weightx = 0.70;
+			gbc.weighty = 0.01;
+			addCompItem(barLengthSlider, gbc, 3, 0, 1, 1);
+			
+			gbc.fill = GridBagConstraints.BOTH;
+			gbc.anchor = GridBagConstraints.CENTER;
+			gbc.weightx = 1.0;
+			gbc.weighty = 0.99;
+			addCompItem(scrollPane, gbc, 0, 1, 4, 1);
+		}
+		else
+		{
+			contentPane.remove(sliderMultipleLabel);
+			contentPane.remove(sliderMultiple);
+			contentPane.remove(barLengthLabel);
+			contentPane.remove(barLengthSlider);
+			contentPane.remove(scrollPane);
+			
+			gbc.fill = GridBagConstraints.BOTH;
+			gbc.anchor = GridBagConstraints.CENTER;
+			gbc.weightx = 1;
+			gbc.weighty = 1;
+			addCompItem(scrollPane, gbc, 0, 0, 1, 1);
+		}
+		
+		//Now call validate so that these componant changes are displayed.
+		validate();
+	}
 		
 	private void addCompItem(Component c, GridBagConstraints gbc, int x, int y, int w, int h)
 	{
@@ -720,11 +982,11 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 			gbc.gridwidth = w;
 			gbc.gridheight = h;
 			
-			getContentPane().add(c, gbc);
+			contentPane.add(c, gbc);
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "SMW04");
+			jRacy.systemError(e, null, "SMW07");
 		}
 	}
 	
@@ -742,7 +1004,7 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 	{
 		try{
 			//First, do the currentSMWGeneralData.
-			if(sortByFunctionID)
+			if(sortByMappingID)
 			{
 				if(descendingOrder)
 					currentSMWGeneralData = sMWData.getSMWGeneralData("FIdDE");
@@ -765,7 +1027,7 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 			}
 			
 			//Now do the currentSMWMeanData.
-			if(sortByFunctionID)
+			if(sortByMappingID)
 			{
 				if(descendingOrder)
 					currentSMWMeanData = sMWData.getSMWMeanData("FIdDE");
@@ -776,8 +1038,9 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 			{
 				if(descendingOrder)
 					currentSMWMeanData = sMWData.getSMWMeanData("NDE");
-				else
+				else{
 					currentSMWMeanData = sMWData.getSMWMeanData("NAE");
+					}
 			}
 			else if(sortByMillisecond)
 			{
@@ -789,7 +1052,7 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "SMW05");
+			jRacy.systemError(e, null, "SMW08");
 		}
 		
 	}
@@ -815,7 +1078,7 @@ public class StaticMainWindow extends JFrame implements ActionListener, Observer
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "SMW06");
+			jRacy.systemError(e, null, "SMW09");
 		}
 		
 		return false;
