@@ -3,7 +3,7 @@
 
 	Title:			jRacy
 	Author:			Robert Bell
-	Description:	The container for the FunctionDataWindowPanel.
+	Description:	The container for the MappingDataWindowPanel.
 */
 
 package jRacy;
@@ -14,8 +14,9 @@ import java.awt.event.*;
 import javax.swing.*;
 import javax.swing.border.*;
 import javax.swing.event.*;
+import java.awt.print.*;
 
-public class ThreadDataWindow extends JFrame implements ActionListener, MenuListener, Observer
+public class ThreadDataWindow extends JFrame implements ActionListener, MenuListener, Observer, ChangeListener
 {
 	
 	public ThreadDataWindow()
@@ -30,28 +31,33 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "TDW01");
+			jRacy.systemError(e, null, "TDW01");
 		}
 	}
 	
-	public ThreadDataWindow(int inServerNumber,
+	public ThreadDataWindow(Trial inTrial,
+							int inServerNumber,
 							int inContextNumber,
 							int inThreadNumber,
 							StaticMainWindowData inSMWData)
 	{
 		try
 		{
+			
+			trial = inTrial;
+			sMWData = inSMWData;
+			
+			lPWindow = new LocalPrefWindow(trial, this);
+			
 			setLocation(new java.awt.Point(300, 200));
 			setSize(new java.awt.Dimension(700, 450));
 			
-			sMWData = inSMWData;
-			
 			server = inServerNumber;
 			context = inContextNumber;
 			thread = inThreadNumber;
 			
 			//Now set the title.
-			this.setTitle("n,c,t, " + server + "," + context + "," + thread);
+			this.setTitle("n,c,t, " + server + "," + context + "," + thread + " - " + trial.getProfilePathName());
 			
 			//Add some window listener code
 			addWindowListener(new java.awt.event.WindowAdapter() {
@@ -93,6 +99,16 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 			//******************************
 			JMenu fileMenu = new JMenu("File");
 			
+			/*//Add a menu item.
+			JMenuItem printItem = new JMenuItem("Print");
+			printItem.addActionListener(this);
+			fileMenu.add(printItem);*/
+			
+			//Add a menu item.
+			JMenuItem editPrefItem = new JMenuItem("Edit jRacy Preferences!");
+			editPrefItem.addActionListener(this);
+			fileMenu.add(editPrefItem);
+			
 			//Add a menu item.
 			JMenuItem closeItem = new JMenuItem("Close This Window");
 			closeItem.addActionListener(this);
@@ -110,66 +126,64 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 			//Options menu.
 			//******************************
 			JMenu optionsMenu = new JMenu("Options");
-			
 			optionsMenu.addMenuListener(this);
 			
 			//***********
 			//Submenu.
 			JMenu sortMenu = new JMenu("Sort by ...");
-			sortGroup = new ButtonGroup();
 			
+			sortGroup = new ButtonGroup();
 			//Add listeners
-			functionIDButton.addActionListener(this);
 			nameButton.addActionListener(this);
 			millisecondButton.addActionListener(this);
-			
-			sortGroup.add(functionIDButton);
 			sortGroup.add(nameButton);
 			sortGroup.add(millisecondButton);
-			
-			sortMenu.add(functionIDButton);
 			sortMenu.add(nameButton);
 			sortMenu.add(millisecondButton);
+			
+			
+			sortOrderGroup = new ButtonGroup();
+			//Add listeners
+			ascendingButton.addActionListener(this);
+			descendingButton.addActionListener(this);
+			sortOrderGroup.add(ascendingButton);
+			sortOrderGroup.add(descendingButton);
+			sortMenu.add(ascendingButton);
+			sortMenu.add(descendingButton);
+			
+			sortMenu.insertSeparator(2);
+			
 			optionsMenu.add(sortMenu);
 			//End - Submenu.
 			//***********
 			
-			//***********
-			//Submenu.
-			JMenu sortOrderMenu = new JMenu("Sort Order");
-			sortOrderGroup = new ButtonGroup();
-			
-			//Add listeners
-			ascendingButton.addActionListener(this);
-			descendingButton.addActionListener(this);
-			
-			sortOrderGroup.add(ascendingButton);
-			sortOrderGroup.add(descendingButton);
-			
-			sortOrderMenu.add(ascendingButton);
-			sortOrderMenu.add(descendingButton);
-			optionsMenu.add(sortOrderMenu);
-			//End - Submenu.
-			//***********
-			
 			//Add a submenu.
-			JMenu inclusiveExclusiveMenu = new JMenu("Select Inclusive or Exclusive");
-			inclusiveExclusiveGroup = new ButtonGroup();
+			JMenu metricMenu = new JMenu("Select Metric");
+			metricGroup = new ButtonGroup();
 			
 			//Add listeners
 			inclusiveRadioButton.addActionListener(this);
 			exclusiveRadioButton.addActionListener(this);
+			numOfCallsRadioButton.addActionListener(this);
+			numOfSubRoutinesRadioButton.addActionListener(this);
+			userSecPerCallRadioButton.addActionListener(this);
 			
-			inclusiveExclusiveGroup.add(inclusiveRadioButton);
-			inclusiveExclusiveGroup.add(exclusiveRadioButton);
-			inclusiveExclusiveMenu.add(inclusiveRadioButton);
-			inclusiveExclusiveMenu.add(exclusiveRadioButton);
-			optionsMenu.add(inclusiveExclusiveMenu);
+			metricGroup.add(inclusiveRadioButton);
+			metricGroup.add(exclusiveRadioButton);
+			metricGroup.add(numOfCallsRadioButton);
+			metricGroup.add(numOfSubRoutinesRadioButton);
+			metricGroup.add(userSecPerCallRadioButton);
+			metricMenu.add(inclusiveRadioButton);
+			metricMenu.add(exclusiveRadioButton);
+			metricMenu.add(numOfCallsRadioButton);
+			metricMenu.add(numOfSubRoutinesRadioButton);
+			metricMenu.add(userSecPerCallRadioButton);
+			optionsMenu.add(metricMenu);
 			//End Submenu.
 			
 			//***********
 			//Submenu.
-			JMenu valuePercentMenu = new JMenu("Select Value or Percent");
+			valuePercentMenu = new JMenu("Select Value or Percent");
 			valuePercentGroup = new ButtonGroup();
 			
 			//Add listeners
@@ -187,6 +201,7 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 			
 			//***********
 			//Submenu.
+			unitsMenu = new JMenu("Select Units");
 			unitsGroup = new ButtonGroup();
 			
 			//Add listeners
@@ -205,6 +220,16 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 			//End - Submenu.
 			//***********
 			
+			displaySlidersButton = new JRadioButtonMenuItem("Display Sliders", false);
+			//Add a listener for this radio button.
+			displaySlidersButton.addActionListener(this);
+			optionsMenu.add(displaySlidersButton);
+			
+			formatValuesButton = new JRadioButtonMenuItem("Format Values", true);
+			//Add a listener for this radio button.
+			formatValuesButton.addActionListener(this);
+			optionsMenu.add(formatValuesButton);
+			
 			//******************************
 			//End - Options menu.
 			//******************************
@@ -214,16 +239,26 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 			//Window menu.
 			//******************************
 			JMenu windowsMenu = new JMenu("Windows");
+			windowsMenu.addMenuListener(this);
 			
-			JMenuItem functionLedgerItem = new JMenuItem("Show Function Ledger");
+			//Add a submenu.
+			JMenuItem mappingLedgerItem = new JMenuItem("Show Function Ledger");
+			mappingLedgerItem.addActionListener(this);
+			windowsMenu.add(mappingLedgerItem);
 			
-			JMenuItem closeAllSubwindowsItem = new JMenuItem("Close All Sub-Windows");
+			//Add a submenu.
+			mappingGroupLedgerItem = new JMenuItem("Show Group Ledger");
+			mappingGroupLedgerItem.addActionListener(this);
+			windowsMenu.add(mappingGroupLedgerItem);
+			
+			//Add a submenu.
+			userEventLedgerItem = new JMenuItem("Show User Event Ledger");
+			userEventLedgerItem.addActionListener(this);
+			windowsMenu.add(userEventLedgerItem);
 			
 			//Add listeners
-			functionLedgerItem.addActionListener(this);
+			JMenuItem closeAllSubwindowsItem = new JMenuItem("Close All Sub-Windows");
 			closeAllSubwindowsItem.addActionListener(this);
-			
-			windowsMenu.add(functionLedgerItem);
 			windowsMenu.add(closeAllSubwindowsItem);
 			//******************************
 			//End - Window menu.
@@ -267,10 +302,10 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 			//Create and add the componants.
 			//******************************
 			//Setting up the layout system for the main window.
-			Container contentPane = getContentPane();
-			GridBagLayout gbl = new GridBagLayout();
+			contentPane = getContentPane();
+			gbl = new GridBagLayout();
 			contentPane.setLayout(gbl);
-			GridBagConstraints gbc = new GridBagConstraints();
+			gbc = new GridBagConstraints();
 			gbc.insets = new Insets(5, 5, 5, 5);
 			
 			//Create some borders.
@@ -281,7 +316,8 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 			//**********
 			//Panel and ScrollPane definition.
 			//**********
-			threadDataWindowPanelRef = new ThreadDataWindowPanel(inServerNumber,
+			threadDataWindowPanelRef = new ThreadDataWindowPanel(trial,
+																 inServerNumber,
 																 inContextNumber,
 																 inThreadNumber, this, sMWData);
 			
@@ -290,40 +326,32 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 			//**********
 			
 			//The scroll panes into which the list shall be placed.
-			JScrollPane threadDataWindowPanelScrollPane = new JScrollPane(threadDataWindowPanelRef);
+			threadDataWindowPanelScrollPane = new JScrollPane(threadDataWindowPanelRef);
 			threadDataWindowPanelScrollPane.setBorder(mainloweredbev);
 			threadDataWindowPanelScrollPane.setPreferredSize(new Dimension(500, 450));
 			
+			
+			//Do the slider stuff, but don't add.  By default, sliders are off.
+			String sliderMultipleStrings[] = {"1.00", "0.75", "0.50", "0.25", "0.10"};
+			sliderMultiple = new JComboBox(sliderMultipleStrings);
+			sliderMultiple.addActionListener(this);
+			
 			barLengthSlider.setPaintTicks(true);
 			barLengthSlider.setMajorTickSpacing(5);
 			barLengthSlider.setMinorTickSpacing(1);
 			barLengthSlider.setPaintLabels(true);
 			barLengthSlider.setSnapToTicks(true);
-			barLengthSlider.addChangeListener(threadDataWindowPanelRef);
-			
-			
-			//Now add the componants to the main screen.
-			gbc.fill = GridBagConstraints.NONE;
-			gbc.anchor = GridBagConstraints.EAST;
-			gbc.weightx = 0.475;
-			gbc.weighty = 0.01;
-			addCompItem(barLengthLabel, gbc, 0, 0, 1, 1);
-			
-			gbc.fill = GridBagConstraints.HORIZONTAL;
-			gbc.anchor = GridBagConstraints.WEST;
-			gbc.weightx = 0.475;
-			gbc.weighty = 0.01;
-			addCompItem(barLengthSlider, gbc, 1, 0, 1, 1);
+			barLengthSlider.addChangeListener(this);
 			
 			gbc.fill = GridBagConstraints.BOTH;
 			gbc.anchor = GridBagConstraints.CENTER;
 			gbc.weightx = 0.95;
 			gbc.weighty = 0.98;
-			addCompItem(threadDataWindowPanelScrollPane, gbc, 0, 1, 2, 1);
+			addCompItem(threadDataWindowPanelScrollPane, gbc, 0, 0, 1, 1);
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "TDW02");
+			jRacy.systemError(e, null, "TDW02");
 		}
 		
 		
@@ -345,8 +373,21 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 			if(EventSrc instanceof JMenuItem)
 			{
 				String arg = evt.getActionCommand();
-				
-				if(arg.equals("Close This Window"))
+				if(arg.equals("Print"))
+				{
+					PrinterJob job = PrinterJob.getPrinterJob();
+					PageFormat defaultFormat = job.defaultPage();
+					PageFormat selectedFormat = job.pageDialog(defaultFormat);
+					job.setPrintable(threadDataWindowPanelRef, selectedFormat);
+					if(job.printDialog()){
+						job.print();
+					}
+				}
+				else if(arg.equals("Edit jRacy Preferences!"))
+				{
+					lPWindow.show();
+				}
+				else if(arg.equals("Close This Window"))
 				{
 					closeThisWindow();
 				}
@@ -356,24 +397,11 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 					dispose();
 					System.exit(0);
 				}
-				else if(arg.equals("function ID"))
-				{
-					if(functionIDButton.isSelected())
-					{
-						sortByFunctionID = true;
-						sortByName = false;
-						sortByMillisecond = false;
-						//Sort the local data.
-						sortLocalData();
-						//Call repaint.
-						threadDataWindowPanelRef.repaint();
-					}
-				}
 				else if(arg.equals("name"))
 				{
 					if(nameButton.isSelected())
 					{
-						sortByFunctionID = false;
+						sortByMappingID = false;
 						sortByName = true;
 						sortByMillisecond = false;
 						//Sort the local data.
@@ -382,11 +410,11 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 						threadDataWindowPanelRef.repaint();
 					}
 				}
-				else if(arg.equals("millisecond"))	//Note the difference in case from the millisecond option below.
+				else if(arg.equals("Selected Metric"))	//Note the difference in case from the millisecond option below.
 				{
 					if(millisecondButton.isSelected())
 					{
-						sortByFunctionID = false;
+						sortByMappingID = false;
 						sortByName = false;
 						sortByMillisecond = true;
 						//Sort the local data.
@@ -421,7 +449,7 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 				{
 					if(inclusiveRadioButton.isSelected())
 					{
-						inclusive = true;
+						metric = "Inclusive";
 						//Sort the local data.
 						sortLocalData();
 						//Call repaint.
@@ -432,7 +460,40 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 				{
 					if(exclusiveRadioButton.isSelected())
 					{
-						inclusive = false;
+						metric = "Exclusive";
+						//Sort the local data.
+						sortLocalData();
+						//Call repaint.
+						threadDataWindowPanelRef.repaint();
+					}
+				}
+				else if(arg.equals("Number of Calls"))
+				{
+					if(numOfCallsRadioButton.isSelected())
+					{
+						metric = "Number of Calls";
+						//Sort the local data.
+						sortLocalData();
+						//Call repaint.
+						threadDataWindowPanelRef.repaint();
+					}
+				}
+				else if(arg.equals("Number of Subroutines"))
+				{
+					if(numOfSubRoutinesRadioButton.isSelected())
+					{
+						metric = "Number of Subroutines";
+						//Sort the local data.
+						sortLocalData();
+						//Call repaint.
+						threadDataWindowPanelRef.repaint();
+					}
+				}
+				else if(arg.equals("Per Call Value"))
+				{
+					if(userSecPerCallRadioButton.isSelected())
+					{
+						metric = "Per Call Value";
 						//Sort the local data.
 						sortLocalData();
 						//Call repaint.
@@ -484,16 +545,54 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 						threadDataWindowPanelRef.repaint();
 					}
 				}
+				else if(arg.equals("Display Sliders"))
+				{
+					if(displaySlidersButton.isSelected())
+					{	
+						displaySiders(true);
+					}
+					else
+					{
+						displaySiders(false);
+					}
+				}
+				else if(arg.equals("Format Values"))
+				{
+					if(formatValuesButton.isSelected())
+					{	
+						formatNumbers = true;
+						//Call repaint.
+						threadDataWindowPanelRef.repaint();
+					}
+					else
+					{
+						formatNumbers = false;
+						//Call repaint.
+						threadDataWindowPanelRef.repaint();
+					}
+				}
 				else if(arg.equals("Show Function Ledger"))
 				{
 					//In order to be in this window, I must have loaded the data. So,
-					//just show the function ledger window.
-					(jRacy.staticSystemData.getGlobalMapping()).displayFunctionLedger();
+					//just show the mapping ledger window.
+					(trial.getGlobalMapping()).displayMappingLedger(0);
+				}
+				else if(arg.equals("Show Group Ledger"))
+				{
+					//In order to be in this window, I must have loaded the data. So,
+					//just show the mapping ledger window.
+					(trial.getGlobalMapping()).displayMappingLedger(1);
+				}
+				else if(arg.equals("Show User Event Ledger"))
+				{
+					//In order to be in this window, I must have loaded the data. So,
+					//just show the mapping ledger window.
+					(trial.getGlobalMapping()).displayMappingLedger(2);
 				}
 				else if(arg.equals("Close All Sub-Windows"))
 				{
 					//Close the all subwindows.
-					jRacy.systemEvents.updateRegisteredObjects("subWindowCloseEvent");
+					trial.getSystemEvents().updateRegisteredObjects("subWindowCloseEvent");
 				}
 				else if(arg.equals("About Racy"))
 				{
@@ -508,20 +607,24 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 					//where they are.
 					jRacy.helpWindow.writeText("This is the thread data window");
 					jRacy.helpWindow.writeText("");
-					jRacy.helpWindow.writeText("This window shows you the values for all functions on this thread.");
+					jRacy.helpWindow.writeText("This window shows you the values for all mappings on this thread.");
 					jRacy.helpWindow.writeText("");
 					jRacy.helpWindow.writeText("Use the options menu to select different ways of displaying the data.");
 					jRacy.helpWindow.writeText("");
-					jRacy.helpWindow.writeText("Right click on any function within this window to bring up a popup");
+					jRacy.helpWindow.writeText("Right click on any mapping within this window to bring up a popup");
 					jRacy.helpWindow.writeText("menu. In this menu you can change or reset the default colour");
-					jRacy.helpWindow.writeText("for the function, or to show more details about the function.");
-					jRacy.helpWindow.writeText("You can also left click any function to hightlight it in the system.");
+					jRacy.helpWindow.writeText("for the mapping, or to show more details about the mapping.");
+					jRacy.helpWindow.writeText("You can also left click any mapping to hightlight it in the system.");
 				}
 			}
+			else if(EventSrc == sliderMultiple)
+			{
+				threadDataWindowPanelRef.changeInMultiples();
+			}
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "TDW03");
+			jRacy.systemError(e, null, "TDW03");
 		}
 	}
 	
@@ -529,6 +632,18 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 	//End - ActionListener code.
 	//******************************
 	
+	//******************************
+	//Change listener code.
+	//******************************
+	public void stateChanged(ChangeEvent event)
+	{
+		threadDataWindowPanelRef.changeInMultiples();
+	}
+	//******************************
+	//End - Change listener code.
+	//******************************
+	
+	
 	//******************************
 	//MenuListener code.
 	//******************************
@@ -536,14 +651,45 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 	{
 		try
 		{
-			if(percent)
-				unitsMenu.setEnabled(false);
+			String trialName = trial.getCounterName();
+			trialName = trialName.toUpperCase();
+			boolean isDefault = false;
+			boolean isTimeMetric = false;
+			
+			if(trialName.equals("DEFAULT"))	
+				isDefault = true;
+			else if(trialName.indexOf("TIME") != -1)
+				isTimeMetric = true;
+			
+			if(trial.groupNamesPresent())
+				mappingGroupLedgerItem.setEnabled(true);
 			else
-				unitsMenu.setEnabled(true);
+				mappingGroupLedgerItem.setEnabled(false);
+				
+			if(trial.userEventsPresent())
+				userEventLedgerItem.setEnabled(true);
+			else
+				userEventLedgerItem.setEnabled(false);
+				
+				
+			
+			if((metric.equals("Number of Calls")) || (metric.equals("Number of Subroutines")) || (metric.equals("Per Call Value"))){
+				valuePercentMenu.setEnabled(false);
+				unitsMenu.setEnabled(false);}
+			else if(percent){
+				valuePercentMenu.setEnabled(true);
+				unitsMenu.setEnabled(false);}
+			else if(!(isDefault || isTimeMetric)){
+				valuePercentMenu.setEnabled(true);
+				unitsMenu.setEnabled(false);
+			}
+			else{
+				valuePercentMenu.setEnabled(true);
+				unitsMenu.setEnabled(true);}
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "TDW04");
+			jRacy.systemError(e, null, "TDW04");
 		}
 		
 	}
@@ -556,6 +702,10 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 	{
 	}
 	
+	//******************************
+	//End - MenuListener code.
+	//******************************
+	
 	//Observer functions.
 	public void update(Observable o, Object arg)
 	{
@@ -572,6 +722,11 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 				//Just need to call a repaint on the ThreadDataWindowPanel.
 				threadDataWindowPanelRef.repaint();
 			}
+			else if(tmpString.equals("dataEvent"))
+			{
+				sortLocalData();
+				threadDataWindowPanelRef.repaint();
+			}
 			else if(tmpString.equals("subWindowCloseEvent"))
 			{
 				closeThisWindow();
@@ -579,71 +734,140 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "TDW05");
+			jRacy.systemError(e, null, "TDW05");
 		}
 	}
 	
+	public void refreshPanel(){
+		threadDataWindowPanelRef.repaint();
+	}
+	
 	//Updates the sorted lists after a change of sorting method takes place.
 	private void sortLocalData()
-	{
+	{	
 		try
 		{
-			if(sortByFunctionID)
+			if(sortByMappingID)
 			{
-				if(inclusive)
+				if(metric.equals("Inclusive"))
 				{
 					if(descendingOrder)
 						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "FIdDI");
 					else
 						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "FIdAI");
 				}
-				else
+				else if(metric.equals("Exclusive"))
 				{
 					if(descendingOrder)
 						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "FIdDE");
 					else
 						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "FIdAE");
 				}
+				else if(metric.equals("Number of Calls"))
+				{
+					if(descendingOrder)
+						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "FIdDNC");
+					else
+						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "FIdANC");
+				}
+				else if(metric.equals("Number of Subroutines"))
+				{
+					if(descendingOrder)
+						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "FIdDNS");
+					else
+						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "FIdANS");
+				}
+				else if(metric.equals("Per Call Value"))
+				{
+					if(descendingOrder)
+						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "FIdDUS");
+					else
+						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "FIdAUS");
+				}
 			}
 			else if(sortByName)
 			{
 				
-				if(inclusive)
+				if(metric.equals("Inclusive"))
 				{
 					if(descendingOrder)
 						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "NDI");
 					else
 						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "NAI");
 				}
-				else
+				else if(metric.equals("Exclusive"))
 				{
 					if(descendingOrder)
 						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "NDE");
 					else
 						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "NAE");
 				}
+				else if(metric.equals("Number of Calls"))
+				{
+					if(descendingOrder)
+						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "NDNC");
+					else
+						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "NANC");
+				}
+				else if(metric.equals("Number of Subroutines"))
+				{
+					if(descendingOrder)
+						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "NDNS");
+					else
+						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "NANS");
+				}
+				else if(metric.equals("Per Call Value"))
+				{
+					if(descendingOrder)
+						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "NDUS");
+					else
+						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "NAUS");
+				}
 			}
 			else if(sortByMillisecond)
 			{
-				if(inclusive)
+				
+				if(metric.equals("Inclusive"))
 				{
 					if(descendingOrder)
 						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "MDI");
 					else
 						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "MAI");
 				}
-				else
+				else if(metric.equals("Exclusive"))
 				{
 					if(descendingOrder)
 						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "MDE");
 					else
 						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "MAE");
 				}
+				else if(metric.equals("Number of Calls"))
+				{
+					if(descendingOrder)
+						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "MDNC");
+					else
+						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "MANC");
+				}
+				else if(metric.equals("Number of Subroutines"))
+				{
+					if(descendingOrder)
+						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "MDNS");
+					else
+						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "MANS");
+				}
+				else if(metric.equals("Per Call Value"))
+				{
+					System.out.println("Here1");
+					if(descendingOrder)
+						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "MDUS");
+					else
+						currentSMWThreadData = sMWData.getSMWThreadData(server, context, thread, "MAUS");
+				}
 			}
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "TDW06");
+			jRacy.systemError(e, null, "TDW06");
 		}
 	}
 	
@@ -654,9 +878,20 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 		return currentSMWThreadData;
 	}
 	
+	public LocalPrefWindow getLocalPrefWindow(){
+		return lPWindow;
+	}
+	
 	public boolean isInclusive()
 	{
-		return inclusive;
+		if(metric.equals("Inclusive"))
+			return true;
+		else
+			return false;
+	}
+	
+	public String getMetric(){
+		return metric;
 	}
 	
 	public boolean isPercent()
@@ -679,12 +914,90 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "TDW07");
+			jRacy.systemError(e, null, "TDW07");
 		}
 		
 		return tmpInt;
 	}
 	
+	public double getSliderMultiple()
+	{
+		String tmpString = null;
+		try
+		{
+			tmpString = (String) sliderMultiple.getSelectedItem();
+			return Double.parseDouble(tmpString);
+		}
+		catch(Exception e)
+		{
+			jRacy.systemError(e, null, "FDW08");
+		}
+		
+		return 0;
+	}
+	
+	private void displaySiders(boolean displaySliders)
+	{
+		if(displaySliders)
+		{
+			//Since the menu option is a toggle, the only component that needs to be
+			//removed is that scrollPane.  We then add back in with new parameters.
+			//This might not be required as it seems to adjust well if left in, but just
+			//to be sure.
+			contentPane.remove(threadDataWindowPanelScrollPane);
+			
+			gbc.fill = GridBagConstraints.NONE;
+			gbc.anchor = GridBagConstraints.EAST;
+			gbc.weightx = 0.10;
+			gbc.weighty = 0.01;
+			addCompItem(sliderMultipleLabel, gbc, 0, 0, 1, 1);
+			
+			gbc.fill = GridBagConstraints.NONE;
+			gbc.anchor = GridBagConstraints.WEST;
+			gbc.weightx = 0.10;
+			gbc.weighty = 0.01;
+			addCompItem(sliderMultiple, gbc, 1, 0, 1, 1);
+			
+			gbc.fill = GridBagConstraints.NONE;
+			gbc.anchor = GridBagConstraints.EAST;
+			gbc.weightx = 0.10;
+			gbc.weighty = 0.01;
+			addCompItem(barLengthLabel, gbc, 2, 0, 1, 1);
+			
+			gbc.fill = GridBagConstraints.HORIZONTAL;
+			gbc.anchor = GridBagConstraints.WEST;
+			gbc.weightx = 0.70;
+			gbc.weighty = 0.01;
+			addCompItem(barLengthSlider, gbc, 3, 0, 1, 1);
+			
+			gbc.fill = GridBagConstraints.BOTH;
+			gbc.anchor = GridBagConstraints.CENTER;
+			gbc.weightx = 0.95;
+			gbc.weighty = 0.98;
+			addCompItem(threadDataWindowPanelScrollPane, gbc, 0, 1, 4, 1);
+		}
+		else
+		{
+			contentPane.remove(sliderMultipleLabel);
+			contentPane.remove(sliderMultiple);
+			contentPane.remove(barLengthLabel);
+			contentPane.remove(barLengthSlider);
+			contentPane.remove(threadDataWindowPanelScrollPane);
+			
+			gbc.fill = GridBagConstraints.BOTH;
+			gbc.anchor = GridBagConstraints.CENTER;
+			gbc.weightx = 0.95;
+			gbc.weighty = 0.98;
+			addCompItem(threadDataWindowPanelScrollPane, gbc, 0, 0, 1, 1);
+		}
+		
+		//Now call validate so that these componant changes are displayed.
+		validate();
+	}
+	
+	public boolean getFormatNumbers(){
+		return formatNumbers;}
+	
 	//Helper functions.
 	private void addCompItem(Component c, GridBagConstraints gbc, int x, int y, int w, int h)
 	{
@@ -699,7 +1012,7 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "TDW08");
+			jRacy.systemError(e, null, "TDW09");
 		}
 	}
 	
@@ -721,35 +1034,40 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 			}
 			
 			setVisible(false);
-			jRacy.systemEvents.deleteObserver(this);
+			trial.getSystemEvents().deleteObserver(this);
 			dispose();
 		}
 		catch(Exception e)
 		{
-			jRacy.systemError(null, "TDW09");
+			jRacy.systemError(e, null, "TDW10");
 		}
 	}
 	
 	//******************************
 	//Instance data.
 	//******************************
-	private JMenu unitsMenu = new JMenu("Select Units");
+	private JMenu unitsMenu;
+	private JMenu valuePercentMenu;
+	private JMenuItem mappingGroupLedgerItem;
+	private JMenuItem userEventLedgerItem;
 	
 	private ButtonGroup sortGroup = null;
 	private ButtonGroup sortOrderGroup = null;
-	private ButtonGroup inclusiveExclusiveGroup = null;
+	private ButtonGroup metricGroup = null;
 	private ButtonGroup valuePercentGroup = null;
 	private ButtonGroup unitsGroup = null;
 	
 	private JRadioButtonMenuItem ascendingButton =  new JRadioButtonMenuItem("Ascending", false);
 	private JRadioButtonMenuItem descendingButton = new JRadioButtonMenuItem("Descending", true);
 	
-	private JRadioButtonMenuItem functionIDButton = new JRadioButtonMenuItem("function ID", false);
 	private JRadioButtonMenuItem nameButton = new JRadioButtonMenuItem("name", false);
-	private JRadioButtonMenuItem millisecondButton =  new JRadioButtonMenuItem("millisecond", true);
+	private JRadioButtonMenuItem millisecondButton =  new JRadioButtonMenuItem("Selected Metric", true);
 	
 	private JRadioButtonMenuItem inclusiveRadioButton =  new JRadioButtonMenuItem("Inclusive", false);
 	private JRadioButtonMenuItem exclusiveRadioButton = new JRadioButtonMenuItem("Exclusive", true);
+	private JRadioButtonMenuItem numOfCallsRadioButton =  new JRadioButtonMenuItem("Number of Calls", false);
+	private JRadioButtonMenuItem numOfSubRoutinesRadioButton = new JRadioButtonMenuItem("Number of Subroutines", false);
+	private JRadioButtonMenuItem userSecPerCallRadioButton = new JRadioButtonMenuItem("Per Call Value", false);
 	
 	private JRadioButtonMenuItem valueButton = new JRadioButtonMenuItem("Value", false);
 	private JRadioButtonMenuItem percentButton = new JRadioButtonMenuItem("Percent", true);
@@ -758,12 +1076,25 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
 	private JRadioButtonMenuItem millisecondsButton = new JRadioButtonMenuItem("Milliseconds", false);
 	private JRadioButtonMenuItem microsecondsButton = new JRadioButtonMenuItem("Microseconds", true);
 	
-	private JLabel barLengthLabel = new JLabel("Bar Mulitiple");
+	private JRadioButtonMenuItem displaySlidersButton;
+	private JRadioButtonMenuItem formatValuesButton;
 	
+	private JLabel sliderMultipleLabel = new JLabel("Slider Mulitiple");
+	private JComboBox sliderMultiple;
+	
+	private JLabel barLengthLabel = new JLabel("Bar Mulitiple");
 	private JSlider barLengthSlider = new JSlider(0, 40, 1);
 	
+	private Container contentPane = null;
+	private GridBagLayout gbl = null;
+	private GridBagConstraints gbc = null;
+	
+	private JScrollPane threadDataWindowPanelScrollPane;
+	
  	private ThreadDataWindowPanel threadDataWindowPanelRef = null;
+ 	private LocalPrefWindow lPWindow = null;
  	
+ 	private Trial trial = null;
  	private StaticMainWindowData sMWData = null;
  	
  	SMWThreadDataElement tmpSMWThreadDataElement = null;
@@ -772,16 +1103,18 @@ public class ThreadDataWindow extends JFrame implements ActionListener, MenuList
  	//Local data.
  	Vector currentSMWThreadData = null;
  	
- 	private boolean sortByFunctionID = false;
+ 	private boolean sortByMappingID = false;
 	private boolean sortByName = false;
 	private boolean sortByMillisecond = true;
 	
 	private boolean descendingOrder = true;
  	
- 	private boolean inclusive = false;
+ 	private String metric = "Exclusive";
  	private boolean percent = true;
  	private String unitsString = "milliseconds";
  	
+ 	private boolean formatNumbers = true;
+ 	
  	private int server = -1;
  	private int context = -1;
  	private int thread = -1;
