@@ -22,6 +22,14 @@
 #endif
 #include "pdbAll.h"
 
+/* defines */
+#ifdef TAU_WINDOWS
+#define TAU_DIR_CHARACTER '\\'
+#else
+#define TAU_DIR_CHARACTER '/'
+#endif /* TAU_WINDOWS */
+
+
 /* For selective instrumentation */
 extern int processInstrumentationRequests(char *fname);
 extern bool instrumentEntity(const string& function_name);
@@ -65,9 +73,22 @@ struct itemRef {
 
 static bool locCmp(const itemRef* r1, const itemRef* r2) {
   if ( r1->line == r2->line )
-    return r1->col < r2->col;
+  {
+    if (r1->col == r2->col)
+    { /* they're both equal */
+      if (r1->kind == BODY_BEGIN) return true; 
+      if (r2->kind == BODY_BEGIN) return false; 
+      return true; 
+    }
+    else
+    { /* col numbers are different */
+      return r1->col < r2->col;
+    }
+  }
   else
+  {
     return r1->line < r2->line;
+  }
 }
 
 static bool itemEqual(const itemRef* r1, const itemRef* r2) {
@@ -85,7 +106,7 @@ void getCXXReferences(vector<itemRef *>& itemvec, PDB& pdb, pdbFile *file) {
        rit!=routines.end(); ++rit) 
   {
     if ( (*rit)->location().file() == file && !(*rit)->isCompilerGenerated() && 
-	 ((*rit)->kind() != pdbItem::RO_EXT) && 
+	 (((*rit)->bodyBegin().line() != 0) && (*rit)->kind() != pdbItem::RO_EXT) && 
 	 (instrumentEntity((*rit)->fullName())) ) 
     {
 #ifdef DEBUG
@@ -156,9 +177,26 @@ void getCXXReferences(vector<itemRef *>& itemvec, PDB& pdb, pdbFile *file) {
 	}
 	else 
 	{ 
-	  // it is a member function add the CT macro
-          itemvec.push_back(new itemRef(*te, false, 
-	    (*te)->bodyBegin().line(), (*te)->bodyBegin().col()));
+#ifdef DEBUG
+	  cout <<"Before adding false to the member function, we must verify that it is not static"<<endl;
+#endif /* DEBUG */
+	  const pdbCRoutine *tr = (*te)->funcProtoInst();
+	  if (((tekind == pdbItem::TE_FUNC) || (tekind == pdbItem::TE_MEMFUNC))
+	      && ((tr) && (tr->isStatic())))
+
+	  { /* check to see if there's a prototype instantiation entry */
+	    /* it is indeed a static member function of a class template */
+	    /* DO NOT add CT(*this) to the static member function */
+	    
+            itemvec.push_back(new itemRef(*te, true, 
+	      (*te)->bodyBegin().line(), (*te)->bodyBegin().col()));
+	  }
+	  else
+	  {
+	    // it is a member function add the CT macro
+            itemvec.push_back(new itemRef(*te, false, 
+	      (*te)->bodyBegin().line(), (*te)->bodyBegin().col()));
+	  }
 	}
       }
       else 
@@ -262,6 +300,7 @@ void getFReferences(vector<itemRef *>& itemvec, PDB& pdb, pdbFile *file) {
   {
     pdbRoutine::locvec retlocations = (*rit)->returnLocations();
     pdbRoutine::locvec stoplocations = (*rit)->stopLocations();
+
     if ( (*rit)->location().file() == file &&  
          ((*rit)->kind() != pdbItem::RO_FSTFN) &&  
 	 ((*rit)->firstExecStmtLocation().file()) && 
@@ -297,7 +336,7 @@ void getFReferences(vector<itemRef *>& itemvec, PDB& pdb, pdbFile *file) {
           cout <<" Stop Locations : "<< (*slit)->line() << " col "
              << (*slit)->col() <<endl;
 #endif /* DEBUG */
-          itemvec.push_back(new itemRef(*rit, RETURN,
+          itemvec.push_back(new itemRef(*rit, EXIT,
                 (*slit)->line(), (*slit)->col()));
         }
     }
@@ -400,6 +439,7 @@ int instrumentCXXFile(PDB& pdb, pdbFile* f, string& outfile, string& group_name,
 #ifdef DEBUG
   cout << "Processing " << file << " ..." << endl;
 #endif 
+  memset(inbuf, INBUF_SIZE, 0); // reset to zero
 
 
   // initialize reference vector
@@ -515,6 +555,7 @@ int instrumentCXXFile(PDB& pdb, pdbFile* f, string& outfile, string& group_name,
 	
       } // else      
 
+      memset(inbuf, INBUF_SIZE, 0); // reset to zero
     } // while loop
 
   } // For loop
@@ -616,7 +657,7 @@ void processReturnExpression(ostream& ostr, string& ret_expression, itemRef *it,
 /* -------------------------------------------------------------------------- */
 /* -- Instrumentation routine for a C++ program ----------------------------- */
 /* -------------------------------------------------------------------------- */
-int instrumentCFile(PDB& pdb, pdbFile* f, string& outfile, string& group_name, string& header_file) 
+bool instrumentCFile(PDB& pdb, pdbFile* f, string& outfile, string& group_name, string& header_file) 
 { 
   string file(f->name());
   static char inbuf[INBUF_SIZE]; // to read the line
@@ -637,6 +678,7 @@ int instrumentCFile(PDB& pdb, pdbFile* f, string& outfile, string& group_name, s
 #endif
 
 
+  memset(inbuf, INBUF_SIZE, 0); // reset to zero
   // initialize reference vector
   vector<itemRef *> itemvec;
   getCReferences(itemvec, pdb, f);
@@ -882,6 +924,7 @@ int instrumentCFile(PDB& pdb, pdbFile* f, string& outfile, string& group_name, s
           } /* for it/lit */
         lit=it; 
       } /* else line no*/
+      memset(inbuf, INBUF_SIZE, 0); // reset to zero
     } /* while */
   } /* while lit != end */
   // For loop is over now flush out the remaining lines to the output file
@@ -892,23 +935,32 @@ int instrumentCFile(PDB& pdb, pdbFile* f, string& outfile, string& group_name, s
   // written everything. quit and debug!
   ostr.close();
 
-
+  return true;
 } /* End of instrumentCFile */ 
 
 
 #define WRITE_SPACE(os, c) { char ch = c; if (!((ch == ' ') || (ch == '\t'))) ch = ' '; \
 				os << ch; }
 
+#define WRITE_TAB(os, column) if (column < 7) os<<"\t";
+
+/* In Fortran programs, it is a bad idea to begin the first column with a C
+ * as it can be confused with a comment. So, we should check and see if the
+ * "call TAU_PROFILE..." statement starts on the first column and if so, we 
+ * should introduce a tab there. Hence, the need for the WRITE_TAB macro */
+
 
 /* -------------------------------------------------------------------------- */
 /* -- Get a list of instrumentation points for a C++ program ---------------- */
 /* -------------------------------------------------------------------------- */
-int instrumentFFile(PDB& pdb, pdbFile* f, string& outfile, string& group_name) 
+bool instrumentFFile(PDB& pdb, pdbFile* f, string& outfile, string& group_name) 
 { 
   string file(f->name());
   static char inbuf[INBUF_SIZE]; // to read the line
+  char *checkbuf=NULL; // Assign inbuf to checkbuf for return processing
   // open outfile for instrumented version of source file
   ofstream ostr(outfile.c_str());
+  int space, i, j, k;
   if (!ostr) {
     cerr << "Error: Cannot open '" << outfile << "'" << endl;
     return false;
@@ -923,6 +975,7 @@ int instrumentFFile(PDB& pdb, pdbFile* f, string& outfile, string& group_name)
   cout << "Processing " << file << " in instrumentFFile..." << endl;
 #endif
 
+  memset(inbuf, INBUF_SIZE, 0); // reset to zero
   // initialize reference vector
   vector<itemRef *> itemvec;
   getFReferences(itemvec, pdb, f);
@@ -939,7 +992,6 @@ int instrumentFFile(PDB& pdb, pdbFile* f, string& outfile, string& group_name)
     cout <<"S: "<< (*lit)->item->fullName() << " line "<< (*lit)->line << " col " << (*lit)->col << endl;
 #endif
     bool instrumented = false;
-    bool isProgram = false;
 
     while((instrumented == false) && (istr.getline(inbuf, INBUF_SIZE)) )
     {
@@ -1002,52 +1054,63 @@ int instrumentFFile(PDB& pdb, pdbFile* f, string& outfile, string& group_name)
 #ifdef DEBUG
 	    cout <<"Body Begin: Routine " <<(*it)->item->fullName()<<endl;
 #endif /* DEBUG */
-             	for(int i=0; i< ((*it)->col)-1; i++)
+             	for(i=0; i< ((*it)->col)-1; i++)
 		{ 
 #ifdef DEBUG
 	  	  cout << "Writing (1): "<<inbuf[i]<<endl;
 #endif /* DEBUG */
 	  	  WRITE_SPACE(ostr,inbuf[i])
 		}
+
+		WRITE_TAB(ostr,(*it)->col);
 		ostr <<"integer profiler(2)"<<endl;
 		/* spaces */
-     		for (int space = 0; space < (*it)->col-1 ; space++) 
+     		for (space = 0; space < (*it)->col-1 ; space++) 
 		  WRITE_SPACE(ostr, inbuf[space]) 
+
+		WRITE_TAB(ostr,(*it)->col);
 		ostr <<"save profiler"<<endl<<endl;
-     		for (int space = 0; space < (*it)->col-1 ; space++) 
+
+     		for (space = 0; space < (*it)->col-1 ; space++) 
 		  WRITE_SPACE(ostr, inbuf[space]) 
 		if (((pdbRoutine *)(*it)->item)->kind() == pdbItem::RO_FPROG)
 		{
 #ifdef DEBUG
 	  	  cout <<"Routine is main fortran program "<<endl;
 #endif /* DEBUG */
+		  WRITE_TAB(ostr,(*it)->col);
 		  ostr <<"call TAU_PROFILE_INIT()"<<endl;
-		  isProgram = true;
 		  /* put spaces on the next line */
      		  for (int space = 0; space < (*it)->col-1 ; space++) 
 		    WRITE_SPACE(ostr, inbuf[space]) 
+
+		  WRITE_TAB(ostr,(*it)->col);
 		  ostr <<"call TAU_PROFILE_TIMER(profiler,'" <<
 		    (*it)->item->fullName()<< "')"<<endl;
 		}
 		else { /* For all routines */
 		  if (strcmp(group_name.c_str(), "TAU_USER") != 0)
   		  { /* Write the following lines only when -DTAU_GROUP=string is defined */
+		    WRITE_TAB(ostr,(*it)->col);
 		    ostr<<"call TAU_PROFILE_TIMER(profiler,'" <<
     		       group_name.substr(10)<<">"<< (*it)->item->fullName()<< "')"<<endl;
 		  }
 		  else 
 		  { /* group_name is not defined, write the default fullName of the routine */
+		    WRITE_TAB(ostr,(*it)->col);
 		    ostr <<"call TAU_PROFILE_TIMER(profiler,'" <<
 		      (*it)->item->fullName()<< "')"<<endl;
 		  }
   		}
 		/* spaces */
-     		for (int space = 0; space < (*it)->col-1 ; space++) 
+     		for (space = 0; space < (*it)->col-1 ; space++) 
 		  WRITE_SPACE(ostr, inbuf[space]) 
+
+		WRITE_TAB(ostr,(*it)->col);
 		ostr <<"call TAU_PROFILE_START(profiler)"<<endl;
 
 		/* write the original statement */
-     		for (int k = 0; k < write_upto ; k++) 
+     		for (k = 0; k < write_upto ; k++) 
 		  ostr<< inbuf[k];
 
 		/* should we write the carriage return? */
@@ -1056,19 +1119,74 @@ int instrumentFFile(PDB& pdb, pdbFile* f, string& outfile, string& group_name)
 
 		instrumented = true;
 		break;
+	  case EXIT:
 	  case RETURN:
 #ifdef DEBUG
-	        cout <<"RETURN statement "<<endl;
+	        cout <<"RETURN/EXIT statement "<<endl;
 		cout <<"inbuf = "<<inbuf<<endl;
 	        cout <<"line ="<<(*it)->line<<" col = "<<(*it)->col<<endl;
 #endif /* DEBUG */
 		/* Check to see if it is not a comment and has a "if" in the string */
+                checkbuf = new char[strlen(inbuf)]; 
+                if (checkbuf == (char *) NULL) 
+                {
+                  perror("ERROR: new returns NULL while creating checkbuf");
+                  exit(1);
+                }
+                for  (i = 0; i < (*it)->col; i++)
+                {
+	           checkbuf[i] = inbuf[i];
+                }
+                checkbuf[i] = '\0'; 
       		if ((!((inbuf[0] == 'c') || (inbuf[0] == 'C') || (inbuf[0] == '!'))) && 
-	  	    (strstr(inbuf,"if") != NULL))
+	  	    (strstr(checkbuf,"if") != NULL))
 		  is_if_stmt = true;
                 else
 		  is_if_stmt = false;
 
+      		if ((is_if_stmt == false) && (!((inbuf[0] == 'c') || (inbuf[0] == 'C') || (inbuf[0] == '!'))) && 
+	  	    (strstr(checkbuf,"IF") != NULL))
+                { /* only if the earlier clause was false will this be executed */
+		  is_if_stmt = true;
+                }
+
+		/* Check to see if return is in a continuation line */
+		/* such as :
+		 *     if(  (ii/=jj  .or. kk<=0)  .and. &
+		 *           & (kcheck==0  .or. ii/=lsav+1 .or. kk>0) ) return
+		 */
+
+		if (is_if_stmt == false) 
+		{
+#ifdef DEBUG
+		  cout <<"col = "<<(*it)->col <<endl;
+#endif /* DEBUG */
+		  for(int c = ((*it)->col)-2; c > 0; c--)
+		  {
+#ifdef DEBUG
+		    cout <<"c = "<<c<<"inbuf[c] = "<<inbuf[c]<<endl;
+#endif /* DEBUG */
+	            if (inbuf[c] == ' ' || inbuf[c] == '\t') continue;
+		    if (inbuf[c] == ')' ) 
+		    { /* return is in a continuation line - has " ) return" */
+#ifdef DEBUG
+		       cout <<"inbuf[c] = "<<inbuf[c]<<endl;
+#endif /* DEBUG */
+		       is_if_stmt = true;
+		       break;
+		    }
+		    else
+		    {
+		       is_if_stmt = false;
+		       break;
+		    }
+		  }
+		}
+		/* Here, either is_if_stmt is true or it is a plain return*/
+#ifdef DEBUG
+	cout <<"if_stmt = "<<is_if_stmt<<endl;
+#endif /* DEBUG */
+
 	        if (lit == it)
 		{ /* Has body begin already written the beginning of the statement? */
 		  /* No. Write it (since it is same as lit) */
@@ -1087,14 +1205,23 @@ int instrumentFFile(PDB& pdb, pdbFile* f, string& outfile, string& group_name)
 		  ostr << "          ";
 		}
 	
-		ostr <<"call TAU_PROFILE_STOP(profiler)"<<endl;
+		WRITE_TAB(ostr,(*it)->col);
+		/* before writing stop/exit examine the kind */
+		if ((*it)->kind == EXIT)
+		{ /* Turn off the timers. This is similar to abort/exit in C */
+		  ostr <<"call TAU_PROFILE_EXIT('exit')"<<endl;
+		}
+		else
+		{
+		  ostr <<"call TAU_PROFILE_STOP(profiler)"<<endl;
+		}
 
-     		for (int space = 0; space < (*it)->col-1 ; space++) 
+     		for (space = 0; space < (*it)->col-1 ; space++) 
 		  WRITE_SPACE(ostr, inbuf[space])
 
 		instrumented = true;
 	
-        	for(int j= ((*it)->col)-1; j <write_upto; j++)
+        	for(j= ((*it)->col)-1; j <write_upto; j++)
 		{ 
 #ifdef DEBUG
 	  	  cout <<"Writing(2): "<<inbuf[j]<<endl;
@@ -1119,6 +1246,7 @@ int instrumentFFile(PDB& pdb, pdbFile* f, string& outfile, string& group_name)
         } /* for it/lit */
         lit = it;		
       } /* reached line */
+      memset(inbuf, INBUF_SIZE, 0); // reset to zero
     } /* while */
   } /* while lit!= end */
   // For loop is over now flush out the remaining lines to the output file
@@ -1128,6 +1256,7 @@ int instrumentFFile(PDB& pdb, pdbFile* f, string& outfile, string& group_name)
   }
   // written everything. quit and debug!
   ostr.close();
+  return true; /* end of instrumentFFile */
 }
 
 /* -------------------------------------------------------------------------- */
@@ -1171,6 +1300,51 @@ void setGroupName(PDB& p, string& group_name)
 
 }
 
+/* -------------------------------------------------------------------------- */
+/* -- Fuzzy Match. Allows us to match files that don't quite match properly, 
+ * but infact refer to the same file. For e.g., /home/pkg/foo.cpp and ./foo.cpp
+ * or foo.cpp and ./foo.cpp. This routine allows us to match such files! 
+ * -------------------------------------------------------------------------- */
+bool fuzzyMatch(const string& a, const string& b)
+{ /* This function allows us to match string like ./foo.cpp with
+     /home/pkg/foo.cpp */
+  if (a == b)
+  { /* the two files do match */
+#ifdef DEBUG
+    cout <<"fuzzyMatch returns true for "<<a<<" and "<<b<<endl;
+#endif /* DEBUG */
+    return true;
+  }
+  else 
+  { /* fuzzy match */
+    /* Extract the name without the / character */
+    int loca = a.find_last_of(TAU_DIR_CHARACTER);
+    int locb = b.find_last_of(TAU_DIR_CHARACTER);
+
+    /* truncate the strings */
+    string trunca(a,loca+1);
+    string truncb(b,locb+1);
+    /*
+    cout <<"trunca = "<<trunca<<endl;
+    cout <<"truncb = "<<truncb<<endl;
+    */
+    if (trunca == truncb) 
+    {
+#ifdef DEBUG
+      cout <<"fuzzyMatch returns true for "<<a<<" and "<<b<<endl;
+#endif /* DEBUG */
+      return true;
+    }
+    else
+    {
+#ifdef DEBUG
+      cout <<"fuzzyMatch returns false for "<<a<<" and "<<b<<endl;
+#endif /* DEBUG */
+      return false;
+    }
+  }
+}
+
 /* -------------------------------------------------------------------------- */
 /* -- Instrument the program using C, C++ or F90 instrumentation routines --- */
 /* -------------------------------------------------------------------------- */
@@ -1187,7 +1361,6 @@ int main(int argc, char **argv)
     return 1;
   }
   PDB p(argv[1]); if ( !p ) return 1;
-  char *gr_name;
   setGroupName(p, group_name);
   bool outFileNameSpecified = false;
   int i; 
@@ -1304,14 +1477,19 @@ int main(int argc, char **argv)
   }
 
 
+  bool instrumentThisFile;
+  bool fuzzyMatchResult;
+  bool fileInstrumented = false;
   for (PDB::filevec::const_iterator it=p.getFileVec().begin();
        it!=p.getFileVec().end(); ++it) 
   {
-     bool instrumentThisFile = false;
-     if (((*it)->name() == string(filename)) && 
+     /* reset this variable at the beginning of the loop */
+     instrumentThisFile = false;
+     if ((fuzzyMatchResult = fuzzyMatch((*it)->name(), string(filename))) && 
          (instrumentThisFile = processFileForInstrumentation(string(filename))))
      { /* should we instrument this file? Yes */
        PDB::lang_t l = p.language();
+       fileInstrumented = true; /* We will instrument this file */
 
 #ifdef DEBUG
        cout <<" *** FILE *** "<< (*it)->name()<<endl;
@@ -1346,17 +1524,31 @@ int main(int argc, char **argv)
      } /* don't instrument this file. Should we copy in to out? */
      else
      { 
-       if (((*it)->name() == string(filename)) &&
-         (instrumentThisFile == false))
+       if ((fuzzyMatchResult == true) && (instrumentThisFile == false))
        { /* we should copy the file to outFile */
          ifstream ifs(filename);
          ofstream ofs(outFileName.c_str());
          /* copy ifs to ofs */
          if (ifs.is_open() && ofs.is_open())
            ofs << ifs.rdbuf(); /* COPY */ 
+	 instrumentThisFile = true; /* sort of like instrumentation,
+		more like processed this file. Later we need to know
+		if no files were processed */
        }
      }
   }
+  if (fileInstrumented == false)
+  { /* no files were processed */
+#ifdef DEBUG
+    cout <<"No files were processed"<<endl;
+#endif /* DEBUG */
+    /* We should copy this file to outfile */
+    ifstream ifsc(filename);
+    ofstream ofsc(outFileName.c_str());
+    /* copy ifsc to ofsc */
+    if (ifsc.is_open() && ofsc.is_open())
+       ofsc << ifsc.rdbuf(); /* COPY */ 
+  }
 
   /* start with routines */
 /* 
@@ -1381,8 +1573,8 @@ int main(int argc, char **argv)
   
 /***************************************************************************
  * $RCSfile: tau_instrumentor.cpp,v $   $Author: sameer $
- * $Revision: 1.45 $   $Date: 2003/07/15 17:57:49 $
- * VERSION_ID: $Id: tau_instrumentor.cpp,v 1.45 2003/07/15 17:57:49 sameer Exp $
+ * $Revision: 1.62 $   $Date: 2004/06/21 17:13:09 $
+ * VERSION_ID: $Id: tau_instrumentor.cpp,v 1.62 2004/06/21 17:13:09 sameer Exp $
  ***************************************************************************/
 
 
